@@ -1,8 +1,10 @@
+@file:Suppress("InvalidPackageDeclaration")
 package com.unciv.build
 
 import com.badlogic.gdx.graphics.Texture
 import com.badlogic.gdx.tools.texturepacker.TexturePacker
 import com.badlogic.gdx.utils.Json
+import com.unciv.build.AndroidImagePacker.packImages
 import java.io.File
 import java.nio.file.Files
 import java.nio.file.attribute.BasicFileAttributes
@@ -62,14 +64,15 @@ object AndroidImagePacker {
 
         // pack for mods
         val modDirectory = File("mods")
-        if (modDirectory.exists()) {
-            for (mod in modDirectory.listFiles()!!) {
-                if (!mod.isHidden) {
-                    try {
-                        packImagesPerMod(mod.path, mod.path, defaultSettings)
-                    } catch (ex: Throwable) {
-                    }
-                }
+        if (!modDirectory.exists())
+            return
+        for (mod in modDirectory.listFiles()!!) {
+            if (mod.isHidden)
+                continue
+            try {
+                packImagesPerMod(mod.path, mod.path, defaultSettings)
+            } catch (ex: Throwable) {
+                ex.printStackTrace()
             }
         }
     }
@@ -82,7 +85,6 @@ object AndroidImagePacker {
             atlasList += packFileName
             packImagesIfOutdated(defaultSettings, file, output, packFileName)
         }
-        atlasList.remove("game")
         val listFile = File("$output${File.separator}Atlases.json")
         if (atlasList.isEmpty()) listFile.delete()
         else listFile.writeText(atlasList.sorted().joinToString(",","[","]"))
@@ -103,8 +105,8 @@ object AndroidImagePacker {
             if (File(input).listTree().none {
                 val attr: BasicFileAttributes = Files.readAttributes(it.toPath(), BasicFileAttributes::class.java)
                 val createdAt: Long = attr.creationTime().toMillis()
-                it.extension in listOf("png", "jpg", "jpeg")
-                        && (it.lastModified() > atlasModTime || createdAt > atlasModTime)
+                (it.extension in listOf("png", "jpg", "jpeg") || it.name == "TexturePacker.settings")
+                    && (it.lastModified() > atlasModTime || createdAt > atlasModTime)
             }) return
         }
 
@@ -129,4 +131,3 @@ object AndroidImagePacker {
         }
     }
 }
-
