@@ -3,8 +3,8 @@ package com.unciv
 object Constants {
     const val settler = "Settler"
     const val eraSpecificUnit = "Era Starting Unit"
-    const val spreadReligion = "Spread Religion"
-    const val removeHeresy = "Remove Foreign religions from your own cities"
+    val all = setOf("All", "all")
+    const val NO_ID = -1
 
     const val english = "English"
 
@@ -16,6 +16,9 @@ object Constants {
     /** The "Coastal" terrain _filter_ */
     const val coastal = "Coastal"
 
+    /** Used as filter and the name of the pseudo-TerrainFeature defining river Stats */
+    const val river = "River"
+
     const val mountain = "Mountain"
     const val hill = "Hill"
     const val plains = "Plains"
@@ -68,6 +71,7 @@ object Constants {
 
     const val OK = "OK"
     const val close = "Close"
+    const val cancel = "Cancel"
     const val yes = "Yes"
     const val no = "No"
     const val loading = "Loading..."
@@ -79,9 +83,6 @@ object Constants {
     const val embarked = "Embarked"
     const val wounded = "Wounded"
 
-
-    const val rising = "Rising"
-    const val lowering = "Lowering"
     const val remove = "Remove "
     const val repair = "Repair"
 
@@ -106,4 +107,10 @@ object Constants {
 
     const val defaultFontSize = 18
     const val headingFontSize = 24
+
+    /** URL to the root of the Unciv repository, including trailing slash */
+    // Note: Should the project move, this covers external links, but not comments e.g. mentioning issues
+    const val uncivRepoURL = "https://github.com/yairm210/Unciv/"
+    /** URL to the wiki, including trailing slash */
+    const val wikiURL = "https://yairm210.github.io/Unciv/"
 }
