@@ -37,9 +37,17 @@ object BackwardCompatibility {
         removeTechAndPolicies()
     }
 
+    fun GameInfo.migrateGreatGeneralPools() {
+        for (civ in civilizations) civ.greatPeople.run {
+            if (pointsForNextGreatGeneral >= pointsForNextGreatGeneralCounter["Great General"]) {
+                pointsForNextGreatGeneralCounter["Great General"] = pointsForNextGreatGeneral
+            } else pointsForNextGreatGeneral = pointsForNextGreatGeneralCounter["Great General"]
+        }
+    }
+
     private fun GameInfo.removeUnitsAndPromotions() {
         for (tile in tileMap.values) {
-            for (unit in tile.getUnits()) {
+            for (unit in tile.getUnits().toList()) {
                 if (!ruleset.units.containsKey(unit.name)) tile.removeUnit(unit)
 
                 for (promotion in unit.promotions.promotions.toList())
@@ -56,7 +64,7 @@ object BackwardCompatibility {
                     tile.roadStatus = RoadStatus.None
                     tile.roadIsPillaged = false
                 }
-                if (tile.improvementIsPillaged){
+                if (tile.improvementIsPillaged) {
                     tile.improvement = null
                     tile.improvementIsPillaged = false
                 }
@@ -66,9 +74,9 @@ object BackwardCompatibility {
     private fun GameInfo.handleMissingReferencesForEachCity() {
         for (city in civilizations.asSequence().flatMap { it.cities.asSequence() }) {
 
-            for (building in city.cityConstructions.getBuiltBuildings()) {
-                if (!ruleset.buildings.containsKey(building.name))
-                    city.cityConstructions.removeBuilding(building.name)
+            for (building in city.cityConstructions.builtBuildings.toList()) {
+                if (!ruleset.buildings.containsKey(building))
+                    city.cityConstructions.builtBuildings.remove(building)
             }
 
             fun isInvalidConstruction(construction: String) =
@@ -192,4 +200,12 @@ object BackwardCompatibility {
         }
         historyStartTurn = turns
     }
+
+    fun GameInfo.ensureUnitIds(){
+        if (lastUnitId == 0) lastUnitId = tileMap.values.asSequence()
+            .flatMap { it.getUnits() }.maxOfOrNull { it.id } ?: 0
+        for (unit in tileMap.values.flatMap { it.getUnits() }) {
+            if (unit.id == 0) unit.id = ++lastUnitId
+        }
+    }
 }
