@@ -14,12 +14,11 @@ import com.unciv.logic.map.tile.Tile
 import com.unciv.models.metadata.GameParameters
 import com.unciv.models.metadata.GameSetupInfo
 import com.unciv.models.metadata.Player
-import com.unciv.models.ruleset.ModOptionsConstants
 import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.RulesetCache
 import com.unciv.models.ruleset.unique.StateForConditionals
-import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.ruleset.unique.UniqueTriggerActivation
+import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.ruleset.unit.BaseUnit
 import com.unciv.models.stats.Stats
 import com.unciv.models.translations.equalsPlaceholderText
@@ -166,39 +165,41 @@ object GameStarter {
     }
 
     private fun addCivTechs(gameInfo: GameInfo, ruleset: Ruleset, gameSetupInfo: GameSetupInfo) {
-        for (civInfo in gameInfo.civilizations.filter { !it.isBarbarian() }) {
+        fun Civilization.addTechSilently(name: String) {
+            // check if the technology is in the ruleset and not already researched
+            if (!ruleset.technologies.containsKey(name)) return
+            if (tech.isResearched(name)) return
+            tech.addTechnology(name, false)
+        }
 
-        for(tech in ruleset.technologies.values.filter { it.hasUnique(UniqueType.StartingTech) })
-            {
-                civInfo.tech.addTechnology(tech.name, false)
-            }
+        for (civInfo in gameInfo.civilizations) {
+            if (civInfo.isBarbarian()) continue
+
+            for (tech in ruleset.technologies.values.filter { it.hasUnique(UniqueType.StartingTech) })
+                civInfo.addTechSilently(tech.name)
 
             if (!civInfo.isHuman())
                 for (tech in gameInfo.getDifficulty().aiFreeTechs)
-                    civInfo.tech.addTechnology(tech, false)
+                    civInfo.addTechSilently(tech)
 
             // generic start with technology unique
-            for (unique in civInfo.getMatchingUniques(UniqueType.StartsWithTech)) {
-                // get the parameter from the unique
-                val techName = unique.params[0]
-
-                // check if the technology is in the ruleset and not already researched
-                if (ruleset.technologies.containsKey(techName) && !civInfo.tech.isResearched(techName))
-                    civInfo.tech.addTechnology(techName, false)
-            }
+            for (unique in civInfo.getMatchingUniques(UniqueType.StartsWithTech))
+                civInfo.addTechSilently(unique.params[0])
 
             // add all techs to spectators
             if (civInfo.isSpectator())
                 for (tech in ruleset.technologies.values)
-                    if (!civInfo.tech.isResearched(tech.name))
-                        civInfo.tech.addTechnology(tech.name, false)
+                    civInfo.addTechSilently(tech.name)
 
-            for (tech in ruleset.technologies.values
-                    .filter { ruleset.eras[it.era()]!!.eraNumber < ruleset.eras[gameSetupInfo.gameParameters.startingEra]!!.eraNumber })
-                if (!civInfo.tech.isResearched(tech.name))
-                    civInfo.tech.addTechnology(tech.name, false)
+            // Add techs for advanced starting era
+            val startingEraNumber = ruleset.eras[gameSetupInfo.gameParameters.startingEra]!!.eraNumber
+            for (tech in ruleset.technologies.values) {
+                if (ruleset.eras[tech.era()]!!.eraNumber >= startingEraNumber) continue
+                civInfo.addTechSilently(tech.name)
+            }
 
-            civInfo.popupAlerts.clear() // Since adding technologies generates popups...
+            // Since adding technologies generates popups (addTechnology parameter showNotification only suppresses Notifications)
+            civInfo.popupAlerts.clear()
         }
     }
 
@@ -350,9 +351,7 @@ object GameStarter {
         for (player in chosenPlayers) {
             val civ = Civilization(player.chosenCiv)
             when (player.chosenCiv) {
-                Constants.spectator ->
-                    civ.playerType = player.playerType
-                in usedMajorCivs -> {
+                in usedMajorCivs, Constants.spectator -> {
                     civ.playerType = player.playerType
                     civ.playerId = player.playerId
                 }
@@ -416,7 +415,7 @@ object GameStarter {
         ruleset: Ruleset
     ) {
         val startingEra = gameInfo.gameParameters.startingEra
-        val settlerLikeUnits = ruleset.units.filter { it.value.hasUnique(UniqueType.FoundCity) }
+        val settlerLikeUnits = ruleset.units.filter { it.value.isCityFounder() }
 
         for (civ in gameInfo.civilizations.filter { !it.isBarbarian() && !it.isSpectator() }) {
             val startingLocation = startingLocations[civ]!!
@@ -429,14 +428,14 @@ object GameStarter {
             //Trigger any global or nation uniques that should triggered.
             //We may need the starting location for some uniques, which is why we're doing it now
             val startingTriggers = (ruleset.globalUniques.uniqueObjects + civ.nation.uniqueObjects)
-            for (unique in startingTriggers.filter { !it.hasTriggerConditional() })
-                if(unique.isTriggerable)
-                    UniqueTriggerActivation.triggerCivwideUnique(unique, civ, tile = startingLocation)
+            for (unique in startingTriggers.filter { !it.hasTriggerConditional() && it.conditionalsApply(civ) })
+                UniqueTriggerActivation.triggerUnique(unique, civ, tile = startingLocation)
         }
     }
 
     private fun getStartingUnitsForEraAndDifficulty(civ: Civilization, gameInfo: GameInfo, ruleset: Ruleset, startingEra: String): MutableList<String> {
-        val startingUnits = ruleset.eras[startingEra]!!.getStartingUnits(ruleset)
+        val startingUnits = ruleset.eras[startingEra]?.getStartingUnits(ruleset)
+            ?: throw Exception("Era $startingEra does not exist in the ruleset!")
 
         // Add extra units granted by difficulty
         startingUnits.addAll(when {
@@ -454,7 +453,7 @@ object GameStarter {
         ruleset: Ruleset,
         eraUnitReplacement: String,
         settlerLikeUnits: Map<String, BaseUnit>
-    ): String? {
+    ): BaseUnit? {
         var unit = unitParam // We want to change it and this is the easiest way to do so
         if (unit == Constants.eraSpecificUnit) unit = eraUnitReplacement
         if (unit == Constants.settler && Constants.settler !in ruleset.units) {
@@ -464,7 +463,7 @@ object GameStarter {
                         && it.value.isCivilian()
                 }
             if (buildableSettlerLikeUnits.isEmpty()) return null // No settlers in this mod
-            return civ.getEquivalentUnit(buildableSettlerLikeUnits.keys.random()).name
+            return civ.getEquivalentUnit(buildableSettlerLikeUnits.keys.random())
         }
         if (unit == "Worker" && "Worker" !in ruleset.units) {
             val buildableWorkerLikeUnits = ruleset.units.filter {
@@ -472,9 +471,9 @@ object GameStarter {
                     it.value.isBuildable(civ) && it.value.isCivilian()
             }
             if (buildableWorkerLikeUnits.isEmpty()) return null // No workers in this mod
-            return civ.getEquivalentUnit(buildableWorkerLikeUnits.keys.random()).name
+            return civ.getEquivalentUnit(buildableWorkerLikeUnits.keys.random())
         }
-        return civ.getEquivalentUnit(unit).name
+        return civ.getEquivalentUnit(unit)
     }
 
     private fun adjustStartingUnitsForCityStatesAndOneCityChallenge(
@@ -484,7 +483,7 @@ object GameStarter {
         settlerLikeUnits: Map<String, BaseUnit>
     ) {
         // Adjust starting units for city states
-        if (civ.isCityState() && !gameInfo.ruleset.modOptions.uniques.contains(ModOptionsConstants.allowCityStatesSpawnUnits)) {
+        if (civ.isCityState() && !gameInfo.ruleset.modOptions.hasUnique(UniqueType.AllowCityStatesSpawnUnits)) {
             val startingSettlers = startingUnits.filter { settlerLikeUnits.contains(it) }
 
             startingUnits.clear()
@@ -539,7 +538,7 @@ object GameStarter {
     ): HashMap<Civilization, Tile> {
 
         val civsOrderedByAvailableLocations = getCivsOrderedByAvailableLocations(civs, tileMap)
- 
+
         for (minimumDistanceBetweenStartingLocations in tileMap.tileMatrix.size / 6 downTo 0) {
             val freeTiles = getFreeTiles(tileMap, landTilesInBigEnoughGroup, minimumDistanceBetweenStartingLocations)
 
@@ -581,12 +580,12 @@ object GameStarter {
     ): HashMap<Civilization, Tile>? {
         val startingLocations = HashMap<Civilization, Tile>()
         for (civ in civsOrderedByAvailableLocations) {
-            
+
             val startingLocation = getCivStartingLocation(civ, tileMap, freeTiles, startScores)
             startingLocation ?: break
-            
+
             startingLocations[civ] = startingLocation
-            
+
             val distanceToNext = minimumDistanceBetweenStartingLocations /
                 (if (civ.isCityState()) 2 else 1) // We allow city states to squeeze in tighter
             freeTiles.removeAll(tileMap.getTilesInDistance(startingLocation.position, distanceToNext)
