@@ -10,9 +10,11 @@ import com.unciv.logic.map.tile.Tile
 import com.unciv.models.ruleset.Building
 import com.unciv.models.ruleset.INonPerpetualConstruction
 import com.unciv.models.ruleset.Victory
+import com.unciv.models.ruleset.nation.PersonalityValue
 import com.unciv.models.ruleset.tile.ResourceType
 import com.unciv.models.ruleset.tile.TileImprovement
 import com.unciv.models.ruleset.unique.LocalUniqueCache
+import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.ruleset.unit.BaseUnit
 import com.unciv.models.stats.Stat
@@ -21,14 +23,14 @@ import com.unciv.ui.screens.victoryscreen.RankingType
 
 object Automation {
 
-    fun rankTileForCityWork(tile: Tile, city: City, cityStats: Stats, localUniqueCache: LocalUniqueCache = LocalUniqueCache(false)): Float {
+    fun rankTileForCityWork(tile: Tile, city: City, localUniqueCache: LocalUniqueCache = LocalUniqueCache(false)): Float {
         val stats = tile.stats.getTileStats(city, city.civ, localUniqueCache)
-        return rankStatsForCityWork(stats, city, cityStats, false, localUniqueCache)
+        return rankStatsForCityWork(stats, city, false, localUniqueCache)
     }
 
-    fun rankSpecialist(specialist: String, city: City, cityStats: Stats, localUniqueCache: LocalUniqueCache): Float {
+    fun rankSpecialist(specialist: String, city: City, localUniqueCache: LocalUniqueCache): Float {
         val stats = city.cityStats.getStatsOfSpecialist(specialist, localUniqueCache)
-        var rank = rankStatsForCityWork(stats, city, cityStats, true, localUniqueCache)
+        var rank = rankStatsForCityWork(stats, city, true, localUniqueCache)
         // derive GPP score
         var gpp = 0f
         if (city.getRuleset().specialists.containsKey(specialist)) { // To solve problems in total remake mods
@@ -40,13 +42,14 @@ object Automation {
         return rank
     }
 
-    val zeroFoodFocuses = setOf(CityFocus.CultureFocus, CityFocus.FaithFocus, CityFocus.GoldFocus,
-        CityFocus.HappinessFocus, CityFocus.ProductionFocus, CityFocus.ScienceFocus)
-    private fun rankStatsForCityWork(stats: Stats, city: City, cityStats: Stats, specialist: Boolean, localUniqueCache: LocalUniqueCache): Float {
-        val cityAIFocus = city.cityAIFocus
-        val yieldStats = stats.clone()
 
-        if (specialist) {
+    fun rankStatsForCityWork(stats: Stats, city: City, areWeRankingSpecialist: Boolean, localUniqueCache: LocalUniqueCache): Float {
+        val cityAIFocus = city.getCityFocus()
+        val yieldStats = stats.clone()
+        val civPersonality = city.civ.getPersonality()
+        val cityStatsObj = city.cityStats
+
+        if (areWeRankingSpecialist) {
             // If you have the Food Bonus, count as 1 extra food production (base is 2food)
             for (unique in localUniqueCache.forCityGetMatchingUniques(city, UniqueType.FoodConsumptionBySpecialists))
                 if (city.matchesFilter(unique.params[1]))
@@ -55,16 +58,21 @@ object Automation {
             for (unique in localUniqueCache.forCityGetMatchingUniques(city, UniqueType.UnhappinessFromPopulationTypePercentageChange))
                 if (city.matchesFilter(unique.params[2]) && unique.params[1] == "Specialists")
                     yieldStats.happiness -= (unique.params[0].toFloat() / 100f)  // relative val is negative, make positive
-            if (city.civ.getHappiness() < 0) yieldStats.happiness *= 2  // double weight for unhappy civilization
         }
 
-        val surplusFood = cityStats[Stat.Food]
+        val surplusFood = city.cityStats.currentCityStats[Stat.Food]
+        // If current Production converts Food into Production, then calculate increased Production Yield
+        if (cityStatsObj.canConvertFoodToProduction(surplusFood, city.cityConstructions.getCurrentConstruction())) {
+            // calculate delta increase of food->prod. This isn't linear
+            yieldStats.production += cityStatsObj.getProductionFromExcessiveFood(surplusFood+yieldStats.food) - cityStatsObj.getProductionFromExcessiveFood(surplusFood)
+            yieldStats.food = 0f  // all food goes to 0
+        }
         // Apply base weights
         yieldStats.applyRankingWeights()
 
         if (surplusFood > 0 && city.avoidGrowth) {
             yieldStats.food = 0f // don't need more food!
-        } else if (cityAIFocus in zeroFoodFocuses) {
+        } else if (cityAIFocus in CityFocus.zeroFoodFocuses()) {
             // Focus on non-food/growth
             if (surplusFood < 0)
                 yieldStats.food *= 8 // Starving, need Food, get to 0
@@ -76,8 +84,6 @@ object Automation {
                 yieldStats.food *= 8
             else if (city.population.population < 5)
                 yieldStats.food *= 3
-            else if (cityAIFocus == CityFocus.FoodFocus)
-                yieldStats.food *= 2
         }
 
         if (city.population.population < 5) {
@@ -88,14 +94,18 @@ object Automation {
             if (city.civ.gold < 0 && city.civ.stats.statsForNextTurn.gold <= 0)
                 yieldStats.gold *= 2 // We have a global problem
 
-            if (city.tiles.size < 12 || city.civ.wantsToFocusOn(Victory.Focus.Culture))
+            if (city.tiles.size < 12)
                 yieldStats.culture *= 2
 
-            if (city.civ.getHappiness() < 0 && !specialist) // since this doesn't get updated, may overshoot
+            if (city.civ.getHappiness() < 0)
                 yieldStats.happiness *= 2
+        }
 
-            if (city.civ.wantsToFocusOn(Victory.Focus.Science))
-                yieldStats.science *= 2
+        for (stat in Stat.values()) {
+            if (city.civ.wantsToFocusOn(stat))
+                yieldStats[stat] *= 2f
+
+            yieldStats[stat] *= civPersonality.scaledFocus(PersonalityValue[stat])
         }
 
         // Apply City focus
@@ -185,7 +195,7 @@ object Automation {
                 .filter { it.isRanged() }
                 .maxByOrNull { it.cost }!!
         }
-        else if (isMissingNavalUnitsForCityDefence && militaryUnits.any { isNavalMeleeUnit(it) }){
+        else if (isMissingNavalUnitsForCityDefence && militaryUnits.any { isNavalMeleeUnit(it) }) {
             chosenUnit = militaryUnits
                 .filter { isNavalMeleeUnit(it) }
                 .maxBy { it.cost }
@@ -229,7 +239,7 @@ object Automation {
 
         // If we have vision of our entire starting continent (ish) we are not afraid
         civInfo.gameInfo.tileMap.assignContinents(TileMap.AssignContinentsMode.Ensure)
-        val startingContinent = civInfo.getCapital(true)!!.getCenterTile().getContinent()
+        val startingContinent = civInfo.getCapital()!!.getCenterTile().getContinent()
         val startingContinentSize = civInfo.gameInfo.tileMap.continentSizes[startingContinent]
         if (startingContinentSize != null && startingContinentSize < civInfo.viewableTiles.size * multiplier)
             return false
@@ -247,7 +257,7 @@ object Automation {
         construction: INonPerpetualConstruction
     ): Boolean {
         return allowCreateImprovementBuildings(civInfo, city, construction)
-                && allowSpendingResource(civInfo, construction)
+            && allowSpendingResource(civInfo, construction, city)
     }
 
     @Suppress("MemberVisibilityCanBePrivate")
@@ -260,7 +270,7 @@ object Automation {
     ): Boolean {
         if (construction !is Building) return true
         if (!construction.hasCreateOneImprovementUnique()) return true  // redundant but faster???
-        val improvement = construction.getImprovementToCreate(city.getRuleset()) ?: return true
+        val improvement = construction.getImprovementToCreate(city.getRuleset(), civInfo) ?: return true
         return city.getTiles().any {
             it.improvementFunctions.canBuildImprovement(improvement, civInfo)
         }
@@ -268,7 +278,7 @@ object Automation {
 
     /** Determines whether the AI should be willing to spend strategic resources to build
      *  [construction] for [civInfo], assumes that we are actually able to do so. */
-    fun allowSpendingResource(civInfo: Civilization, construction: INonPerpetualConstruction): Boolean {
+    fun allowSpendingResource(civInfo: Civilization, construction: INonPerpetualConstruction, cityInfo: City? = null): Boolean {
         // City states do whatever they want
         if (civInfo.isCityState())
             return true
@@ -277,7 +287,9 @@ object Automation {
         if (construction.name in civInfo.gameInfo.spaceResources)
             return true
 
-        val requiredResources = construction.getResourceRequirementsPerTurn()
+        val requiredResources = if (construction is BaseUnit)
+            construction.getResourceRequirementsPerTurn(StateForConditionals(civInfo))
+        else construction.getResourceRequirementsPerTurn(StateForConditionals(civInfo, cityInfo))
         // Does it even require any resources?
         if (requiredResources.isEmpty())
             return true
@@ -295,9 +307,11 @@ object Automation {
             for (city in civInfo.cities) {
                 val otherConstruction = city.cityConstructions.getCurrentConstruction()
                 if (otherConstruction is Building)
-                    futureForBuildings += otherConstruction.getResourceRequirementsPerTurn()[resource]
+                    futureForBuildings += otherConstruction.getResourceRequirementsPerTurn(
+                        StateForConditionals(civInfo, city))[resource]
                 else
-                    futureForUnits += otherConstruction.getResourceRequirementsPerTurn()[resource]
+                    futureForUnits += otherConstruction.getResourceRequirementsPerTurn(
+                        StateForConditionals(civInfo))[resource]
             }
 
             // Make sure we have some for space
@@ -345,8 +359,8 @@ object Automation {
         return when {
             powerLevelComparison > 2 -> ThreatLevel.VeryHigh
             powerLevelComparison > 1.5f -> ThreatLevel.High
-            powerLevelComparison < (1 / 1.5f) -> ThreatLevel.Low
             powerLevelComparison < 0.5f -> ThreatLevel.VeryLow
+            powerLevelComparison < (1 / 1.5f) -> ThreatLevel.Low
             else -> ThreatLevel.Medium
         }
     }
@@ -357,16 +371,17 @@ object Automation {
         return city.getTiles().filter {
             it.improvementFunctions.canBuildImprovement(improvement, city.civ)
         }.maxByOrNull {
-            rankTileForCityWork(it, city, city.cityStats.currentCityStats, localUniqueCache)
+            rankTileForCityWork(it, city, localUniqueCache)
         }
     }
 
     // Ranks a tile for any purpose except the expansion algorithm of cities
-    internal fun rankTile(tile: Tile?, civInfo: Civilization): Float {
+    internal fun rankTile(tile: Tile?, civInfo: Civilization,
+                          localUniqueCache: LocalUniqueCache): Float {
         if (tile == null) return 0f
         val tileOwner = tile.getOwner()
         if (tileOwner != null && tileOwner != civInfo) return 0f // Already belongs to another civilization, useless to us
-        val stats = tile.stats.getTileStats(null, civInfo)
+        val stats = tile.stats.getTileStats(null, civInfo, localUniqueCache)
         var rank = rankStatsValue(stats, civInfo)
         if (tile.improvement == null) rank += 0.5f // improvement potential!
         if (tile.isPillaged()) rank += 0.6f
@@ -396,12 +411,12 @@ object Automation {
         // Resources are good: less points
         if (tile.hasViewableResource(city.civ)) {
             if (tile.tileResource.resourceType != ResourceType.Bonus) score -= 105
-            else if (distance <= 3) score -= 104
+            else if (distance <= city.getWorkRange()) score -= 104
         } else {
             // Water tiles without resources aren't great
             if (tile.isWater) score += 25
             // Can't work it anyways
-            if (distance > 3) score += 100
+            if (distance > city.getWorkRange()) score += 100
         }
 
         if (tile.naturalWonder != null) score -= 105
@@ -415,12 +430,12 @@ object Automation {
         for (adjacentTile in tile.neighbors.filter { it.getOwner() == null }) {
             val adjacentDistance = city.getCenterTile().aerialDistanceTo(adjacentTile)
             if (adjacentTile.hasViewableResource(city.civ) &&
-                (adjacentDistance < 3 ||
+                (adjacentDistance < city.getWorkRange() ||
                     adjacentTile.tileResource.resourceType != ResourceType.Bonus
                 )
             ) score -= 1
             if (adjacentTile.naturalWonder != null) {
-                if (adjacentDistance < 3) adjacentNaturalWonder = true
+                if (adjacentDistance < city.getWorkRange()) adjacentNaturalWonder = true
                 score -= 1
             }
         }
@@ -445,9 +460,11 @@ object Automation {
                 else
                     stats.gold / 3 // 3 gold is much worse than 2 production
 
+        rank += stats.happiness
         rank += stats.production
         rank += stats.science
         rank += stats.culture
+        rank += stats.faith
         return rank
     }
 }
