@@ -2,13 +2,14 @@ package com.unciv.logic.automation.civilization
 
 import com.unciv.Constants
 import com.unciv.logic.city.City
-import com.unciv.models.ruleset.INonPerpetualConstruction
 import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.civilization.managers.ReligionState
 import com.unciv.logic.map.tile.Tile
+import com.unciv.models.Counter
 import com.unciv.models.ruleset.Belief
 import com.unciv.models.ruleset.BeliefType
 import com.unciv.models.ruleset.Victory
+import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.stats.Stat
 import kotlin.math.min
@@ -41,7 +42,9 @@ object ReligionAutomation {
         val citiesWithoutOurReligion = civInfo.cities.filter { it.religion.getMajorityReligion() != civInfo.religionManager.religion!! }
         // The original had a cap at 4 missionaries total, but 1/4 * the number of cities should be more appropriate imo
         if (citiesWithoutOurReligion.count() >
-            4 * civInfo.units.getCivUnits().count { it.canDoLimitedAction(Constants.spreadReligion) || it.canDoLimitedAction(Constants.removeHeresy) }
+            4 * civInfo.units.getCivUnits().count {
+                    it.hasUnique(UniqueType.CanSpreadReligion)
+                    || it.hasUnique(UniqueType.CanRemoveHeresy) }
         ) {
             val (city, pressureDifference) = citiesWithoutOurReligion.map { city ->
                 city to city.religion.getPressureDeficit(civInfo.religionManager.religion?.name)
@@ -77,7 +80,7 @@ object ReligionAutomation {
         // Todo: buy Great People post industrial era
 
         // Just buy missionaries to spread our religion outside of our civ
-        if (civInfo.units.getCivUnits().count { it.canDoLimitedAction(Constants.spreadReligion) } < 4) {
+        if (civInfo.units.getCivUnits().count { it.hasUnique(UniqueType.CanSpreadReligion) } < 4) {
             buyMissionaryInAnyCity(civInfo)
             return
         }
@@ -90,13 +93,12 @@ object ReligionAutomation {
             val buildings = city.religion.getMajorityReligion()!!.buildingsPurchasableByBeliefs
             val buildingToBePurchased = buildings
                 .asSequence()
-                .map { civInfo.getEquivalentBuilding(it).name }
-                .map { city.cityConstructions.getConstruction(it) as INonPerpetualConstruction }
+                .map { civInfo.getEquivalentBuilding(it) }
                 .filter { it.isPurchasable(city.cityConstructions) }
                 .filter { (it.getStatBuyCost(city, Stat.Faith) ?: return@filter false) <= civInfo.religionManager.storedFaith }
                 .minByOrNull { it.getStatBuyCost(city, Stat.Faith)!! }
                 ?: continue
-            city.cityConstructions.purchaseConstruction(buildingToBePurchased.name, -1, true, Stat.Faith)
+            city.cityConstructions.purchaseConstruction(buildingToBePurchased, -1, true, Stat.Faith)
             return
         }
     }
@@ -104,7 +106,7 @@ object ReligionAutomation {
     private fun buyMissionaryInAnyCity(civInfo: Civilization) {
         if (civInfo.religionManager.religionState < ReligionState.Religion) return
         var missionaries = civInfo.gameInfo.ruleset.units.values.filter { unit ->
-            unit.getMatchingUniques(UniqueType.CanActionSeveralTimes).filter { it.params[0] == Constants.spreadReligion }.any()
+                unit.hasUnique(UniqueType.CanSpreadReligion)
         }
         missionaries = missionaries.map { civInfo.getEquivalentUnit(it) }
 
@@ -117,7 +119,7 @@ object ReligionAutomation {
             ?: return
 
 
-        val hasUniqueToTakeCivReligion = civInfo.gameInfo.ruleset.units[missionaryConstruction.name]!!.hasUnique(UniqueType.TakeReligionOverBirthCity)
+        val hasUniqueToTakeCivReligion = missionaryConstruction.hasUnique(UniqueType.TakeReligionOverBirthCity)
 
         val validCitiesToBuy = civInfo.cities.filter {
             (hasUniqueToTakeCivReligion || it.religion.getMajorityReligion() == civInfo.religionManager.religion)
@@ -128,7 +130,11 @@ object ReligionAutomation {
         if (validCitiesToBuy.isEmpty()) return
 
         val citiesWithBonusCharges = validCitiesToBuy.filter { city ->
-            city.getLocalMatchingUniques(UniqueType.UnitStartingActions).filter { it.params[2] == Constants.spreadReligion }.any()
+            city.getMatchingUniques(UniqueType.UnitStartingPromotions).any {
+                val promotionName = it.params[2]
+                val promotion = city.getRuleset().unitPromotions[promotionName] ?: return@any false
+                promotion.hasUnique(UniqueType.CanSpreadReligion)
+            }
         }
         val holyCity = validCitiesToBuy.firstOrNull { it.isHolyCityOf(civInfo.religionManager.religion!!.name) }
 
@@ -138,21 +144,20 @@ object ReligionAutomation {
             else -> validCitiesToBuy.first()
         }
 
-        cityToBuyMissionary.cityConstructions.purchaseConstruction(missionaryConstruction.name, -1, true, Stat.Faith)
+        cityToBuyMissionary.cityConstructions.purchaseConstruction(missionaryConstruction, -1, true, Stat.Faith)
         return
     }
 
     private fun buyGreatProphetInAnyCity(civInfo: Civilization) {
         if (civInfo.religionManager.religionState < ReligionState.Religion) return
         var greatProphetUnit = civInfo.religionManager.getGreatProphetEquivalent() ?: return
-        greatProphetUnit = civInfo.getEquivalentUnit(greatProphetUnit).name
-        val greatProphetConstruction = civInfo.cities.first().cityConstructions.getConstruction(greatProphetUnit) as INonPerpetualConstruction
+        greatProphetUnit = civInfo.getEquivalentUnit(greatProphetUnit)
         val cityToBuyGreatProphet = civInfo.cities
             .asSequence()
-            .filter { greatProphetConstruction.isPurchasable(it.cityConstructions) }
-            .filter { greatProphetConstruction.canBePurchasedWithStat(it, Stat.Faith) }
-            .filter { (greatProphetConstruction.getStatBuyCost(it, Stat.Faith) ?: return@filter false) <= civInfo.religionManager.storedFaith }
-            .minByOrNull { greatProphetConstruction.getStatBuyCost(it, Stat.Faith)!! }
+            .filter { greatProphetUnit.isPurchasable(it.cityConstructions) }
+            .filter { greatProphetUnit.canBePurchasedWithStat(it, Stat.Faith) }
+            .filter { (greatProphetUnit.getStatBuyCost(it, Stat.Faith) ?: return@filter false) <= civInfo.religionManager.storedFaith }
+            .minByOrNull { greatProphetUnit.getStatBuyCost(it, Stat.Faith)!! }
             ?: return
         cityToBuyGreatProphet.cityConstructions.purchaseConstruction(greatProphetUnit, -1, true, Stat.Faith)
     }
@@ -160,8 +165,7 @@ object ReligionAutomation {
     private fun buyInquisitorNear(civInfo: Civilization, city: City) {
         if (civInfo.religionManager.religionState < ReligionState.Religion) return
         var inquisitors = civInfo.gameInfo.ruleset.units.values.filter {
-            it.getMapUnit(civInfo).canDoLimitedAction(Constants.removeHeresy)
-            || it.hasUnique(UniqueType.PreventSpreadingReligion)
+            it.hasUnique(UniqueType.CanRemoveHeresy) || it.hasUnique(UniqueType.PreventSpreadingReligion)
         }
 
         inquisitors = inquisitors.map { civInfo.getEquivalentUnit(it) }
@@ -175,7 +179,7 @@ object ReligionAutomation {
             ?: return
 
 
-        val hasUniqueToTakeCivReligion = civInfo.gameInfo.ruleset.units[inquisitorConstruction.name]!!.hasUnique(UniqueType.TakeReligionOverBirthCity)
+        val hasUniqueToTakeCivReligion = inquisitorConstruction.hasUnique(UniqueType.TakeReligionOverBirthCity)
 
         val validCitiesToBuy = civInfo.cities.filter {
             (hasUniqueToTakeCivReligion || it.religion.getMajorityReligion() == civInfo.religionManager.religion)
@@ -183,16 +187,10 @@ object ReligionAutomation {
             && inquisitorConstruction.isPurchasable(it.cityConstructions)
             && inquisitorConstruction.canBePurchasedWithStat(it, Stat.Faith)
         }
-        if (validCitiesToBuy.isEmpty()) return
-
         val cityToBuy = validCitiesToBuy
-            .filter {
-                inquisitorConstruction.isPurchasable(it.cityConstructions)
-                && inquisitorConstruction.canBePurchasedWithStat(it, Stat.Faith)
-            }
             .minByOrNull { it.getCenterTile().aerialDistanceTo(city.getCenterTile()) } ?: return
 
-        cityToBuy.cityConstructions.purchaseConstruction(inquisitorConstruction.name, -1, true, Stat.Faith)
+        cityToBuy.cityConstructions.purchaseConstruction(inquisitorConstruction, -1, true, Stat.Faith)
     }
 
     // endregion
@@ -203,7 +201,7 @@ object ReligionAutomation {
         var score = 0f
 
         for (city in civInfo.cities) {
-            for (tile in city.getCenterTile().getTilesInDistance(3)) {
+            for (tile in city.getCenterTile().getTilesInDistance(city.getWorkRange())) {
                 val tileScore = beliefBonusForTile(belief, tile, city)
                 score += tileScore * when {
                     city.workedTiles.contains(tile.position) -> 8
@@ -331,7 +329,7 @@ object ReligionAutomation {
 
         for (unique in belief.uniqueObjects) {
             val modifier =
-                if (unique.conditionals.any { it.type == UniqueType.ConditionalOurUnit && it.params[0] == civInfo.religionManager.getGreatProphetEquivalent() }) 1/2f
+                if (unique.conditionals.any { it.type == UniqueType.ConditionalOurUnit && it.params[0] == civInfo.religionManager.getGreatProphetEquivalent()?.name }) 1/2f
                 else 1f
             // Some city-filters are modified by personality (non-enemy foreign cities)
             score += modifier * when (unique.type) {
@@ -393,5 +391,91 @@ object ReligionAutomation {
         return score
     }
 
+
+    internal fun chooseReligiousBeliefs(civInfo: Civilization) {
+        choosePantheon(civInfo)
+        foundReligion(civInfo)
+        enhanceReligion(civInfo)
+        chooseFreeBeliefs(civInfo)
+    }
+
+    private fun choosePantheon(civInfo: Civilization) {
+        if (!civInfo.religionManager.canFoundOrExpandPantheon()) return
+        // So looking through the source code of the base game available online,
+        // the functions for choosing beliefs total in at around 400 lines.
+        // https://github.com/Gedemon/Civ5-DLL/blob/aa29e80751f541ae04858b6d2a2c7dcca454201e/CvGameCoreDLL_Expansion1/CvReligionClasses.cpp
+        // line 4426 through 4870.
+        // This is way too much work for now, so I'll just choose a random pantheon instead.
+        // Should probably be changed later, but it works for now.
+        val chosenPantheon = chooseBeliefOfType(civInfo, BeliefType.Pantheon)
+            ?: return // panic!
+        civInfo.religionManager.chooseBeliefs(
+            listOf(chosenPantheon),
+            useFreeBeliefs = civInfo.religionManager.usingFreeBeliefs()
+        )
+    }
+
+    private fun foundReligion(civInfo: Civilization) {
+        if (civInfo.religionManager.religionState != ReligionState.FoundingReligion) return
+        val availableReligionIcons = civInfo.gameInfo.ruleset.religions
+            .filterNot { civInfo.gameInfo.religions.values.map { religion -> religion.name }.contains(it) }
+        val favoredReligion = civInfo.nation.favoredReligion
+        val religionIcon =
+            if (favoredReligion != null && favoredReligion in availableReligionIcons
+                && (1..10).random() <= 5) favoredReligion
+            else availableReligionIcons.randomOrNull()
+                ?: return // Wait what? How did we pass the checking when using a great prophet but not this?
+
+        civInfo.religionManager.foundReligion(religionIcon, religionIcon)
+
+        val chosenBeliefs = chooseBeliefs(civInfo, civInfo.religionManager.getBeliefsToChooseAtFounding()).toList()
+        civInfo.religionManager.chooseBeliefs(chosenBeliefs)
+    }
+
+    private fun enhanceReligion(civInfo: Civilization) {
+        if (civInfo.religionManager.religionState != ReligionState.EnhancingReligion) return
+        civInfo.religionManager.chooseBeliefs(
+            chooseBeliefs(civInfo, civInfo.religionManager.getBeliefsToChooseAtEnhancing()).toList()
+        )
+    }
+
+    private fun chooseFreeBeliefs(civInfo: Civilization) {
+        if (!civInfo.religionManager.hasFreeBeliefs()) return
+        civInfo.religionManager.chooseBeliefs(
+            chooseBeliefs(civInfo, civInfo.religionManager.freeBeliefsAsEnums()).toList(),
+            useFreeBeliefs = true
+        )
+    }
+
+    private fun chooseBeliefs(civInfo: Civilization, beliefsToChoose: Counter<BeliefType>): HashSet<Belief> {
+        val chosenBeliefs = hashSetOf<Belief>()
+        // The `continue`s should never be reached, but just in case I'd rather have the AI have a
+        // belief less than make the game crash. The `continue`s should only be reached whenever
+        // there are not enough beliefs to choose, but there should be, as otherwise we could
+        // not have used a great prophet to found/enhance our religion.
+        for (belief in BeliefType.values()) {
+            if (belief == BeliefType.None) continue
+            repeat(beliefsToChoose[belief]) {
+                chosenBeliefs.add(
+                    chooseBeliefOfType(civInfo, belief, chosenBeliefs) ?: return@repeat
+                )
+            }
+        }
+        return chosenBeliefs
+    }
+
+    private fun chooseBeliefOfType(civInfo: Civilization, beliefType: BeliefType, additionalBeliefsToExclude: HashSet<Belief> = hashSetOf()): Belief? {
+        return civInfo.gameInfo.ruleset.beliefs.values
+            .filter {
+                (it.type == beliefType || beliefType == BeliefType.Any)
+                    && !additionalBeliefsToExclude.contains(it)
+                    && civInfo.religionManager.getReligionWithBelief(it) == null
+                    && it.getMatchingUniques(UniqueType.OnlyAvailable, StateForConditionals.IgnoreConditionals)
+                    .none { unique -> !unique.conditionalsApply(civInfo) }
+            }
+            .maxByOrNull { ReligionAutomation.rateBelief(civInfo, it) }
+    }
+
+
     //endregion
 }
