@@ -13,14 +13,14 @@ import com.unciv.logic.battle.TargetHelper
 import com.unciv.logic.city.City
 import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.civilization.NotificationCategory
+import com.unciv.logic.civilization.diplomacy.DiplomacyFlags
 import com.unciv.logic.civilization.diplomacy.DiplomaticStatus
-import com.unciv.logic.civilization.managers.ReligionState
 import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.logic.map.tile.Tile
-import com.unciv.models.UnitActionType
-import com.unciv.models.metadata.GameSettings
+import com.unciv.models.UpgradeUnitAction
+import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.UniqueType
-import com.unciv.ui.screens.worldscreen.unit.actions.UnitActions
+import com.unciv.models.ruleset.unit.BaseUnit
 import com.unciv.ui.screens.worldscreen.unit.actions.UnitActionsPillage
 import com.unciv.ui.screens.worldscreen.unit.actions.UnitActionsUpgrade
 
@@ -34,7 +34,7 @@ object UnitAutomation {
                 && tile.neighbors.any { !unit.civ.hasExplored(it) }
                 && (!unit.civ.isCityState() || tile.neighbors.any { it.getOwner() == unit.civ }) // Don't want city-states exploring far outside their borders
                 && unit.getDamageFromTerrain(tile) <= 0    // Don't take unnecessary damage
-                && tile.getTilesInDistance(3) .none { containsEnemyMilitaryUnit(unit, it) } // don't walk in range of enemy units
+                && unit.civ.threatManager.getDistanceToClosestEnemyUnit(tile, 3) > 3 // don't walk in range of enemy units
                 && unit.movement.canMoveTo(tile) // expensive, evaluate last
                 && unit.movement.canReach(tile) // expensive, evaluate last
     }
@@ -112,7 +112,7 @@ object UnitAutomation {
                 && unit.movement.canReach(tile) // expensive, evaluate last
     }
 
-    fun wander(unit: MapUnit, stayInTerritory: Boolean = false, tilesToAvoid:Set<Tile> = setOf()) {
+    fun wander(unit: MapUnit, stayInTerritory: Boolean = false, tilesToAvoid: Set<Tile> = setOf()) {
         val unitDistanceToTiles = unit.movement.getDistanceToTiles()
         val reachableTiles = unitDistanceToTiles
                 .filter {
@@ -130,24 +130,42 @@ object UnitAutomation {
     }
 
     internal fun tryUpgradeUnit(unit: MapUnit): Boolean {
-        val isHuman = unit.civ.isHuman()
-        if (!UncivGame.Current.settings.automatedUnitsCanUpgrade && isHuman) return false
-        if (unit.baseUnit.upgradesTo == null) return false
-        val upgradedUnit = unit.upgrade.getUnitToUpgradeTo()
-        if (!upgradedUnit.isBuildable(unit.civ)) return false // for resource reasons, usually
+        if (unit.civ.isHuman() && !UncivGame.Current.settings.automatedUnitsCanUpgrade
+            && UncivGame.Current.worldScreen?.autoPlay?.isAutoPlayingAndFullAutoPlayAI() == false) return false
 
-        if (upgradedUnit.getResourceRequirementsPerTurn().keys.any { !unit.baseUnit.requiresResource(it) }) {
+        val upgradeUnits = getUnitsToUpgradeTo(unit)
+        if (upgradeUnits.none()) return false // for resource reasons, usually
+        val upgradedUnit = upgradeUnits.minBy { it.cost }
+
+        if (upgradedUnit.getResourceRequirementsPerTurn(StateForConditionals(unit.civ, unit = unit)).keys.any { !unit.requiresResource(it) }) {
             // The upgrade requires new resource types, so check if we are willing to invest them
             if (!Automation.allowSpendingResource(unit.civ, upgradedUnit)) return false
         }
 
-        val upgradeAction = UnitActionsUpgrade.getUpgradeAction(unit)
-            ?: return false
+        val upgradeActions = UnitActionsUpgrade.getUpgradeActions(unit)
 
-        upgradeAction.action?.invoke()
+        upgradeActions.firstOrNull{ (it as UpgradeUnitAction).unitToUpgradeTo == upgradedUnit }?.action?.invoke() ?: return false
+        //todo Incorrect - an _unsuccessful_ upgrade might have _resurrected_ the original in which case it's a new clone, and unit.isDestroyed is still true
         return unit.isDestroyed // a successful upgrade action will destroy this unit
     }
 
+    /** Get the base unit this map unit could upgrade to, respecting researched tech and nation uniques only.
+     *  Note that if the unit can't upgrade, the current BaseUnit is returned.
+     */
+    private fun getUnitsToUpgradeTo(unit: MapUnit): Sequence<BaseUnit> {
+
+        fun isInvalidUpgradeDestination(baseUnit: BaseUnit): Boolean {
+            if (!unit.civ.tech.isResearched(baseUnit))
+                return true
+            return baseUnit.getMatchingUniques(UniqueType.OnlyAvailable, StateForConditionals.IgnoreConditionals)
+                .any { !it.conditionalsApply(StateForConditionals(unit.civ, unit = unit)) }
+        }
+
+        return unit.baseUnit.getRulesetUpgradeUnits(StateForConditionals(unit.civ, unit = unit))
+            .map { unit.civ.getEquivalentUnit(it) }
+            .filter { !isInvalidUpgradeDestination(it) && unit.upgrade.canUpgrade(it) }
+    }
+
     fun automateUnitMoves(unit: MapUnit) {
         check(!unit.civ.isBarbarian()) { "Barbarians is not allowed here." }
 
@@ -156,48 +174,58 @@ object UnitAutomation {
 
 
         if (unit.isCivilian()) {
-            automateCivilianUnit(unit)
+            CivilianUnitAutomation.automateCivilianUnit(unit, getDangerousTiles(unit))
             return
         }
 
-        if (unit.promotions.canBePromoted()) {
+        while (unit.promotions.canBePromoted() &&
+            // Restrict Human automated units from promotions via setting
+                (UncivGame.Current.settings.automatedUnitsChoosePromotions || unit.civ.isAI())) {
             val availablePromotions = unit.promotions.getAvailablePromotions()
                 .filterNot { it.hasUnique(UniqueType.SkipPromotion) }
-            if (availablePromotions.any())
-                unit.promotions.addPromotion(availablePromotions.toList().random().name)
+            if (availablePromotions.none()) break
+            unit.promotions.addPromotion(
+                availablePromotions.filter { it.hasUnique(UniqueType.FreePromotion) }.toList().randomOrNull()?.name
+                    ?: availablePromotions.toList().random().name)
         }
 
         //This allows for military units with certain civilian abilities to behave as civilians in peace and soldiers in war
         if ((unit.hasUnique(UniqueType.BuildImprovements) || unit.hasUnique(UniqueType.FoundCity) ||
                 unit.hasUnique(UniqueType.ReligiousUnit) || unit.hasUnique(UniqueType.CreateWaterImprovements))
                 && !unit.civ.isAtWar()){
-            automateCivilianUnit(unit)
+            CivilianUnitAutomation.automateCivilianUnit(unit, getDangerousTiles(unit))
             return
         }
 
-        if (unit.baseUnit.isAirUnit() && unit.canIntercept())
-            return SpecificUnitAutomation.automateFighter(unit)
+        // Note that not all nukes have to be air units
+        if (unit.isNuclearWeapon()) {
+            return AirUnitAutomation.automateNukes(unit)
+        }
 
-        if (unit.baseUnit.isAirUnit())
-            return SpecificUnitAutomation.automateBomber(unit)
+        if (unit.baseUnit.isAirUnit()) {
+            if (unit.canIntercept())
+                return AirUnitAutomation.automateFighter(unit)
 
-        if (unit.baseUnit.isNuclearWeapon())
-            return SpecificUnitAutomation.automateNukes(unit)
+            if (unit.hasUnique(UniqueType.SelfDestructs))
+                return AirUnitAutomation.automateMissile(unit)
 
-        if (unit.hasUnique(UniqueType.SelfDestructs))
-            return SpecificUnitAutomation.automateMissile(unit)
+            return AirUnitAutomation.automateBomber(unit)
+        }
+
+        // Accompany settlers
+        if (tryAccompanySettlerOrGreatPerson(unit)) return
 
         if (tryGoToRuinAndEncampment(unit) && unit.currentMovement == 0f) return
 
         if (tryUpgradeUnit(unit)) return
 
-        // Accompany settlers
-        if (tryAccompanySettlerOrGreatPerson(unit)) return
+        if (unit.health < 50 && (tryRetreat(unit) || tryHealUnit(unit))) return // do nothing but heal
+
+        // If there are no enemies nearby and we can heal here, wait until we are at full health
+        if (unit.health < 100 && canUnitHealInTurnsOnCurrentTile(unit,2, 4)) return
 
         if (tryHeadTowardsOurSiegedCity(unit)) return
 
-        if (unit.health < 50 && tryHealUnit(unit)) return // do nothing but heal
-
         // if a embarked melee unit can land and attack next turn, do not attack from water.
         if (BattleHelper.tryDisembarkUnitToAttackPosition(unit)) return
 
@@ -207,7 +235,7 @@ object UnitAutomation {
         if (tryTakeBackCapturedCity(unit)) return
 
         // Focus all units without a specific target on the enemy city closest to one of our cities
-        if (tryHeadTowardsEnemyCity(unit)) return
+        if (HeadTowardsEnemyCityAutomation.tryHeadTowardsEnemyCity(unit)) return
 
         if (tryGarrisoningRangedLandUnit(unit)) return
 
@@ -222,6 +250,8 @@ object UnitAutomation {
 
         if (unit.health < 100 && tryHealUnit(unit)) return
 
+        if (tryPrepare(unit)) return
+
         // else, try to go to unreached tiles
         if (tryExplore(unit)) return
 
@@ -232,140 +262,13 @@ object UnitAutomation {
             wander(unit, stayInTerritory = true)
     }
 
-    private fun automateCivilianUnit(unit: MapUnit) {
-        if (tryRunAwayIfNeccessary(unit)) return
-
-        if (unit.currentTile.isCityCenter() && unit.currentTile.getCity()!!.isCapital()
-                && !unit.hasUnique(UniqueType.AddInCapital)
-                && unit.civ.units.getCivUnits().any { unit.hasUnique(UniqueType.AddInCapital) }){
-            // First off get out of the way, then decide if you actually want to do something else
-            val tilesCanMoveTo = unit.movement.getDistanceToTiles()
-                .filter { unit.movement.canMoveTo(it.key) }
-            if (tilesCanMoveTo.isNotEmpty())
-                unit.movement.moveToTile(tilesCanMoveTo.minByOrNull { it.value.totalDistance }!!.key)
-        }
-
-        val tilesWhereWeWillBeCaptured = unit.currentTile.getTilesInDistance(5)
-            .mapNotNull { it.militaryUnit }
-            .filter { it.civ.isAtWarWith(unit.civ) }
-            .flatMap { it.movement.getReachableTilesInCurrentTurn() }
-            .filter { it.militaryUnit?.civ != unit.civ }
-            .toSet()
-
-        if (unit.hasUnique(UniqueType.FoundCity))
-            return SpecificUnitAutomation.automateSettlerActions(unit, tilesWhereWeWillBeCaptured)
-
-        if (unit.cache.hasUniqueToBuildImprovements)
-            return unit.civ.getWorkerAutomation().automateWorkerAction(unit, tilesWhereWeWillBeCaptured)
-
-        if (unit.cache.hasUniqueToCreateWaterImprovements){
-            if (!unit.civ.getWorkerAutomation().automateWorkBoats(unit))
-                tryExplore(unit)
-            return
-        }
-
-        if (unit.hasUnique(UniqueType.MayFoundReligion)
-                && unit.civ.religionManager.religionState < ReligionState.Religion
-                && unit.civ.religionManager.mayFoundReligionAtAll(unit)
-        )
-            return SpecificUnitAutomation.foundReligion(unit)
-
-        if (unit.hasUnique(UniqueType.MayEnhanceReligion)
-                && unit.civ.religionManager.religionState < ReligionState.EnhancedReligion
-                && unit.civ.religionManager.mayEnhanceReligionAtAll(unit)
-        )
-            return SpecificUnitAutomation.enhanceReligion(unit)
-
-        // We try to add any unit in the capital we can, though that might not always be desirable
-        // For now its a simple option to allow AI to win a science victory again
-        if (unit.hasUnique(UniqueType.AddInCapital))
-            return SpecificUnitAutomation.automateAddInCapital(unit)
-
-        //todo this now supports "Great General"-like mod units not combining 'aura' and citadel
-        // abilities, but not additional capabilities if automation finds no use for those two
-        if (unit.cache.hasStrengthBonusInRadiusUnique
-                && SpecificUnitAutomation.automateGreatGeneral(unit))
-            return
-        if (unit.cache.hasCitadelPlacementUnique && SpecificUnitAutomation.automateCitadelPlacer(unit))
-            return
-        if (unit.cache.hasCitadelPlacementUnique || unit.cache.hasStrengthBonusInRadiusUnique)
-            return SpecificUnitAutomation.automateGreatGeneralFallback(unit)
-
-        if (unit.civ.religionManager.maySpreadReligionAtAll(unit))
-            return SpecificUnitAutomation.automateMissionary(unit)
-
-        if (unit.hasUnique(UniqueType.PreventSpreadingReligion) || unit.canDoLimitedAction(Constants.removeHeresy))
-            return SpecificUnitAutomation.automateInquisitor(unit)
-
-        val isLateGame = isLateGame(unit.civ)
-        // Great scientist -> Hurry research if late game
-        if (isLateGame) {
-            val hurryResearch = UnitActions.getUnitActions(unit)
-                .firstOrNull { it.type == UnitActionType.HurryResearch }?.action
-            if (hurryResearch != null) {
-                hurryResearch()
-                return
-            }
-        }
-
-        // Great merchant -> Conduct trade mission if late game and if not at war.
-        // TODO: This could be more complex to walk to the city state that is most beneficial to
-        //  also have more influence.
-        if (unit.hasUnique(UniqueType.CanTradeWithCityStateForGoldAndInfluence)
-                // Don't wander around with the great merchant when at war. Barbs might also be a
-                // problem, but hopefully by the time we have a great merchant, they're under control.
-                && !unit.civ.isAtWar()
-                && isLateGame
-        ) {
-            val tradeMissionCanBeConductedEventually =
-                    SpecificUnitAutomation.conductTradeMission(unit)
-            if (tradeMissionCanBeConductedEventually)
-                return
-        }
-
-        // Great engineer -> Try to speed up wonder construction if late game
-        if (isLateGame &&
-                (unit.hasUnique(UniqueType.CanSpeedupConstruction)
-                        || unit.hasUnique(UniqueType.CanSpeedupWonderConstruction))) {
-            val wonderCanBeSpedUpEventually = SpecificUnitAutomation.speedupWonderConstruction(unit)
-            if (wonderCanBeSpedUpEventually)
-                return
-        }
-
-
-        // This has to come after the individual abilities for the great people that can also place
-        // instant improvements (e.g. great scientist).
-        if (unit.hasUnique(UniqueType.ConstructImprovementInstantly)
-        ) {
-            // catch great prophet for civs who can't found/enhance/spread religion
-            // includes great people plus moddable units
-            val improvementCanBePlacedEventually =
-                    SpecificUnitAutomation.automateImprovementPlacer(unit)
-            if (!improvementCanBePlacedEventually)
-                startGoldenAgeIfHasAbility(unit)
-        }
-
-        // TODO: The AI tends to have a lot of great generals. Maybe there should be a cutoff
-        //  (depending on number of cities) and after that they should just be used to start golden
-        //  ages?
-
-        return // The AI doesn't know how to handle unknown civilian units
-    }
-
-    private fun isLateGame(civ: Civilization): Boolean {
-        val researchCompletePercent =
-                (civ.tech.researchedTechnologies.size * 1.0f) / civ.gameInfo.ruleset.technologies.size
-        return researchCompletePercent >= 0.8f
-    }
-
-    private fun startGoldenAgeIfHasAbility(unit: MapUnit) {
-        UnitActions.getUnitActions(unit).firstOrNull { it.type == UnitActionType.StartGoldenAge }?.action?.invoke()
-    }
 
     /** @return true only if the unit has 0 movement left */
     private fun tryAttacking(unit: MapUnit): Boolean {
         repeat(unit.maxAttacksPerTurn() - unit.attacksThisTurn) {
             if (BattleHelper.tryAttackNearbyEnemy(unit)) return true
+            // Calvary style tctic, attack and then retreat
+            if (unit.health < 50 && tryRetreat(unit)) return true
         }
         return false
     }
@@ -384,31 +287,80 @@ object UnitAutomation {
         return true
     }
 
+    private fun tryRetreat(unit: MapUnit): Boolean {
+        if (!unit.civ.isAtWar()) return false
+        // Precondition: This must be a military unit
+        if (unit.isCivilian()) return false
+        if (unit.baseUnit.isAirUnit()) return false
+        // Better to do a more healing oriented move then
+        if (unit.civ.threatManager.getDistanceToClosestEnemyUnit(unit.getTile(),6, true) > 4) return false
+
+
+        val unitDistanceToTiles = unit.movement.getDistanceToTiles()
+        val closestCity = unit.civ.cities.minByOrNull { it.getCenterTile().aerialDistanceTo(unit.getTile()) }?.takeIf { it.getCenterTile().aerialDistanceTo(unit.getTile()) < 20 }
+
+        // Finding the distance to the closest enemy is expensive, so lets sort the tiles using a cheaper function
+        val sortedTilesToRetreatTo: Sequence<Tile> = if (closestCity != null) {
+            // If we have a city, lets favor the tiles closer to that city
+            unitDistanceToTiles.asSequence().map { it.key }.sortedBy { it.aerialDistanceTo(closestCity.getCenterTile()) }
+        } else {
+            // Rare case, what if we don't have a city nearby?
+            // Lets favor the tiles that don't have enemies close by
+            // Ideally we should check in a greater radius but might get way too expensive
+            unitDistanceToTiles.asSequence().map { it.key }.sortedByDescending { unit.civ.threatManager.getDistanceToClosestEnemyUnit(it, 3, false) }
+        }
+
+        val ourDistanceToClosestEnemy = unit.civ.threatManager.getDistanceToClosestEnemyUnit(unit.getTile(),6, false)
+        // Lets check all tiles and swap with the first one
+        for (retreatTile in sortedTilesToRetreatTo) {
+            val tileDistanceToClosestEnemy = unit.civ.threatManager.getDistanceToClosestEnemyUnit(retreatTile,6,false)
+            if (ourDistanceToClosestEnemy >= tileDistanceToClosestEnemy) continue
+
+            val otherUnit = retreatTile.militaryUnit
+            if (otherUnit == null) {
+                // See if we can retreat to the tile
+                if (unit.movement.canMoveTo(retreatTile)) {
+                    unit.movement.moveToTile(retreatTile)
+                    return true
+                }
+            } else if (otherUnit.civ == unit.civ) {
+                // The tile is taken, lets see if we want to swap retreat to it
+                if (otherUnit.health > 80) {
+                    if (otherUnit.baseUnit.isRanged()) {
+                        // Don't swap ranged units closer than they have to be
+                        val range = otherUnit.baseUnit.range
+                        if (ourDistanceToClosestEnemy < range)
+                            continue
+                    }
+                    if (unit.movement.canUnitSwapTo(retreatTile)) {
+                        unit.movement.swapMoveToTile(retreatTile)
+                        return true
+                    }
+                }
+            }
+        }
+        return false
+    }
+
     private fun tryHealUnit(unit: MapUnit): Boolean {
         if (unit.baseUnit.isRanged() && unit.hasUnique(UniqueType.HealsEvenAfterAction))
             return false // will heal anyway, and attacks don't hurt
 
-        if (tryPillageImprovement(unit)) return true
+        // Try pillage improvements until healed
+        while(tryPillageImprovement(unit, false)) {
+            // If we are fully healed and can still do things, lets keep on going by returning false
+            if (unit.currentMovement == 0f || unit.health == 100) return unit.currentMovement == 0f
+        }
+
         val unitDistanceToTiles = unit.movement.getDistanceToTiles()
         if (unitDistanceToTiles.isEmpty()) return true // can't move, so...
 
+        // If the unit can heal on this tile in two turns, just heal here
+        if (canUnitHealInTurnsOnCurrentTile(unit,3,)) return true
+
         val currentUnitTile = unit.getTile()
 
-        val nearbyRangedEnemyUnits = unit.currentTile.getTilesInDistance(3)
-                .flatMap { tile -> tile.getUnits().filter { unit.civ.isAtWarWith(it.civ) } }
-
-        val tilesInRangeOfAttack = nearbyRangedEnemyUnits
-                .flatMap { it.getTile().getTilesInDistance(it.getRange()) }
-
-        val tilesWithinBombardmentRange = unit.currentTile.getTilesInDistance(3)
-                .filter { it.isCityCenter() && it.getCity()!!.civ.isAtWarWith(unit.civ) }
-                .flatMap { it.getTilesInDistance(it.getCity()!!.range) }
-
-        val tilesWithTerrainDamage = unit.currentTile.getTilesInDistance(3)
-                .filter { unit.getDamageFromTerrain(it) > 0 }
-
-        val dangerousTiles = (tilesInRangeOfAttack + tilesWithinBombardmentRange + tilesWithTerrainDamage).toHashSet()
-
+        val dangerousTiles = unit.civ.threatManager.getDangerousTiles(unit, 4)
 
         val viableTilesForHealing = unitDistanceToTiles.keys
                 .filter { it !in dangerousTiles && unit.movement.canMoveTo(it) }
@@ -454,22 +406,59 @@ object UnitAutomation {
         return true
     }
 
-    fun tryPillageImprovement(unit: MapUnit): Boolean {
+    /**
+     * @return true if the tile is safe and the unit can heal to full within [turns]
+     */
+    private fun canUnitHealInTurnsOnCurrentTile(unit: MapUnit, turns: Int, noEnemyDistance: Int = 3): Boolean {
+        if (unit.hasUnique(UniqueType.HealsEvenAfterAction)) return false // We can keep on moving
+        // Check if we are not in a safe city and there is an enemy nearby this isn't a good tile to heal on
+        if (!(unit.getTile().isCityCenter() && unit.getTile().getCity()!!.health > 50)
+            && unit.civ.threatManager.getDistanceToClosestEnemyUnit(unit.getTile(), noEnemyDistance) <= noEnemyDistance) return false
+
+        val healthRequiredPerTurn =  (100 - unit.health) / turns
+        return healthRequiredPerTurn <= unit.rankTileForHealing(unit.getTile())
+    }
+
+    private fun getDangerousTiles(unit: MapUnit): HashSet<Tile> {
+        val nearbyEnemyUnits = unit.currentTile.getTilesInDistance(3)
+            .flatMap { tile -> tile.getUnits().filter { unit.civ.isAtWarWith(it.civ) } }
+
+        val tilesInRangeOfAttack = nearbyEnemyUnits
+            .flatMap { it.getTile().getTilesInDistance((it.getMaxMovement() - 1) + it.getRange()) }
+
+        val tilesWithinBombardmentRange = unit.currentTile.getTilesInDistance(3)
+            .filter { it.isCityCenter() && it.getCity()!!.civ.isAtWarWith(unit.civ) }
+            .flatMap { it.getTilesInDistance(it.getCity()!!.getBombardRange()) }
+
+        val tilesWithTerrainDamage = unit.currentTile.getTilesInDistance(3)
+            .filter { unit.getDamageFromTerrain(it) > 0 }
+
+        return (tilesInRangeOfAttack + tilesWithinBombardmentRange + tilesWithTerrainDamage).toHashSet()
+    }
+
+    /**
+     * @return true if the unit was able to pillage a tile, false otherwise
+     */
+    fun tryPillageImprovement(unit: MapUnit, onlyPillageToHeal: Boolean = false): Boolean {
         if (unit.isCivilian()) return false
         val unitDistanceToTiles = unit.movement.getDistanceToTiles()
         val tilesThatCanWalkToAndThenPillage = unitDistanceToTiles
             .filter { it.value.totalDistance < unit.currentMovement }.keys
-            .filter { unit.movement.canMoveTo(it) && UnitActions.canPillage(unit, it)
+            .filter { unit.movement.canMoveTo(it) && UnitActionsPillage.canPillage(unit, it)
                     && (it.canPillageTileImprovement()
-                    || (it.canPillageRoad() && it.getRoadOwner() != null && unit.civ.isAtWarWith(it.getRoadOwner()!!)))}
+                    || (!onlyPillageToHeal && it.canPillageRoad() && it.getRoadOwner() != null && unit.civ.isAtWarWith(it.getRoadOwner()!!))) }
 
         if (tilesThatCanWalkToAndThenPillage.isEmpty()) return false
-        val tileToPillage = tilesThatCanWalkToAndThenPillage.maxByOrNull { it.getDefensiveBonus() }!!
+        val tileToPillage = tilesThatCanWalkToAndThenPillage.maxByOrNull { it.getDefensiveBonus(false) }!!
         if (unit.getTile() != tileToPillage)
             unit.movement.moveToTile(tileToPillage)
 
-        UnitActionsPillage.getPillageAction(unit)?.action?.invoke()
-        return unit.currentMovement == 0f
+        if (unit.currentTile != tileToPillage) return false
+
+        // We CANNOT use invokeUnitAction, since the default unit action contains a popup, which - when automated -
+        //  runs a UI action on a side thread leading to crash!
+        UnitActionsPillage.getPillageAction(unit, unit.currentTile)?.action?.invoke()
+        return true
     }
 
     /** Move towards the closest attackable enemy of the [unit].
@@ -478,7 +467,7 @@ object UnitAutomation {
      *  Tiles attack from which would result in instant death of the [unit] are ignored. */
     private fun tryAdvanceTowardsCloseEnemy(unit: MapUnit): Boolean {
         // this can be sped up if we check each layer separately
-        val unitDistanceToTiles = unit.movement.getDistanceToTilesWithinTurn(
+        val unitDistanceToTiles = unit.movement.getDistanceToTilesAtPosition(
                 unit.getTile().position,
                 unit.getMaxMovement() * CLOSE_ENEMY_TURNS_AWAY_LIMIT
         )
@@ -509,14 +498,43 @@ object UnitAutomation {
         return false
     }
 
+    /**
+     * Tries to move the unit to the closest city that is close to a target civilization
+     */
+    private fun tryPrepare(unit: MapUnit): Boolean {
+        val civInfo = unit.civ
+
+        fun hasPreparationFlag(targetCiv: Civilization): Boolean {
+            val diploManager = civInfo.getDiplomacyManager(targetCiv)
+            if (diploManager.hasFlag(DiplomacyFlags.Denunciation) 
+                    || diploManager.otherCivDiplomacy().hasFlag(DiplomacyFlags.Denunciation)) return true
+            if (diploManager.hasFlag(DiplomacyFlags.WaryOf) && diploManager.getFlag(DiplomacyFlags.WaryOf) < 0) return true
+            return false
+        }
+
+        val hostileCivs = civInfo.getKnownCivs().filter { it.isAtWarWith(civInfo) || hasPreparationFlag(it) }
+        val citiesToDefend = hostileCivs.mapNotNull { NextTurnAutomation.getClosestCities(civInfo, it) }
+                .sortedBy { unit.getTile().aerialDistanceTo(it.city1.getCenterTile()) }
+
+        // Move to the closest city with a tile we can enter nearby
+        for ((city, enemyCity) in citiesToDefend) {
+            if (unit.getTile().aerialDistanceTo(city.getCenterTile()) <= 2) return true
+            val tileToMoveTo = city.getCenterTile().getTilesInDistance(2).firstOrNull { unit.movement.canMoveTo(it) && unit.movement.canReach(it) } ?: continue
+            unit.movement.headTowards(tileToMoveTo)
+            return true
+        }
+        return false
+    }
+
     private fun tryAccompanySettlerOrGreatPerson(unit: MapUnit): Boolean {
+        val distanceToTiles = unit.movement.getDistanceToTiles()
         val settlerOrGreatPersonToAccompany = unit.civ.units.getCivUnits()
             .firstOrNull {
                 val tile = it.currentTile
                 it.isCivilian() &&
                         (it.hasUnique(UniqueType.FoundCity) || unit.isGreatPerson())
-                        && tile.militaryUnit == null && unit.movement.canMoveTo(tile)
-                        && unit.movement.getDistanceToTiles().containsKey(tile)
+                        && (tile == unit.currentTile || tile.militaryUnit == null && unit.movement.canMoveTo(tile))
+                        && distanceToTiles.containsKey(tile)
             } ?: return false
         unit.movement.headTowards(settlerOrGreatPersonToAccompany.currentTile)
         return true
@@ -545,114 +563,6 @@ object UnitAutomation {
         return unit.currentMovement == 0f
     }
 
-    fun tryHeadTowardsEnemyCity(unit: MapUnit): Boolean {
-        if (unit.civ.cities.isEmpty()) return false
-
-        // only focus on *attacking* 1 enemy at a time otherwise you'll lose on both fronts
-
-        val enemies = unit.civ.getKnownCivs()
-            .filter { unit.civ.isAtWarWith(it) && it.cities.isNotEmpty() }
-
-        val closestEnemyCity = enemies
-            .mapNotNull { NextTurnAutomation.getClosestCities(unit.civ, it) }
-            .minByOrNull { it.aerialDistance }?.city2
-          ?: return false // no attackable cities found
-
-        // Our main attack target is the closest city, but we're fine with deviating from that a bit
-        var enemyCitiesByPriority = closestEnemyCity.civ.cities
-            .associateWith { it.getCenterTile().aerialDistanceTo(closestEnemyCity.getCenterTile()) }
-            .filterNot { it.value > 10 } // anything 10 tiles away from the target is irrelevant
-            .asSequence().sortedBy { it.value }.map { it.key } // sort the list by closeness to target - least is best!
-
-        if (unit.baseUnit.isRanged()) // ranged units don't harm capturable cities, waste of a turn
-            enemyCitiesByPriority = enemyCitiesByPriority.filterNot { it.health == 1 }
-
-        val closestReachableEnemyCity = enemyCitiesByPriority
-                .firstOrNull { unit.movement.canReach(it.getCenterTile()) }
-
-        if (closestReachableEnemyCity != null) {
-            return headTowardsEnemyCity(
-                unit,
-                closestReachableEnemyCity.getCenterTile(),
-                // This should be cached after the `canReach` call above.
-                unit.movement.getShortestPath(closestReachableEnemyCity.getCenterTile())
-            )
-        }
-        return false
-    }
-
-
-    private fun headTowardsEnemyCity(
-        unit: MapUnit,
-        closestReachableEnemyCity: Tile,
-        shortestPath: List<Tile>
-    ): Boolean {
-        val unitDistanceToTiles = unit.movement.getDistanceToTiles()
-        val unitRange = unit.getRange()
-
-        if (unitRange > 2) { // long-ranged unit, should never be in a bombardable position
-            val tilesInBombardRange = closestReachableEnemyCity.getTilesInDistance(2).toSet()
-            val tileToMoveTo =
-                    unitDistanceToTiles.asSequence()
-                            .filter {
-                                it.key.aerialDistanceTo(closestReachableEnemyCity) <=
-                                        unitRange && it.key !in tilesInBombardRange
-                                        && unit.getDamageFromTerrain(it.key) <= 0 // Don't set up on a mountain
-                            }
-                        .minByOrNull { it.value.totalDistance }?.key
-
-            // move into position far away enough that the bombard doesn't hurt
-            if (tileToMoveTo != null) {
-                unit.movement.headTowards(tileToMoveTo)
-                return true
-            }
-            return false
-        }
-
-        // None of the stuff below is relevant if we're still quite far away from the city, so we
-        // short-circuit here for performance reasons.
-        val minDistanceFromCityToConsiderForLandingArea = 3
-        val maxDistanceFromCityToConsiderForLandingArea = 5
-        if (unit.currentTile.aerialDistanceTo(closestReachableEnemyCity) > maxDistanceFromCityToConsiderForLandingArea
-                // Even in the worst case of only being able to move 1 tile per turn, we would still
-                // not overshoot.
-                && shortestPath.size > minDistanceFromCityToConsiderForLandingArea ) {
-            unit.movement.moveToTile(shortestPath[0])
-            return true
-        }
-
-        val ourUnitsAroundEnemyCity = closestReachableEnemyCity.getTilesInDistance(6)
-            .flatMap { it.getUnits() }
-            .filter { it.isMilitary() && it.civ == unit.civ }
-
-        val city = closestReachableEnemyCity.getCity()!!
-        val cityCombatant = CityCombatant(city)
-
-        val expectedDamagePerTurn = ourUnitsAroundEnemyCity
-            .map { BattleDamage.calculateDamageToDefender(MapUnitCombatant(it), cityCombatant) }
-            .sum() // City heals 20 per turn
-
-        if (expectedDamagePerTurn < city.health && // If we can take immediately, go for it
-                (expectedDamagePerTurn <= 20 || city.health / (expectedDamagePerTurn-20) > 5)){ // otherwise check if we can take within a couple of turns
-
-            // We won't be able to take this even with 5 turns of continuous damage!
-            // don't head straight to the city, try to head to landing grounds -
-            // this is against tha AI's brilliant plan of having everyone embarked and attacking via sea when unnecessary.
-            val tileToHeadTo = closestReachableEnemyCity.getTilesInDistanceRange(3..5)
-                    .filter { it.isLand && unit.getDamageFromTerrain(it) <= 0 } // Don't head for hurty terrain
-                    .sortedBy { it.aerialDistanceTo(unit.currentTile) }
-                    .firstOrNull { (unit.movement.canMoveTo(it) || it == unit.currentTile) && unit.movement.canReach(it) }
-
-            if (tileToHeadTo != null) { // no need to worry, keep going as the movement alg. says
-                unit.movement.headTowards(tileToHeadTo)
-            }
-            return true
-        }
-
-        unit.movement.moveToTile(shortestPath[0]) // go for it!
-
-        return true
-    }
 
     fun tryEnterOwnClosestCity(unit: MapUnit): Boolean {
         val closestCity = unit.civ.cities
@@ -675,6 +585,7 @@ object UnitAutomation {
 
     private fun chooseBombardTarget(city: City): ICombatant? {
         var targets = TargetHelper.getBombardableTiles(city).map { Battle.getMapCombatantOfTile(it)!! }
+            .filterNot { it.isCivilian() && !it.getUnitType().hasUnique(UniqueType.Uncapturable) } // Don't bombard capturable civilians
         if (targets.none()) return null
 
         val siegeUnits = targets
@@ -717,7 +628,7 @@ object UnitAutomation {
                 .firstOrNull { unit.movement.canReach(it) }
 
         if (closestReachableCapturedCity != null) {
-            return headTowardsEnemyCity(
+            return HeadTowardsEnemyCityAutomation.headTowardsEnemyCity(
                 unit,
                 closestReachableCapturedCity,
                 // This should be cached after the `canReach` call above.
@@ -725,7 +636,6 @@ object UnitAutomation {
             )
         }
         return false
-
     }
 
     private fun tryGarrisoningRangedLandUnit(unit: MapUnit): Boolean {
@@ -736,14 +646,6 @@ object UnitAutomation {
                     && unit.movement.canMoveTo(centerTile)
         }
 
-        fun isCityThatNeedsDefendingInWartime(city: City): Boolean {
-            if (city.health < city.getMaxHealth()) return true // this city is under attack!
-            for (enemyCivCity in unit.civ.diplomacy.values
-                    .filter { it.diplomaticStatus == DiplomaticStatus.War }
-                    .map { it.otherCiv() }.flatMap { it.cities })
-                if (city.getCenterTile().aerialDistanceTo(enemyCivCity.getCenterTile()) <= 5) return true // this is an edge city that needs defending
-            return false
-        }
 
         val citiesToTry = if (!unit.civ.isAtWar()) {
             if (unit.getTile().isCityCenter()) return true // It's always good to have a unit in the city center, so if you haven't found anyone around to attack, forget it.
@@ -765,6 +667,15 @@ object UnitAutomation {
         return true
     }
 
+    private fun isCityThatNeedsDefendingInWartime(city: City): Boolean {
+        if (city.health < city.getMaxHealth()) return true // this city is under attack!
+        for (enemyCivCity in city.civ.diplomacy.values
+            .filter { it.diplomaticStatus == DiplomaticStatus.War }
+            .map { it.otherCiv() }.flatMap { it.cities })
+            if (city.getCenterTile().aerialDistanceTo(enemyCivCity.getCenterTile()) <= 5) return true // this is an edge city that needs defending
+        return false
+    }
+
     private fun tryStationingMeleeNavalUnit(unit: MapUnit): Boolean {
         fun isMeleeNaval(mapUnit: MapUnit) = mapUnit.baseUnit.isMelee() && mapUnit.type.isWaterUnit()
 
@@ -804,55 +715,5 @@ object UnitAutomation {
         unit.action = null
     }
 
-    /** Returns whether the civilian spends its turn hiding and not moving */
-    private fun tryRunAwayIfNeccessary(unit: MapUnit): Boolean {
-        // This is a little 'Bugblatter Beast of Traal': Run if we can attack an enemy
-        // Cheaper than determining which enemies could attack us next turn
-        val enemyUnitsInWalkingDistance = unit.movement.getDistanceToTiles().keys
-            .filter { containsEnemyMilitaryUnit(unit, it) }
-
-        if (enemyUnitsInWalkingDistance.isNotEmpty() && !unit.baseUnit.isMilitary()
-            && unit.getTile().militaryUnit == null && !unit.getTile().isCityCenter()) {
-            runAway(unit)
-            return true
-        }
-
-        return false
-    }
-
-    private fun runAway(unit: MapUnit) {
-        val reachableTiles = unit.movement.getDistanceToTiles()
-        val enterableCity = reachableTiles.keys
-            .firstOrNull { it.isCityCenter() && unit.movement.canMoveTo(it) }
-        if (enterableCity != null) {
-            unit.movement.moveToTile(enterableCity)
-            return
-        }
-        val defensiveUnit = reachableTiles.keys
-            .firstOrNull {
-                it.militaryUnit != null && it.militaryUnit!!.civ == unit.civ && it.civilianUnit == null
-            }
-        if (defensiveUnit != null) {
-            unit.movement.moveToTile(defensiveUnit)
-            return
-        }
-        val tileFurthestFromEnemy = reachableTiles.keys
-            .filter { unit.movement.canMoveTo(it) && unit.getDamageFromTerrain(it) < unit.health }
-            .maxByOrNull { countDistanceToClosestEnemy(unit, it) }
-            ?: return // can't move anywhere!
-        unit.movement.moveToTile(tileFurthestFromEnemy)
-    }
-
-
-    private fun countDistanceToClosestEnemy(unit: MapUnit, tile: Tile): Int {
-        for (i in 1..3)
-            if (tile.getTilesAtDistance(i).any { containsEnemyMilitaryUnit(unit, it) })
-                return i
-        return 4
-    }
-
-    private fun containsEnemyMilitaryUnit(unit: MapUnit, tile: Tile) =
-        tile.militaryUnit != null
-        && tile.militaryUnit!!.civ.isAtWarWith(unit.civ)
 
 }
