@@ -1,8 +1,10 @@
 package com.unciv.logic.city
 
 import com.badlogic.gdx.math.Vector2
+import com.unciv.Constants
 import com.unciv.GUI
 import com.unciv.logic.IsPartOfGameInfoSerialization
+import com.unciv.logic.MultiFilter
 import com.unciv.logic.city.managers.CityConquestFunctions
 import com.unciv.logic.city.managers.CityEspionageManager
 import com.unciv.logic.city.managers.CityExpansionManager
@@ -10,19 +12,16 @@ import com.unciv.logic.city.managers.CityPopulationManager
 import com.unciv.logic.city.managers.CityReligionManager
 import com.unciv.logic.city.managers.SpyFleeReason
 import com.unciv.logic.civilization.Civilization
-import com.unciv.logic.civilization.diplomacy.DiplomacyFlags
 import com.unciv.logic.map.TileMap
 import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.logic.map.tile.Tile
-import com.unciv.models.Counter
-import com.unciv.models.ruleset.ModOptionsConstants
-import com.unciv.models.ruleset.tile.ResourceSupplyList
-import com.unciv.models.ruleset.tile.ResourceType
+import com.unciv.models.ruleset.Building
 import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.Unique
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.ruleset.unit.BaseUnit
+import com.unciv.models.stats.INamed
 import com.unciv.models.stats.Stat
 import java.util.UUID
 import kotlin.math.roundToInt
@@ -34,17 +33,13 @@ enum class CityFlags {
 }
 
 
-class City : IsPartOfGameInfoSerialization {
-    @Suppress("JoinDeclarationAndAssignment")
+class City : IsPartOfGameInfoSerialization, INamed {
     @Transient
     lateinit var civ: Civilization
 
     @Transient
     private lateinit var centerTile: Tile  // cached for better performance
 
-    @Transient
-    val range = 2
-
     @Transient
     lateinit var tileMap: TileMap
 
@@ -57,7 +52,7 @@ class City : IsPartOfGameInfoSerialization {
 
     var location: Vector2 = Vector2.Zero
     var id: String = UUID.randomUUID().toString()
-    var name: String = ""
+    override var name: String = ""
     var foundingCiv = ""
     // This is so that cities in resistance that are recaptured aren't in resistance anymore
     var previousOwner = ""
@@ -88,7 +83,18 @@ class City : IsPartOfGameInfoSerialization {
     var hasSoldBuildingThisTurn = false
     var isPuppet = false
     var updateCitizens = false  // flag so that on startTurn() the Governor reassigns Citizens
-    var cityAIFocus: CityFocus = CityFocus.NoFocus
+
+    @delegate:Transient
+    val neighboringCities: List<City> by lazy { 
+        civ.gameInfo.getCities().filter { it != this && it.getCenterTile().isExplored(civ) && it.getCenterTile().aerialDistanceTo(getCenterTile()) <= 12 }.toList()
+    }
+
+    var cityAIFocus: String = CityFocus.NoFocus.name
+    fun getCityFocus() = CityFocus.values().firstOrNull { it.name == cityAIFocus } ?: CityFocus.NoFocus
+    fun setCityFocus(cityFocus: CityFocus){ cityAIFocus = cityFocus.name }
+
+
+
     var avoidGrowth: Boolean = false
     @Transient var currentGPPBonus: Int = 0  // temporary variable saved for rankSpecialist()
 
@@ -150,13 +156,16 @@ class City : IsPartOfGameInfoSerialization {
     fun isCapital(): Boolean = cityConstructions.getBuiltBuildings().any { it.hasUnique(UniqueType.IndicatesCapital) }
     fun isCoastal(): Boolean = centerTile.isCoastalTile()
 
-    fun capitalCityIndicator(): String {
-        val indicatorBuildings = getRuleset().buildings.values
-            .asSequence()
+    fun getBombardRange(): Int = civ.gameInfo.ruleset.modOptions.constants.baseCityBombardRange
+    fun getWorkRange(): Int = civ.gameInfo.ruleset.modOptions.constants.cityWorkRange
+    fun getExpandRange(): Int = civ.gameInfo.ruleset.modOptions.constants.cityExpandRange
+
+    fun capitalCityIndicator(): Building? {
+        val indicatorBuildings = getRuleset().buildings.values.asSequence()
             .filter { it.hasUnique(UniqueType.IndicatesCapital) }
 
         val civSpecificBuilding = indicatorBuildings.firstOrNull { it.uniqueTo == civ.civName }
-        return civSpecificBuilding?.name ?: indicatorBuildings.first().name
+        return civSpecificBuilding ?: indicatorBuildings.firstOrNull()
     }
 
     fun isConnectedToCapital(connectionTypePredicate: (Set<String>) -> Boolean = { true }): Boolean {
@@ -176,107 +185,17 @@ class City : IsPartOfGameInfoSerialization {
     fun isWeLoveTheKingDayActive() = hasFlag(CityFlags.WeLoveTheKing)
     fun isInResistance() = hasFlag(CityFlags.Resistance)
     fun isBlockaded(): Boolean {
-
-        // Landlocked cities are not blockaded
-        if (!isCoastal())
-            return false
-
         // Coastal cities are blocked if every adjacent water tile is blocked
-        for (tile in getCenterTile().neighbors) {
-
-            // Consider only water tiles
-            if (!tile.isWater)
-                continue
-
-            // One unblocked tile breaks whole city blockade
-            if (!tile.isBlockaded())
-                return false
+        if (!isCoastal()) return false
+        return getCenterTile().neighbors.filter { it.isWater }.all {
+            it.isBlockaded()
         }
-
-        // All tiles are blocked
-        return true
     }
 
     fun getRuleset() = civ.gameInfo.ruleset
 
-    fun getCityResources(): ResourceSupplyList {
-        val cityResources = ResourceSupplyList()
-
-        for (tileInfo in getTiles().filter { it.resource != null }) {
-            val resource = tileInfo.tileResource
-            val amount = getTileResourceAmount(tileInfo)
-            if (amount > 0) cityResources.add(resource, "Tiles", amount)
-        }
-
-        for (tileInfo in getTiles()) {
-            val stateForConditionals = StateForConditionals(civ, this, tile = tileInfo)
-            if (tileInfo.getUnpillagedImprovement() == null) continue
-            val tileImprovement = tileInfo.getUnpillagedTileImprovement()
-            for (unique in tileImprovement!!.getMatchingUniques(UniqueType.ProvidesResources, stateForConditionals)) {
-                val resource = getRuleset().tileResources[unique.params[1]] ?: continue
-                cityResources.add(
-                    resource, "Improvements",
-                    unique.params[0].toInt()
-                )
-            }
-            for (unique in tileImprovement.getMatchingUniques(UniqueType.ConsumesResources, stateForConditionals)) {
-                val resource = getRuleset().tileResources[unique.params[1]] ?: continue
-                cityResources.add(
-                    resource, "Improvements",
-                    -1 * unique.params[0].toInt()
-                )
-            }
-        }
-
-        val freeBuildings = civ.civConstructions.getFreeBuildings(id)
-        for (building in cityConstructions.getBuiltBuildings()) {
-            // Free buildings cost no resources
-            if (building.name in freeBuildings) continue
-            cityResources.subtractResourceRequirements(building.getResourceRequirementsPerTurn(), getRuleset(), "Buildings")
-        }
-
-        for (unique in getLocalMatchingUniques(UniqueType.ProvidesResources, StateForConditionals(civ, this))) { // E.G "Provides [1] [Iron]"
-            val resource = getRuleset().tileResources[unique.params[1]]
-                ?: continue
-            cityResources.add(
-                resource, "Buildings",
-                unique.params[0].toInt()
-            )
-        }
-
-        if (civ.isCityState() && isCapital() && civ.cityStateResource != null) {
-            cityResources.add(
-                getRuleset().tileResources[civ.cityStateResource]!!,
-                "Mercantile City-State"
-            )
-        }
-
-        return cityResources
-    }
-
-    /** Gets the number of resources available to this city
-     * Accommodates both city-wide and civ-wide resources */
-    fun getResourceAmount(resourceName: String): Int {
-        val resource = getRuleset().tileResources[resourceName] ?: return 0
-
-        if (resource.hasUnique(UniqueType.CityResource))
-            return getCityResources().asSequence().filter { it.resource == resource }.sumOf { it.amount }
-        return civ.getResourceAmount(resourceName)
-    }
-
-    private fun getTileResourceAmount(tile: Tile): Int {
-        if (tile.resource == null) return 0
-        if (!tile.providesResources(civ)) return 0
-
-        val resource = tile.tileResource
-        var amountToAdd = if (resource.resourceType == ResourceType.Strategic) tile.resourceAmount
-            else 1
-        if (resource.resourceType == ResourceType.Luxury
-            && containsBuildingUnique(UniqueType.ProvidesExtraLuxuryFromCityResources))
-            amountToAdd += 1
-
-        return amountToAdd
-    }
+    fun getResourcesGeneratedByCity() = CityResources.getResourcesGeneratedByCity(this)
+    fun getAvailableResourceAmount(resourceName: String) = CityResources.getAvailableResourceAmount(this, resourceName)
 
     fun isGrowing() = foodForNextTurn() > 0
     fun isStarving() = foodForNextTurn() < 0
@@ -284,68 +203,11 @@ class City : IsPartOfGameInfoSerialization {
     fun foodForNextTurn() = cityStats.currentCityStats.food.roundToInt()
 
 
-    fun containsBuildingUnique(uniqueType: UniqueType) =
-        cityConstructions.builtBuildingUniqueMap.getUniques(uniqueType).any()
+    fun containsBuildingUnique(uniqueType: UniqueType, state: StateForConditionals = StateForConditionals(this)) =
+        cityConstructions.builtBuildingUniqueMap.getMatchingUniques(uniqueType, state).any()
 
-    fun getGreatPersonPercentageBonus(): Int{
-        var allGppPercentageBonus = 0
-        for (unique in getMatchingUniques(UniqueType.GreatPersonPointPercentage)) {
-            if (!matchesFilter(unique.params[1])) continue
-            allGppPercentageBonus += unique.params[0].toInt()
-        }
-
-        // Sweden UP
-        for (otherCiv in civ.getKnownCivs()) {
-            if (!civ.getDiplomacyManager(otherCiv).hasFlag(DiplomacyFlags.DeclarationOfFriendship))
-                continue
-
-            for (ourUnique in civ.getMatchingUniques(UniqueType.GreatPersonBoostWithFriendship))
-                allGppPercentageBonus += ourUnique.params[0].toInt()
-            for (theirUnique in otherCiv.getMatchingUniques(UniqueType.GreatPersonBoostWithFriendship))
-                allGppPercentageBonus += theirUnique.params[0].toInt()
-        }
-        return allGppPercentageBonus
-    }
-
-    fun getGreatPersonPointsForNextTurn(): HashMap<String, Counter<String>> {
-        val sourceToGPP = HashMap<String, Counter<String>>()
-
-        val specialistsCounter = Counter<String>()
-        for ((specialistName, amount) in population.getNewSpecialists())
-            if (getRuleset().specialists.containsKey(specialistName)) { // To solve problems in total remake mods
-                val specialist = getRuleset().specialists[specialistName]!!
-                specialistsCounter.add(specialist.greatPersonPoints.times(amount))
-            }
-        sourceToGPP["Specialists"] = specialistsCounter
-
-        val buildingsCounter = Counter<String>()
-        for (building in cityConstructions.getBuiltBuildings())
-            buildingsCounter.add(building.greatPersonPoints)
-        sourceToGPP["Buildings"] = buildingsCounter
-
-        val stateForConditionals = StateForConditionals(civInfo = civ, city = this)
-        for ((_, gppCounter) in sourceToGPP) {
-            for (unique in civ.getMatchingUniques(UniqueType.GreatPersonEarnedFaster, stateForConditionals)) {
-                val unitName = unique.params[0]
-                if (!gppCounter.containsKey(unitName)) continue
-                gppCounter.add(unitName, gppCounter[unitName] * unique.params[1].toInt() / 100)
-            }
-
-            val allGppPercentageBonus = getGreatPersonPercentageBonus()
-
-            for (unitName in gppCounter.keys)
-                gppCounter.add(unitName, gppCounter[unitName] * allGppPercentageBonus / 100)
-        }
-
-        return sourceToGPP
-    }
-
-    fun getGreatPersonPoints(): Counter<String> {
-        val gppCounter = Counter<String>()
-        for (entry in getGreatPersonPointsForNextTurn().values)
-            gppCounter.add(entry)
-        return gppCounter
-    }
+    fun getGreatPersonPercentageBonus() = GreatPersonPointsBreakdown.getGreatPersonPercentageBonus(this)
+    fun getGreatPersonPoints() = GreatPersonPointsBreakdown(this).sum()
 
     fun addStat(stat: Stat, amount: Int) {
         when (stat) {
@@ -357,14 +219,25 @@ class City : IsPartOfGameInfoSerialization {
 
     fun getStatReserve(stat: Stat): Int {
         return when (stat) {
+            Stat.Production -> cityConstructions.getWorkDone(cityConstructions.getCurrentConstruction().name)
             Stat.Food -> population.foodStored
             else -> civ.getStatReserve(stat)
         }
     }
 
+    fun hasStatToBuy(stat: Stat, price: Int): Boolean {
+        return when {
+            civ.gameInfo.gameParameters.godMode -> true
+            price == 0 -> true
+            else -> getStatReserve(stat) >= price
+        }
+    }
+
     internal fun getMaxHealth() =
         200 + cityConstructions.getBuiltBuildings().sumOf { it.cityHealth }
 
+    fun getStrength() = cityConstructions.getBuiltBuildings().sumOf { it.cityStrength }.toFloat()
+
     override fun toString() = name // for debug
 
     fun isHolyCity(): Boolean = religion.religionThisIsTheHolyCityOf != null && !religion.isBlockedHolyCity
@@ -373,8 +246,8 @@ class City : IsPartOfGameInfoSerialization {
     fun canBeDestroyed(justCaptured: Boolean = false): Boolean {
         if (civ.gameInfo.gameParameters.noCityRazing) return false
 
-        val allowRazeCapital = civ.gameInfo.ruleset.modOptions.uniques.contains(ModOptionsConstants.allowRazeCapital)
-        val allowRazeHolyCity = civ.gameInfo.ruleset.modOptions.uniques.contains(ModOptionsConstants.allowRazeHolyCity)
+        val allowRazeCapital = civ.gameInfo.ruleset.modOptions.hasUnique(UniqueType.AllowRazeCapital)
+        val allowRazeHolyCity = civ.gameInfo.ruleset.modOptions.hasUnique(UniqueType.AllowRazeHolyCity)
 
         if (isOriginalCapital && !allowRazeCapital) return false
         if (isHolyCity() && !allowRazeHolyCity) return false
@@ -390,7 +263,7 @@ class City : IsPartOfGameInfoSerialization {
         this.civ = civInfo
         tileMap = civInfo.gameInfo.tileMap
         centerTile = tileMap[location]
-        tilesInRange = getCenterTile().getTilesInDistance(3).toHashSet()
+        tilesInRange = getCenterTile().getTilesInDistance(getWorkRange()).toHashSet()
         population.city = this
         expansion.city = this
         expansion.setTransients()
@@ -431,7 +304,7 @@ class City : IsPartOfGameInfoSerialization {
         if (resetLocked) {
             workedTiles = hashSetOf()
             lockedTiles = hashSetOf()
-        } else {
+        } else if(cityAIFocus != CityFocus.Manual.name){
             workedTiles = lockedTiles
         }
         if (!manualSpecialists)
@@ -470,7 +343,7 @@ class City : IsPartOfGameInfoSerialization {
         if (isCapital()) civ.moveCapitalToNextLargest(null)
 
         civ.cities = civ.cities.toMutableList().apply { remove(this@City) }
-        getCenterTile().changeImprovement("City ruins")
+        getCenterTile().setImprovement("City ruins")
 
         // Edge case! What if a water unit is in a city, and you raze the city?
         // Well, the water unit has to return to the water!
@@ -509,11 +382,11 @@ class City : IsPartOfGameInfoSerialization {
 
     internal fun tryUpdateRoadStatus() {
         if (getCenterTile().roadStatus == RoadStatus.None) {
-            val roadImprovement = RoadStatus.Road.improvement(getRuleset())
+            val roadImprovement = getRuleset().roadImprovement
             if (roadImprovement != null && roadImprovement.techRequired in civ.tech.techsResearched)
                 getCenterTile().roadStatus = RoadStatus.Road
         } else if (getCenterTile().roadStatus != RoadStatus.Railroad) {
-            val railroadImprovement = RoadStatus.Railroad.improvement(getRuleset())
+            val railroadImprovement = getRuleset().railroadImprovement
             if (railroadImprovement != null && railroadImprovement.techRequired in civ.tech.techsResearched)
                 getCenterTile().roadStatus = RoadStatus.Railroad
         }
@@ -523,8 +396,12 @@ class City : IsPartOfGameInfoSerialization {
         getRuleset().buildings[buildingName]!!.cost / 10
 
     fun sellBuilding(buildingName: String) {
-        cityConstructions.removeBuilding(buildingName)
-        civ.addGold(getGoldForSellingBuilding(buildingName))
+        sellBuilding(getRuleset().buildings[buildingName]!!)
+    }
+
+    fun sellBuilding(building: Building) {
+        cityConstructions.removeBuilding(building)
+        civ.addGold(getGoldForSellingBuilding(building.name))
         hasSoldBuildingThisTurn = true
 
         population.unassignExtraPopulation() // If the building provided specialists, release them to other work
@@ -543,18 +420,23 @@ class City : IsPartOfGameInfoSerialization {
     }
 
     /** Implements [UniqueParameterType.CityFilter][com.unciv.models.ruleset.unique.UniqueParameterType.CityFilter] */
-    fun matchesFilter(filter: String, viewingCiv: Civilization = civ): Boolean {
+    fun matchesFilter(filter: String, viewingCiv: Civilization? = civ): Boolean {
+        return MultiFilter.multiFilter(filter, { matchesSingleFilter(it, viewingCiv) })
+    }
+
+    private fun matchesSingleFilter(filter: String, viewingCiv: Civilization? = civ): Boolean {
         return when (filter) {
             "in this city" -> true // Filtered by the way uniques are found
             "in all cities" -> true
-            "in other cities" -> true // Filtered by the way uniques are found
-            "in all coastal cities" -> isCoastal()
-            "in capital" -> isCapital()
-            "in all non-occupied cities" -> !cityStats.hasExtraAnnexUnhappiness() || isPuppet
+            in Constants.all -> true
+            "in your cities", "Your" -> viewingCiv == civ
+            "in all coastal cities", "Coastal" -> isCoastal()
+            "in capital", "Capital" -> isCapital()
+            "in all non-occupied cities", "Non-occupied" -> !cityStats.hasExtraAnnexUnhappiness() || isPuppet
             "in all cities with a world wonder" -> cityConstructions.getBuiltBuildings()
                 .any { it.isWonder }
             "in all cities connected to capital" -> isConnectedToCapital()
-            "in all cities with a garrison" -> isGarrisoned()
+            "in all cities with a garrison", "Garrisoned" -> isGarrisoned()
             "in all cities in which the majority religion is a major religion" ->
                 religion.getMajorityReligionName() != null
                 && religion.getMajorityReligion()!!.isMajorReligion()
@@ -562,18 +444,22 @@ class City : IsPartOfGameInfoSerialization {
                 religion.getMajorityReligionName() != null
                 && religion.getMajorityReligion()!!.isEnhancedReligion()
             "in non-enemy foreign cities" ->
-                viewingCiv != civ
+                viewingCiv != null && viewingCiv != civ
                 && !civ.isAtWarWith(viewingCiv)
-            "in foreign cities" -> viewingCiv != civ
-            "in annexed cities" -> foundingCiv != civ.civName && !isPuppet
-            "in puppeted cities" -> isPuppet
-            "in holy cities" -> isHolyCity()
+            "in enemy cities", "Enemy" -> civ.isAtWarWith(viewingCiv ?: civ)
+            "in foreign cities", "Foreign" -> viewingCiv != null && viewingCiv != civ
+            "in annexed cities", "Annexed" -> foundingCiv != civ.civName && !isPuppet
+            "in puppeted cities", "Puppeted" -> isPuppet
+            "in resisting cities", "Resisting" -> isInResistance()
+            "in cities being razed", "Razing" -> isBeingRazed
+            "in holy cities", "Holy" -> isHolyCity()
             "in City-State cities" -> civ.isCityState()
             // This is only used in communication to the user indicating that only in cities with this
             // religion a unique is active. However, since religion uniques only come from the city itself,
             // this will always be true when checked.
             "in cities following this religion" -> true
-            else -> false
+            "in cities following our religion" -> viewingCiv?.religionManager?.religion == religion.getMajorityReligion()
+            else -> civ.matchesFilter(filter)
         }
     }
 
@@ -588,26 +474,35 @@ class City : IsPartOfGameInfoSerialization {
     // Finds matching uniques provided from both local and non-local sources.
     fun getMatchingUniques(
         uniqueType: UniqueType,
-        stateForConditionals: StateForConditionals = StateForConditionals(civ, this)
+        stateForConditionals: StateForConditionals = StateForConditionals(this),
+        includeCivUniques: Boolean = true
     ): Sequence<Unique> {
-        return civ.getMatchingUniques(uniqueType, stateForConditionals, this) +
+        return if (includeCivUniques)
+            civ.getMatchingUniques(uniqueType, stateForConditionals) +
                 getLocalMatchingUniques(uniqueType, stateForConditionals)
+        else (
+            cityConstructions.builtBuildingUniqueMap.getUniques(uniqueType)
+                + religion.getUniques().filter { it.type == uniqueType }
+            ).filter {
+                !it.isTimedTriggerable && it.conditionalsApply(stateForConditionals)
+            }.flatMap { it.getMultiplied(stateForConditionals) }
     }
 
-    fun getLocalMatchingUniques(uniqueType: UniqueType, stateForConditionals: StateForConditionals = StateForConditionals(civ, this)): Sequence<Unique> {
-        return (
-            cityConstructions.builtBuildingUniqueMap.getUniques(uniqueType).filter { !it.isAntiLocalEffect }
-            + religion.getUniques().filter { it.isOfType(uniqueType) }
-        ).filter {
-            it.conditionalsApply(stateForConditionals)
-        }
+    // Uniques special to this city
+    fun getLocalMatchingUniques(uniqueType: UniqueType, stateForConditionals: StateForConditionals = StateForConditionals(this)): Sequence<Unique> {
+        val uniques = cityConstructions.builtBuildingUniqueMap.getUniques(uniqueType).filter { it.isLocalEffect } +
+            religion.getUniques().filter { it.type == uniqueType }
+        return if (uniques.any()) uniques.filter { !it.isTimedTriggerable && it.conditionalsApply(stateForConditionals) }
+            .flatMap { it.getMultiplied(stateForConditionals) }
+        else uniques
     }
 
-
+    // Uniques coming from this city, but that should be provided globally
     fun getMatchingUniquesWithNonLocalEffects(uniqueType: UniqueType, stateForConditionals: StateForConditionals): Sequence<Unique> {
         val uniques = cityConstructions.builtBuildingUniqueMap.getUniques(uniqueType)
         // Memory performance showed that this function was very memory intensive, thus we only create the filter if needed
-        return if (uniques.any()) uniques.filter { !it.isLocalEffect && it.conditionalsApply(stateForConditionals) }
+        return if (uniques.any()) uniques.filter { !it.isLocalEffect && !it.isTimedTriggerable
+            && it.conditionalsApply(stateForConditionals) }.flatMap { it.getMultiplied(stateForConditionals) }
         else uniques
     }
 
