@@ -30,16 +30,15 @@ import com.unciv.models.ruleset.unit.BaseUnit
 import com.unciv.models.stats.Stat
 import com.unciv.models.stats.Stats
 import com.unciv.models.translations.tr
-import com.unciv.ui.components.Fonts
 import com.unciv.ui.components.extensions.withItem
 import com.unciv.ui.components.extensions.withoutItem
+import com.unciv.ui.components.fonts.Fonts
 import com.unciv.ui.screens.civilopediascreen.CivilopediaCategories
 import com.unciv.ui.screens.civilopediascreen.FormattedLine
 import kotlin.math.ceil
 import kotlin.math.min
 import kotlin.math.roundToInt
 
-
 /**
  * City constructions manager.
  *
@@ -79,7 +78,9 @@ class CityConstructions : IsPartOfGameInfoSerialization {
     var productionOverflow = 0
     private val queueMaxSize = 10
 
-    // Maps cities to the buildings they received
+    /** Maps cities by id to a set of the buildings they received (by nation equivalent name)
+     *  Source: [UniqueType.GainFreeBuildings]
+     */
     val freeBuildingsProvidedFromThisCity: HashMap<String, HashSet<String>> = hashMapOf()
 
     //endregion
@@ -103,16 +104,11 @@ class CityConstructions : IsPartOfGameInfoSerialization {
     fun getConstructableUnits() = city.getRuleset().units.values
         .asSequence().filter { it.isBuildable(this) }
 
-    private fun getBasicStatBuildings(stat: Stat) = city.getRuleset().buildings.values
-        .asSequence()
-        .filter { !it.isAnyWonder() && it.replaces == null && it[stat] > 0f }
-
     /**
      * @return [Stats] provided by all built buildings in city plus the bonus from Library
      */
-    fun getStats(): StatTreeNode {
+    fun getStats(localUniqueCache: LocalUniqueCache): StatTreeNode {
         val stats = StatTreeNode()
-        val localUniqueCache = LocalUniqueCache()
         for (building in getBuiltBuildings())
             stats.addStats(building.getStats(city, localUniqueCache), building.name)
         return stats
@@ -123,7 +119,7 @@ class CityConstructions : IsPartOfGameInfoSerialization {
      */
     fun getMaintenanceCosts(): Int {
         var maintenanceCost = 0
-        val freeBuildings = city.civ.civConstructions.getFreeBuildings(city.id)
+        val freeBuildings = city.civ.civConstructions.getFreeBuildingNames(city)
 
         for (building in getBuiltBuildings())
             if (building.name !in freeBuildings)
@@ -143,13 +139,16 @@ class CityConstructions : IsPartOfGameInfoSerialization {
         return result
     }
 
-    /** @constructionName needs to be a non-perpetual construction, else an empty string is returned */
-    internal fun getTurnsToConstructionString(constructionName: String, useStoredProduction:Boolean = true): String {
-        val construction = getConstruction(constructionName)
+    /** @param constructionName needs to be a non-perpetual construction, else an empty string is returned */
+    internal fun getTurnsToConstructionString(constructionName: String, useStoredProduction: Boolean = true) =
+        getTurnsToConstructionString(getConstruction(constructionName), useStoredProduction)
+
+    /** @param construction needs to be a non-perpetual construction, else an empty string is returned */
+    internal fun getTurnsToConstructionString(construction: IConstruction, useStoredProduction: Boolean = true): String {
         if (construction !is INonPerpetualConstruction) return ""   // shouldn't happen
-        val cost = construction.getProductionCost(city.civ)
-        val turnsToConstruction = turnsToConstruction(constructionName, useStoredProduction)
-        val currentProgress = if (useStoredProduction) getWorkDone(constructionName) else 0
+        val cost = construction.getProductionCost(city.civ, city)
+        val turnsToConstruction = turnsToConstruction(construction.name, useStoredProduction)
+        val currentProgress = if (useStoredProduction) getWorkDone(construction.name) else 0
         val lines = ArrayList<String>()
         val buildable = !construction.getMatchingUniques(UniqueType.Unbuildable)
             .any { it.conditionalsApply(StateForConditionals(city.civ, city)) }
@@ -190,12 +189,23 @@ class CityConstructions : IsPartOfGameInfoSerialization {
     fun isAllBuilt(buildingList: List<String>): Boolean = buildingList.all { isBuilt(it) }
 
     fun isBuilt(buildingName: String): Boolean = builtBuildingObjects.any { it.name == buildingName }
-    @Suppress("MemberVisibilityCanBePrivate")
-    fun isBeingConstructed(constructionName: String): Boolean = currentConstructionFromQueue == constructionName
-    fun isEnqueued(constructionName: String): Boolean = constructionQueue.contains(constructionName)
-    fun isBeingConstructedOrEnqueued(constructionName: String): Boolean = isBeingConstructed(constructionName) || isEnqueued(constructionName)
 
-    fun isQueueFull(): Boolean = constructionQueue.size == queueMaxSize
+    // Note: There was a isEnqueued here functionally identical to isBeingConstructedOrEnqueued,
+    // which was calling both isEnqueued and isBeingConstructed - BUT: currentConstructionFromQueue is just a
+    // a wrapper for constructionQueue[0], so that was redundant. Also, isEnqueued was used nowhere,
+    // and isBeingConstructed _only_ redundantly as described above.
+    // `isEnqueuedForLater` is not optimal code as it can iterate the whole list where checking size
+    // and first() would suffice, but the one current use in CityScreenConstructionMenu isn't critical.
+
+    @Suppress("unused", "MemberVisibilityCanBePrivate")  // kept for illustration
+    /** @return `true` if [constructionName] is the top queue entry, the one receiving production points */
+    fun isBeingConstructed(constructionName: String) = currentConstructionFromQueue == constructionName
+    /** @return `true` if [constructionName] is queued but not the top queue entry */
+    fun isEnqueuedForLater(constructionName: String) = constructionQueue.indexOf(constructionName) > 0
+    /** @return `true` if [constructionName] is anywhere in the construction queue - [isBeingConstructed] **or** [isEnqueuedForLater] */
+    fun isBeingConstructedOrEnqueued(constructionName: String) = constructionQueue.contains(constructionName)
+
+    fun isQueueFull(): Boolean = constructionQueue.size >= queueMaxSize
 
     fun isBuildingWonder(): Boolean {
         val currentConstruction = getCurrentConstruction()
@@ -209,15 +219,15 @@ class CityConstructions : IsPartOfGameInfoSerialization {
 
     /** If the city is constructing multiple units of the same type, subsequent units will require the full cost  */
     fun isFirstConstructionOfItsKind(constructionQueueIndex: Int, name: String): Boolean {
-        // if the construction name is the same as the current construction, it isn't the first
-        return constructionQueueIndex == constructionQueue.indexOfFirst { it == name }
+        // Simply compare index of first found [name] with given index
+        return constructionQueueIndex == constructionQueue.indexOf(name)
     }
 
 
     internal fun getConstruction(constructionName: String): IConstruction {
         val gameBasics = city.getRuleset()
         when {
-            constructionName == "" -> return getConstruction("Nothing")
+            constructionName == "" -> return PerpetualConstruction.idle
             gameBasics.buildings.containsKey(constructionName) -> return gameBasics.buildings[constructionName]!!
             gameBasics.units.containsKey(constructionName) -> return gameBasics.units[constructionName]!!
             else -> {
@@ -230,7 +240,7 @@ class CityConstructions : IsPartOfGameInfoSerialization {
         throw NotBuildingOrUnitException("$constructionName is not a building or a unit!")
     }
 
-    internal fun getBuiltBuildings(): Sequence<Building> = builtBuildingObjects.asSequence()
+    fun getBuiltBuildings(): Sequence<Building> = builtBuildingObjects.asSequence()
 
     fun containsBuildingOrEquivalent(buildingNameOrUnique: String): Boolean =
             isBuilt(buildingNameOrUnique) || getBuiltBuildings().any { it.replaces == buildingNameOrUnique || it.hasUnique(buildingNameOrUnique) }
@@ -244,8 +254,8 @@ class CityConstructions : IsPartOfGameInfoSerialization {
         val constr = getConstruction(constructionName)
         return when {
             constr is PerpetualConstruction -> 0
-            useStoredProduction -> (constr as INonPerpetualConstruction).getProductionCost(city.civ) - getWorkDone(constructionName)
-            else -> (constr as INonPerpetualConstruction).getProductionCost(city.civ)
+            useStoredProduction -> (constr as INonPerpetualConstruction).getProductionCost(city.civ, city) - getWorkDone(constructionName)
+            else -> (constr as INonPerpetualConstruction).getProductionCost(city.civ, city)
         }
     }
 
@@ -257,6 +267,10 @@ class CityConstructions : IsPartOfGameInfoSerialization {
         if (workLeft <= productionOverflow) // if we already have stored up enough production to finish it directly
             return 1 // we'll finish this next turn
 
+        return ceil((workLeft-productionOverflow) / productionForConstruction(constructionName).toDouble()).toInt()
+    }
+
+    fun productionForConstruction(constructionName: String): Int {
         val cityStatsForConstruction: Stats
         if (currentConstructionFromQueue == constructionName) cityStatsForConstruction = city.cityStats.currentCityStats
         else {
@@ -278,16 +292,14 @@ class CityConstructions : IsPartOfGameInfoSerialization {
             cityStatsForConstruction = cityStats.currentCityStats
         }
 
-        val production = cityStatsForConstruction.production.roundToInt()
-
-        return ceil((workLeft-productionOverflow) / production.toDouble()).toInt()
+        return cityStatsForConstruction.production.roundToInt()
     }
 
-    fun hasBuildableStatBuildings(stat: Stat): Boolean {
-        return getBasicStatBuildings(stat)
-            .map { city.civ.getEquivalentBuilding(it.name) }
-            .filter { it.isBuildable(this) || isBeingConstructedOrEnqueued(it.name) }
-            .any()
+    fun cheapestStatBuilding(stat: Stat): Building? {
+        return city.getRuleset().buildings.values
+            .filter { !it.isAnyWonder() && it.isStatRelated(stat) &&
+                (it.isBuildable(this) || isBeingConstructedOrEnqueued(it.name)) }
+            .minByOrNull { it.cost }
     }
 
     //endregion
@@ -321,11 +333,11 @@ class CityConstructions : IsPartOfGameInfoSerialization {
         val construction = getConstruction(currentConstructionFromQueue)
         if (construction is PerpetualConstruction) chooseNextConstruction() // check every turn if we could be doing something better, because this doesn't end by itself
         else {
-            val productionCost = (construction as INonPerpetualConstruction).getProductionCost(city.civ)
+            val productionCost = (construction as INonPerpetualConstruction).getProductionCost(city.civ, city)
             if (inProgressConstructions.containsKey(currentConstructionFromQueue)
                     && inProgressConstructions[currentConstructionFromQueue]!! >= productionCost) {
                 val potentialOverflow = inProgressConstructions[currentConstructionFromQueue]!! - productionCost
-                if (constructionComplete(construction)){
+                if (completeConstruction(construction)) {
                     // See the URL below for explanation for this cap
                     // https://forums.civfanatics.com/threads/hammer-overflow.419352/
                     val maxOverflow = maxOf(productionCost, city.cityStats.currentCityStats.production.roundToInt())
@@ -335,6 +347,7 @@ class CityConstructions : IsPartOfGameInfoSerialization {
                     city.civ.addNotification("No space available to place [${construction.name}] near [${city.name}]",
                         city.location, NotificationCategory.Production, construction.name)
                 }
+                city.civ.civConstructions.builtItemsWithIncreasingCost[construction.name] += 1
             }
         }
     }
@@ -373,9 +386,10 @@ class CityConstructions : IsPartOfGameInfoSerialization {
             if (construction.isBuildable(this))
                 constructionQueue.add(constructionName)
         }
+        chooseNextConstruction()
     }
 
-    private fun validateInProgressConstructions() {
+    fun validateInProgressConstructions() {
         // remove obsolete stuff from in progress constructions - happens often and leaves clutter in memory and save files
         // should have little visible consequences - any accumulated points that may be reused later should stay (nukes when manhattan project city lost, nat wonder when conquered an empty city...), all other points should be refunded
         // Should at least be called before each turn - if another civ completes a wonder after our previous turn, we should get the refund this turn
@@ -402,9 +416,12 @@ class CityConstructions : IsPartOfGameInfoSerialization {
                     }
                 } else if (construction is BaseUnit) {
                     // Production put into upgradable units gets put into upgraded version
-                    if (rejectionReasons.all { it.type == RejectionReasonType.Obsoleted } && construction.upgradesTo != null) {
-                        val upgradedUnitName = city.civ.getEquivalentUnit(construction.upgradesTo!!).name
-                        inProgressConstructions[upgradedUnitName] = (inProgressConstructions[upgradedUnitName] ?: 0) + workDone
+                    val cheapestUpgradeUnit = construction.getRulesetUpgradeUnits(StateForConditionals(city.civ, city))
+                        .map { city.civ.getEquivalentUnit(it) }
+                        .filter { it.isBuildable(this) }
+                        .minByOrNull { it.cost }
+                    if (rejectionReasons.all { it.type == RejectionReasonType.Obsoleted } && cheapestUpgradeUnit != null) {
+                        inProgressConstructions[cheapestUpgradeUnit.name] = (inProgressConstructions[cheapestUpgradeUnit.name] ?: 0) + workDone
                     }
                 }
                 inProgressConstructions.remove(constructionName)
@@ -415,7 +432,7 @@ class CityConstructions : IsPartOfGameInfoSerialization {
     private fun constructionBegun(construction: IConstruction) {
         val costUniques = construction.getMatchingUniquesNotConflicting(UniqueType.CostsResources)
 
-        for (unique in costUniques){
+        for (unique in costUniques) {
             val amount = unique.params[0].toInt()
             val resourceName = unique.params[1]
             city.civ.resourceStockpiles.add(resourceName, -amount)
@@ -440,7 +457,7 @@ class CityConstructions : IsPartOfGameInfoSerialization {
     }
 
     /** Returns false if we tried to construct a unit but it has nowhere to go */
-    private fun constructionComplete(construction: INonPerpetualConstruction): Boolean {
+    fun completeConstruction(construction: INonPerpetualConstruction): Boolean {
         val managedToConstruct = construction.postBuildEvent(this)
         if (!managedToConstruct) return false
 
@@ -492,6 +509,13 @@ class CityConstructions : IsPartOfGameInfoSerialization {
 
     fun addBuilding(buildingName: String) {
         val building = city.getRuleset().buildings[buildingName]!!
+        addBuilding(building)
+    }
+
+    fun addBuilding(building: Building,
+                    /** False when creating initial buildings in city - so we don't "waste" a free building on a building we're going to get anyway, from settler buildings */
+                    tryAddFreeBuildings: Boolean = true) {
+        val buildingName = building.name
         val civ = city.civ
 
         if (building.cityHealth > 0) {
@@ -502,57 +526,50 @@ class CityConstructions : IsPartOfGameInfoSerialization {
         builtBuildingObjects = builtBuildingObjects.withItem(building)
         builtBuildings.add(buildingName)
 
-        city.civ.cache.updateCitiesConnectedToCapital(false) // could be a connecting building, like a harbor
+        updateUniques()
 
         /** Support for [UniqueType.CreatesOneImprovement] */
         applyCreateOneImprovement(building)
 
-
-
         triggerNewBuildingUniques(building)
 
         if (building.hasUnique(UniqueType.EnemyUnitsSpendExtraMovement))
             civ.cache.updateHasActiveEnemyMovementPenalty()
 
         // Korean unique - apparently gives the same as the research agreement
-        if (building.isStatRelated(Stat.Science) && civ.hasUnique(UniqueType.TechBoostWhenScientificBuildingsBuiltInCapital))
+        if (building.isStatRelated(Stat.Science) && civ.hasUnique(UniqueType.TechBoostWhenScientificBuildingsBuiltInCapital)
+            && city.isCapital())
             civ.tech.addScience(civ.tech.scienceOfLast8Turns.sum() / 8)
 
-        val uniqueTypesModifyingYields = listOf(
-            UniqueType.StatsFromTiles, UniqueType.StatsFromTilesWithout, UniqueType.StatsFromObject,
-            UniqueType.StatPercentFromObject, UniqueType.AllStatsPercentFromObject
-        )
-
-        updateUniques()
-
         // Happiness is global, so it could affect all cities
         if (building.isStatRelated(Stat.Happiness)) {
             for (city in civ.cities) {
                 city.reassignPopulationDeferred()
             }
         }
-        else if(uniqueTypesModifyingYields.any { building.hasUnique(it) })
-            city.reassignPopulationDeferred()
+        else city.reassignPopulationDeferred()
 
-        addFreeBuildings()
+        if (tryAddFreeBuildings)
+            city.civ.civConstructions.tryAddFreeBuildings()
     }
 
     fun triggerNewBuildingUniques(building: Building) {
+        val stateForConditionals = StateForConditionals(city.civ, city)
         val triggerNotificationText ="due to constructing [${building.name}]"
 
         for (unique in building.uniqueObjects)
-            if (!unique.hasTriggerConditional())
-                UniqueTriggerActivation.triggerCivwideUnique(unique, city.civ, city, triggerNotificationText = triggerNotificationText)
+            if (!unique.hasTriggerConditional() && unique.conditionalsApply(stateForConditionals))
+                UniqueTriggerActivation.triggerUnique(unique, city, triggerNotificationText = triggerNotificationText)
 
-        for (unique in city.civ.getTriggeredUniques(UniqueType.TriggerUponConstructingBuilding, StateForConditionals(city.civ, city)))
+        for (unique in city.civ.getTriggeredUniques(UniqueType.TriggerUponConstructingBuilding, stateForConditionals))
             if (unique.conditionals.any {it.type == UniqueType.TriggerUponConstructingBuilding && building.matchesFilter(it.params[0])})
-                UniqueTriggerActivation.triggerCivwideUnique(unique, city.civ, city, triggerNotificationText = triggerNotificationText)
+                UniqueTriggerActivation.triggerUnique(unique, city, triggerNotificationText = triggerNotificationText)
 
-        for (unique in city.civ.getTriggeredUniques(UniqueType.TriggerUponConstructingBuildingCityFilter, StateForConditionals(city.civ, city)))
+        for (unique in city.civ.getTriggeredUniques(UniqueType.TriggerUponConstructingBuildingCityFilter, stateForConditionals))
             if (unique.conditionals.any {it.type == UniqueType.TriggerUponConstructingBuildingCityFilter
                     && building.matchesFilter(it.params[0])
                     && city.matchesFilter(it.params[1])})
-                UniqueTriggerActivation.triggerCivwideUnique(unique, city.civ, city, triggerNotificationText = triggerNotificationText)
+                UniqueTriggerActivation.triggerUnique(unique, city, triggerNotificationText = triggerNotificationText)
     }
 
     fun removeBuilding(buildingName: String) {
@@ -561,63 +578,34 @@ class CityConstructions : IsPartOfGameInfoSerialization {
             builtBuildingObjects = builtBuildingObjects.withoutItem(buildingObject)
         else builtBuildingObjects.removeAll{ it.name == buildingName }
         builtBuildings.remove(buildingName)
-        city.civ.cache.updateCitiesConnectedToCapital(false) // could be a connecting building, like a harbor
         updateUniques()
     }
 
-    fun updateUniques(onLoadGame:Boolean = false) {
+    fun removeBuilding(building: Building) {
+        builtBuildingObjects = builtBuildingObjects.withoutItem(building)
+        builtBuildings.remove(building.name)
+        updateUniques()
+    }
+
+    fun removeBuildings(buildings: Set<Building>) {
+        val buildingsToRemove = buildings.map { it.name }.toSet()
+        builtBuildings.removeAll {
+            it in buildingsToRemove
+        }
+        setTransients()
+    }
+
+    fun updateUniques(onLoadGame: Boolean = false) {
         builtBuildingUniqueMap.clear()
         for (building in getBuiltBuildings())
             builtBuildingUniqueMap.addUniques(building.uniqueObjects)
         if (!onLoadGame) {
+            city.civ.cache.updateCitiesConnectedToCapital(false) // could be a connecting building, like a harbor
             city.cityStats.update()
             city.civ.cache.updateCivResources()
         }
     }
 
-    fun addFreeBuildings() {
-        // "Gain a free [buildingName] [cityFilter]"
-        val freeBuildingUniques = city.getLocalMatchingUniques(UniqueType.GainFreeBuildings, StateForConditionals(city.civ, city))
-
-        for (unique in freeBuildingUniques) {
-            val freeBuildingName = city.civ.getEquivalentBuilding(unique.params[0]).name
-            val citiesThatApply = when (unique.params[1]) {
-                "in this city" -> listOf(city)
-                "in other cities" -> city.civ.cities.filter { it !== city }
-                else -> city.civ.cities.filter { it.matchesFilter(unique.params[1]) }
-            }
-
-            for (city in citiesThatApply) {
-                if (city.cityConstructions.containsBuildingOrEquivalent(freeBuildingName)) continue
-                city.cityConstructions.addBuilding(freeBuildingName)
-                if (city.id !in freeBuildingsProvidedFromThisCity)
-                    freeBuildingsProvidedFromThisCity[city.id] = hashSetOf()
-
-                freeBuildingsProvidedFromThisCity[city.id]!!.add(freeBuildingName)
-            }
-        }
-
-        // Civ-level uniques - for these only add free buildings from each city to itself to avoid weirdness on city conquest
-        for (unique in city.civ.getMatchingUniques(UniqueType.GainFreeBuildings, stateForConditionals = StateForConditionals(city.civ, city))) {
-            val freeBuildingName = city.civ.getEquivalentBuilding(unique.params[0]).name
-            if (city.matchesFilter(unique.params[1])) {
-                if (city.id !in freeBuildingsProvidedFromThisCity)
-                    freeBuildingsProvidedFromThisCity[city.id] = hashSetOf()
-                freeBuildingsProvidedFromThisCity[city.id]!!.add(freeBuildingName)
-                if (!isBuilt(freeBuildingName))
-                    addBuilding(freeBuildingName)
-            }
-        }
-
-
-        val autoGrantedBuildings = city.getRuleset().buildings.values
-            .filter { it.hasUnique(UniqueType.GainBuildingWhereBuildable) }
-
-        for (building in autoGrantedBuildings)
-            if (building.isBuildable(city.cityConstructions))
-                addBuilding(building.name)
-    }
-
     /**
      *  Purchase a construction for gold (or another stat)
      *  called from NextTurnAutomation and the City UI
@@ -641,9 +629,33 @@ class CityConstructions : IsPartOfGameInfoSerialization {
         tile: Tile? = null
     ): Boolean {
         val construction = getConstruction(constructionName) as? INonPerpetualConstruction ?: return false
+        return purchaseConstruction(construction, queuePosition, automatic, stat, tile)
+    }
 
+    /**
+     *  Purchase a construction for gold (or another stat)
+     *  called from NextTurnAutomation and the City UI
+     *  Build / place the new item, deduct cost, and maintain queue.
+     *
+     *  @param construction What to buy (needed since buying something not queued is allowed)
+     *  @param queuePosition    Position in the queue or -1 if not from queue
+     *                          Note: -1 does not guarantee queue will remain unchanged (validation)
+     *  @param automatic        Flag whether automation should try to choose what next to build (not coming from UI)
+     *                          Note: settings.autoAssignCityProduction is handled later
+     *  @param stat             Stat object of the stat with which was paid for the construction
+     *  @param tile             Supports [UniqueType.CreatesOneImprovement] the tile to place the improvement from that unique on.
+     *                          Ignored when the [constructionName] does not have that unique. If null and the building has the unique, a tile is chosen automatically.
+     *  @return                 Success (false e.g. unit cannot be placed)
+     */
+    fun purchaseConstruction(
+        construction: INonPerpetualConstruction,
+        queuePosition: Int,
+        automatic: Boolean,
+        stat: Stat = Stat.Gold,
+        tile: Tile? = null
+    ): Boolean {
         // Support UniqueType.CreatesOneImprovement: it is active when getImprovementToCreate returns an improvement
-        val improvementToPlace = (construction as? Building)?.getImprovementToCreate(city.getRuleset())
+        val improvementToPlace = (construction as? Building)?.getImprovementToCreate(city.getRuleset(), city.civ)
         if (improvementToPlace != null) {
             // If active without a predetermined tile to place the improvement on, automate a tile
             val finalTile = tile
@@ -675,7 +687,7 @@ class CityConstructions : IsPartOfGameInfoSerialization {
                     && it.params[2] == stat.name
                 }
             ) {
-                city.civ.civConstructions.boughtItemsWithIncreasingPrice.add(constructionName, 1)
+                city.civ.civConstructions.boughtItemsWithIncreasingPrice.add(construction.name, 1)
             }
         }
 
@@ -686,69 +698,87 @@ class CityConstructions : IsPartOfGameInfoSerialization {
         return true
     }
 
-    fun addCheapestBuildableStatBuilding(stat: Stat): String? {
-        val cheapestBuildableStatBuilding = getBasicStatBuildings(stat)
-            .map { city.civ.getEquivalentBuilding(it.name) }
-            .filter { it.isBuildable(this) || isBeingConstructedOrEnqueued(it.name) }
-            .minByOrNull { it.cost }?.name
-            ?: return null
-
-        constructionComplete(getConstruction(cheapestBuildableStatBuilding) as INonPerpetualConstruction)
-
-        return cheapestBuildableStatBuilding
-    }
-
     private fun removeCurrentConstruction() = removeFromQueue(0, true)
 
     fun chooseNextConstruction() {
-        validateConstructionQueue()
-        if (constructionQueue.isNotEmpty()) {
-            if (currentConstructionFromQueue != ""
-                    // If the USER set a perpetual construction, then keep it!
-                    && (getConstruction(currentConstructionFromQueue) !is PerpetualConstruction || currentConstructionIsUserSet)) return
+        if (!isQueueEmptyOrIdle()) {
+            // If the USER set a perpetual construction, then keep it!
+            if (getConstruction(currentConstructionFromQueue) !is PerpetualConstruction || currentConstructionIsUserSet) return
         }
 
         val isCurrentPlayersTurn = city.civ.gameInfo.isUsersTurn()
                 || !city.civ.gameInfo.gameParameters.isOnlineMultiplayer
-        if ((UncivGame.Current.settings.autoAssignCityProduction && isCurrentPlayersTurn) // only automate if the active human player has the setting to automate production
-                || !city.civ.isHuman() || city.isPuppet) {
+        if ((isCurrentPlayersTurn && (UncivGame.Current.settings.autoAssignCityProduction
+                || UncivGame.Current.worldScreen?.autoPlay?.isAutoPlayingAndFullAutoPlayAI() == true)) // only automate if the active human player has the setting to automate production
+                || city.civ.isAI() || city.isPuppet) {
             ConstructionAutomation(this).chooseNextConstruction()
         }
 
         /** Support for [UniqueType.CreatesOneImprovement] - if an Improvement-creating Building was auto-queued, auto-choose a tile: */
         val building = getCurrentConstruction() as? Building ?: return
-        val improvement = building.getImprovementToCreate(city.getRuleset()) ?: return
+        val improvement = building.getImprovementToCreate(city.getRuleset(), city.civ) ?: return
         if (getTileForImprovement(improvement.name) != null) return
         val newTile = Automation.getTileForConstructionImprovement(city, improvement) ?: return
         newTile.improvementFunctions.markForCreatesOneImprovement(improvement.name)
     }
 
-    fun addToQueue(constructionName: String) {
-        if (isQueueFull()) return
-        val construction = getConstruction(constructionName)
-        if (!construction.isBuildable(this)) return
-        if (construction is Building && isBeingConstructedOrEnqueued(constructionName)) return
-        if (currentConstructionFromQueue == "" || currentConstructionFromQueue == "Nothing") {
-            currentConstructionFromQueue = constructionName
-        } else if (getConstruction(constructionQueue.last()) is PerpetualConstruction) {
-            if (construction is PerpetualConstruction) {  // perpetual constructions will replace each other
-                constructionQueue.removeAt(constructionQueue.size - 1)
+    fun canAddToQueue(construction: IConstruction) =
+        !isQueueFull() &&
+        construction.isBuildable(this) &&
+        !(construction is Building && isBeingConstructedOrEnqueued(construction.name))
+
+    private fun isLastConstructionPerpetual() = constructionQueue.isNotEmpty() &&
+        PerpetualConstruction.isNamePerpetual(constructionQueue.last())
+        // `getConstruction(constructionQueue.last()) is PerpetualConstruction` is clear but more expensive
+
+    fun isQueueEmptyOrIdle() = currentConstructionFromQueue.isEmpty()
+        || currentConstructionFromQueue == PerpetualConstruction.idle.name
+
+    /** Add [construction] to the end or top (controlled by [addToTop]) of the queue with all checks (does nothing if not possible)
+     *
+     *  Note: Overload with string parameter `constructionName` exists as well.
+     */
+    fun addToQueue(construction: IConstruction, addToTop: Boolean = false) {
+        if (!canAddToQueue(construction)) return
+        val constructionName = construction.name
+        when {
+            isQueueEmptyOrIdle() ->
+                currentConstructionFromQueue = constructionName
+            addToTop && construction is PerpetualConstruction && PerpetualConstruction.isNamePerpetual(currentConstructionFromQueue) ->
+                currentConstructionFromQueue = constructionName // perpetual constructions will replace each other
+            addToTop ->
+                constructionQueue.add(0, constructionName)
+            isLastConstructionPerpetual() -> {
+                // Note this also works if currentConstructionFromQueue is perpetual and the only entry - that var is delegated to the first queue position
+                if (construction is PerpetualConstruction) {
+                    // perpetual constructions will replace each other
+                    constructionQueue.removeLast()
+                    constructionQueue.add(constructionName)
+                } else
+                    constructionQueue.add(constructionQueue.size - 1, constructionName) // insert new construction before perpetual one
+            }
+            else ->
                 constructionQueue.add(constructionName)
-            } else
-                constructionQueue.add(constructionQueue.size - 1, constructionName) // insert new construction before perpetual one
-        } else
-            constructionQueue.add(constructionName)
+        }
         currentConstructionIsUserSet = true
     }
 
-    /** If this was done automatically, we should automatically try to choose a new construction and treat it as such */
+    /** Add a construction named [constructionName] to the end of the queue with all checks
+     *
+     *  Note: Delegates to overload with `construction` parameter.
+     */
+    fun addToQueue(constructionName: String) = addToQueue(getConstruction(constructionName))
+
+    /** Remove one entry from the queue by index.
+     *  @param automatic  If this was done automatically, we should automatically try to choose a new construction and treat it as such
+     */
     fun removeFromQueue(constructionQueueIndex: Int, automatic: Boolean) {
         val constructionName = constructionQueue.removeAt(constructionQueueIndex)
 
         // UniqueType.CreatesOneImprovement support
         val construction = getConstruction(constructionName)
         if (construction is Building) {
-            val improvement = construction.getImprovementToCreate(city.getRuleset())
+            val improvement = construction.getImprovementToCreate(city.getRuleset(), city.civ)
             if (improvement != null) {
                 getTileForImprovement(improvement.name)?.stopWorkingOnImprovement()
             }
@@ -756,11 +786,43 @@ class CityConstructions : IsPartOfGameInfoSerialization {
 
         currentConstructionIsUserSet = if (constructionQueue.isEmpty()) {
             if (automatic) chooseNextConstruction()
-            else constructionQueue.add("Nothing") // To prevent Construction Automation
+            else constructionQueue.add(PerpetualConstruction.idle.name) // To prevent Construction Automation
             false
         } else true // we're just continuing the regular queue
     }
 
+    /** Remove all queue entries for [constructionName].
+     *
+     *  Does nothing if there's no entry of that name in the queue.
+     *  If the queue is emptied, no automatic: getSettings().autoAssignCityProduction is ignored! (parameter to be added when needed)
+     */
+    fun removeAllByName(constructionName: String) {
+        while (!isQueueEmptyOrIdle()) {
+            val index = constructionQueue.indexOf(constructionName)
+            if (index < 0) return
+            removeFromQueue(index, false)
+        }
+    }
+
+    /** Moves an entry to the queue top by index.
+     *  No-op when index invalid. Must not be called for PerpetualConstruction entries - unchecked! */
+    fun moveEntryToTop(constructionQueueIndex: Int) {
+        if (constructionQueueIndex == 0 || constructionQueueIndex >= constructionQueue.size) return
+        val constructionName = constructionQueue.removeAt(constructionQueueIndex)
+        constructionQueue.add(0, constructionName)
+    }
+
+    /** Moves an entry by index to the end of the queue, or just before a PerpetualConstruction
+     *  (or replacing a PerpetualConstruction if it itself is one and the queue is by happenstance invalid having more than one of those)
+     */
+    fun moveEntryToEnd(constructionQueueIndex: Int) {
+        if (constructionQueueIndex >= constructionQueue.size) return
+        val constructionName = constructionQueue.removeAt(constructionQueueIndex)
+        // Some of the overhead of addToQueue is redundant here, but if the complex "needs to replace or go before a perpetual" logic is needed, then use it anyway
+        if (isLastConstructionPerpetual()) return addToQueue(constructionName)
+        constructionQueue.add(constructionName)
+    }
+
     fun raisePriority(constructionQueueIndex: Int): Int {
         constructionQueue.swap(constructionQueueIndex - 1, constructionQueueIndex)
         return constructionQueueIndex - 1
@@ -784,13 +846,12 @@ class CityConstructions : IsPartOfGameInfoSerialization {
      *  (skip if none found), then un-mark the tile and place the improvement unless [removeOnly] is set.
      */
     private fun applyCreateOneImprovement(building: Building, removeOnly: Boolean = false) {
-        val improvement = building.getImprovementToCreate(city.getRuleset())
+        val improvement = building.getImprovementToCreate(city.getRuleset(), city.civ)
             ?: return
         val tileForImprovement = getTileForImprovement(improvement.name) ?: return
         tileForImprovement.stopWorkingOnImprovement()  // clears mark
         if (removeOnly) return
-        tileForImprovement.changeImprovement(improvement.name, city.civ)
-        city.civ.lastSeenImprovement[tileForImprovement.position] = improvement.name
+        tileForImprovement.setImprovement(improvement.name, city.civ)
         // If bought the worldscreen will not have been marked to update, and the new improvement won't show until later...
         GUI.setUpdateWorldOnNextRender()
     }
@@ -804,14 +865,14 @@ class CityConstructions : IsPartOfGameInfoSerialization {
         val ruleset = city.getRuleset()
         val indexToRemove = constructionQueue.withIndex().firstNotNullOfOrNull {
             val construction = getConstruction(it.value)
-            val buildingImprovement = (construction as? Building)?.getImprovementToCreate(ruleset)?.name
+            val buildingImprovement = (construction as? Building)?.getImprovementToCreate(ruleset, city.civ)?.name
             it.index.takeIf { buildingImprovement == improvement }
         } ?: return
 
         constructionQueue.removeAt(indexToRemove)
 
         currentConstructionIsUserSet = if (constructionQueue.isEmpty()) {
-            constructionQueue.add("Nothing")
+            constructionQueue.add(PerpetualConstruction.idle.name)
             false
         } else true
     }
