@@ -1,13 +1,10 @@
 package com.unciv.logic.city
 
-import com.unciv.Constants
 import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.models.Counter
 import com.unciv.models.ruleset.Building
-import com.unciv.models.ruleset.GlobalUniques
 import com.unciv.models.ruleset.IConstruction
 import com.unciv.models.ruleset.INonPerpetualConstruction
-import com.unciv.models.ruleset.ModOptionsConstants
 import com.unciv.models.ruleset.unique.LocalUniqueCache
 import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.Unique
@@ -98,9 +95,9 @@ class CityStats(val city: City) {
 
     private fun getStatsFromTradeRoute(): Stats {
         val stats = Stats()
-        if (!city.isCapital() && city.isConnectedToCapital()) {
-            val civInfo = city.civ
-            stats.gold = civInfo.getCapital()!!.population.population * 0.15f + city.population.population * 1.1f - 1 // Calculated by http://civilization.wikia.com/wiki/Trade_route_(Civ5)
+        val capitalForTradeRoutePurposes = city.civ.getCapital()!!
+        if (city != capitalForTradeRoutePurposes && city.isConnectedToCapital()) {
+            stats.gold = capitalForTradeRoutePurposes.population.population * 0.15f + city.population.population * 1.1f - 1 // Calculated by http://civilization.wikia.com/wiki/Trade_route_(Civ5)
             for (unique in city.getMatchingUniques(UniqueType.StatsFromTradeRoute))
                 stats.add(unique.stats)
             val percentageStats = Stats()
@@ -134,7 +131,7 @@ class CityStats(val city: City) {
 
     private fun getStatPercentBonusesFromRailroad(): Stats {
         val stats = Stats()
-        val railroadImprovement = RoadStatus.Railroad.improvement(city.getRuleset())
+        val railroadImprovement = city.getRuleset().railroadImprovement
             ?: return stats // for mods
         val techEnablingRailroad = railroadImprovement.techRequired
         // If we conquered enemy cities connected by railroad, but we don't yet have that tech,
@@ -168,7 +165,7 @@ class CityStats(val city: City) {
             if (!city.matchesFilter(unique.params[1])) continue
 
             growthSources.add(
-                getSourceNameForUnique(unique),
+                unique.getSourceNameForUser(),
                 Stats(food = unique.params[0].toFloat() / 100f * totalFood)
             )
         }
@@ -207,12 +204,12 @@ class CityStats(val city: City) {
 
         val cityStateStatsMultipliers = city.civ.getMatchingUniques(UniqueType.BonusStatsFromCityStates).toList()
 
-        fun addUniqueStats(unique:Unique) {
+        fun addUniqueStats(unique: Unique) {
             val stats = unique.stats.clone()
             if (unique.sourceObjectType==UniqueTarget.CityState)
                 for (multiplierUnique in cityStateStatsMultipliers)
                     stats[Stat.valueOf(multiplierUnique.params[1])] *= multiplierUnique.params[0].toPercent()
-            sourceToStats.addStats(stats, getSourceNameForUnique(unique), unique.sourceObjectName ?: "")
+            sourceToStats.addStats(stats, unique.getSourceNameForUser(), unique.sourceObjectName ?: "")
         }
 
         for (unique in city.getMatchingUniques(UniqueType.StatsPerCity))
@@ -223,7 +220,7 @@ class CityStats(val city: City) {
         for (unique in city.getMatchingUniques(UniqueType.StatsPerPopulation))
             if (city.matchesFilter(unique.params[2])) {
                 val amountOfEffects = (city.population.population / unique.params[1].toInt()).toFloat()
-                sourceToStats.addStats(unique.stats.times(amountOfEffects), getSourceNameForUnique(unique), unique.sourceObjectName ?: "")
+                sourceToStats.addStats(unique.stats.times(amountOfEffects), unique.getSourceNameForUser(), unique.sourceObjectName ?: "")
             }
 
         for (unique in city.getMatchingUniques(UniqueType.StatsFromCitiesOnSpecificTiles))
@@ -235,18 +232,6 @@ class CityStats(val city: City) {
         return sourceToStats
     }
 
-    private fun getSourceNameForUnique(unique: Unique): String {
-        return when (unique.sourceObjectType) {
-            null -> ""
-            UniqueTarget.Global -> GlobalUniques.getUniqueSourceDescription(unique)
-            UniqueTarget.Wonder -> "Wonders"
-            UniqueTarget.Building -> "Buildings"
-            UniqueTarget.Policy -> "Policies"
-            UniqueTarget.CityState -> Constants.cityStates
-            else -> unique.sourceObjectType.name
-        }
-    }
-
     private fun getStatPercentBonusesFromGoldenAge(isGoldenAge: Boolean): Stats {
         val stats = Stats()
         if (isGoldenAge) {
@@ -259,8 +244,8 @@ class CityStats(val city: City) {
     private fun getStatsPercentBonusesFromUniquesBySource(currentConstruction: IConstruction): StatTreeNode {
         val sourceToStats = StatTreeNode()
 
-        fun addUniqueStats(unique:Unique, stat:Stat, amount:Float) {
-            sourceToStats.addStats(Stats().add(stat, amount), getSourceNameForUnique(unique), unique.sourceObjectName ?: "")
+        fun addUniqueStats(unique: Unique, stat: Stat, amount: Float) {
+            sourceToStats.addStats(Stats().add(stat, amount), unique.getSourceNameForUser(), unique.sourceObjectName ?: "")
         }
 
         for (unique in city.getMatchingUniques(UniqueType.StatPercentBonus)) {
@@ -295,14 +280,12 @@ class CityStats(val city: City) {
         for (unique in city.getMatchingUniques(UniqueType.StatPercentFromReligionFollowers))
             addUniqueStats(unique, Stat.valueOf(unique.params[1]),
                 min(
-                    unique.params[0].toFloat() * city.religion.getFollowersOfMajorityReligion(),
+                    unique.params[0].toFloat() * city.religion.getFollowersOfOurReligion(),
                     unique.params[2].toFloat()
                 ))
 
         if (currentConstruction is Building
-            && city.civ.cities.isNotEmpty()
-            && city.civ.getCapital() != null
-            && city.civ.getCapital()!!.cityConstructions.isBuilt(currentConstruction.name)
+            && city.civ.getCapital()?.cityConstructions?.isBuilt(currentConstruction.name) == true
         ) {
             for (unique in city.getMatchingUniques(UniqueType.PercentProductionBuildingsInCapital))
                 addUniqueStats(unique, Stat.Production, unique.params[0].toFloat())
@@ -337,6 +320,12 @@ class CityStats(val city: City) {
             else city.isConnectedToCapital()
     }
 
+    fun getRoadTypeOfConnectionToCapital(): RoadStatus {
+        return if (isConnectedToCapital(RoadStatus.Railroad)) RoadStatus.Railroad
+        else if (isConnectedToCapital(RoadStatus.Road)) RoadStatus.Road
+        else RoadStatus.None
+    }
+
     private fun getBuildingMaintenanceCosts(): Float {
         // Same here - will have a different UI display.
         var buildingsMaintenance = city.cityConstructions.getMaintenanceCosts().toFloat() // this is AFTER the bonus calculation!
@@ -354,9 +343,8 @@ class CityStats(val city: City) {
     //endregion
     //region State-Changing Methods
 
-    fun updateTileStats() {
+    fun updateTileStats(localUniqueCache: LocalUniqueCache = LocalUniqueCache()) {
         val stats = Stats()
-        val localUniqueCache = LocalUniqueCache()
         val workedTiles = city.tilesInRange.asSequence()
             .filter {
                 city.location == it.position
@@ -399,8 +387,6 @@ class CityStats(val city: City) {
         var unhappinessFromCity = -3f // -3 happiness per city
         if (hasExtraAnnexUnhappiness())
             unhappinessFromCity -= 2f
-        if (civInfo.hasUnique(UniqueType.UnhappinessFromCitiesDoubled))
-            unhappinessFromCity *= 2f //doubled for the Indian
 
         var uniqueUnhappinessModifier = 0f
         for (unique in civInfo.getMatchingUniques(UniqueType.UnhappinessFromCitiesPercentage))
@@ -490,12 +476,14 @@ class CityStats(val city: City) {
 
     fun update(currentConstruction: IConstruction = city.cityConstructions.getCurrentConstruction(),
                updateTileStats:Boolean = true,
-               updateCivStats:Boolean = true) {
-        if (updateTileStats) updateTileStats()
+               updateCivStats:Boolean = true,
+               localUniqueCache:LocalUniqueCache = LocalUniqueCache()) {
+
+        if (updateTileStats) updateTileStats(localUniqueCache)
 
         // We need to compute Tile yields before happiness
 
-        val statsFromBuildings = city.cityConstructions.getStats() // this is performance heavy, so calculate once
+        val statsFromBuildings = city.cityConstructions.getStats(localUniqueCache) // this is performance heavy, so calculate once
         updateBaseStatList(statsFromBuildings)
         updateCityHappiness(statsFromBuildings)
         updateStatPercentBonusList(currentConstruction)
@@ -538,7 +526,7 @@ class CityStats(val city: City) {
         }
 
         // AFTER we've gotten all the gold stats figured out, only THEN do we plonk that gold into Science
-        if (city.getRuleset().modOptions.uniques.contains(ModOptionsConstants.convertGoldToScience)) {
+        if (city.getRuleset().modOptions.hasUnique(UniqueType.ConvertGoldToScience)) {
             val amountConverted = (newFinalStatList.values.sumOf { it.gold.toDouble() }
                     * city.civ.tech.goldPercentConvertedToScience).toInt().toFloat()
             if (amountConverted > 0) // Don't want you converting negative gold to negative science yaknow
@@ -555,7 +543,7 @@ class CityStats(val city: City) {
             val removedAmount = newFinalStatList.values.sumOf { it[statToBeRemoved].toDouble() }
 
             newFinalStatList.add(
-                getSourceNameForUnique(unique),
+                unique.getSourceNameForUser(),
                 Stats().apply { this[statToBeRemoved] = -removedAmount.toFloat() }
             )
         }
@@ -593,10 +581,7 @@ class CityStats(val city: City) {
         val buildingsMaintenance = getBuildingMaintenanceCosts() // this is AFTER the bonus calculation!
         newFinalStatList["Maintenance"] = Stats(gold = -buildingsMaintenance.toInt().toFloat())
 
-        if (totalFood > 0
-            && currentConstruction is INonPerpetualConstruction
-            && currentConstruction.hasUnique(UniqueType.ConvertFoodToProductionWhenConstructed)
-        ) {
+        if (canConvertFoodToProduction(totalFood, currentConstruction)) {
             newFinalStatList["Excess food to production"] =
                 Stats(production = getProductionFromExcessiveFood(totalFood), food = -totalFood)
         }
@@ -606,7 +591,7 @@ class CityStats(val city: City) {
             // Note that negative food will also be nullified. Pretty sure that's conform civ V, but haven't checked.
             val amountToRemove = -newFinalStatList.values.sumOf { it[Stat.Food].toDouble() }
             newFinalStatList.add(
-                getSourceNameForUnique(growthNullifyingUnique),
+                growthNullifyingUnique.getSourceNameForUser(),
                 Stats(food = amountToRemove.toFloat())
             )
         }
@@ -619,9 +604,20 @@ class CityStats(val city: City) {
         finalStatList = newFinalStatList
     }
 
-    // calculate the conversion of the excessive food to the production
-    // See for details: https://civilization.fandom.com/wiki/Settler_(Civ5)
-    private fun getProductionFromExcessiveFood(food : Float): Float {
+    fun canConvertFoodToProduction(food: Float, currentConstruction: IConstruction): Boolean {
+        return (food > 0
+            && currentConstruction is INonPerpetualConstruction
+            && currentConstruction.hasUnique(UniqueType.ConvertFoodToProductionWhenConstructed))
+    }
+
+    /**
+     * Calculate the conversion of the excessive food to production when
+     * [UniqueType.ConvertFoodToProductionWhenConstructed] is at front of the build queue
+     * @param food is amount of excess Food generates this turn
+     * See for details: https://civilization.fandom.com/wiki/Settler_(Civ5)
+     * @see calcFoodEaten as well for Food consumed this turn
+     */
+    fun getProductionFromExcessiveFood(food : Float): Float {
         return if (food >= 4.0f ) 2.0f + (food / 4.0f).toInt()
           else if (food >= 2.0f ) 2.0f
           else if (food >= 1.0f ) 1.0f
@@ -641,4 +637,14 @@ class CityStats(val city: City) {
     }
 
     //endregion
+
+    fun getStatDifferenceFromBuilding(building: String): Stats {
+        val newCity = city.clone()
+        newCity.setTransients(city.civ) // Will break the owned tiles. Needs to be reverted before leaving this function
+        newCity.cityConstructions.builtBuildings.add(building)
+        newCity.cityConstructions.setTransients()
+        newCity.cityStats.update(updateCivStats = false)
+        city.expansion.setTransients() // Revert owned tiles to original city
+        return newCity.cityStats.currentCityStats - currentCityStats
+    }
 }
