@@ -14,14 +14,13 @@ import com.unciv.logic.trade.TradeLogic
 import com.unciv.logic.trade.TradeOffer
 import com.unciv.logic.trade.TradeType
 import com.unciv.models.ruleset.unique.UniqueType
-import com.unciv.utils.debug
 import kotlin.math.max
 import kotlin.math.min
 import kotlin.math.roundToInt
 import kotlin.random.Random
 
 /** Helper class for containing 200 lines of "how to move cities between civs" */
-class CityConquestFunctions(val city: City){
+class CityConquestFunctions(val city: City) {
     private val tileBasedRandom = Random(city.getCenterTile().position.toString().hashCode())
 
     private fun getGoldForCapturingCity(conqueringCiv: Civilization): Int {
@@ -41,33 +40,24 @@ class CityConquestFunctions(val city: City){
                 building.hasUnique(UniqueType.NotDestroyedWhenCityCaptured) || building.isWonder -> continue
                 building.hasUnique(UniqueType.IndicatesCapital) -> continue // Palace needs to stay a just a bit longer so moveToCiv isn't confused
                 building.hasUnique(UniqueType.DestroyedWhenCityCaptured) ->
-                    city.cityConstructions.removeBuilding(building.name)
+                    city.cityConstructions.removeBuilding(building)
                 // Regular buildings have a 34% chance of removal
-                tileBasedRandom.nextInt(100) < 34 -> city.cityConstructions.removeBuilding(building.name)
+                tileBasedRandom.nextInt(100) < 34 -> city.cityConstructions.removeBuilding(building)
             }
         }
     }
 
     private fun removeBuildingsOnMoveToCiv(oldCiv: Civilization) {
         // Remove all buildings provided for free to this city
-        for (building in city.civ.civConstructions.getFreeBuildings(city.id)) {
+        for (building in city.civ.civConstructions.getFreeBuildingNames(city)) {
             city.cityConstructions.removeBuilding(building)
         }
-
-        // Remove all buildings provided for free from here to other cities (e.g. CN Tower)
-        for ((cityId, buildings) in city.cityConstructions.freeBuildingsProvidedFromThisCity) {
-            val city = oldCiv.cities.firstOrNull { it.id == cityId } ?: continue
-            debug("Removing buildings %s from city %s", buildings, city.name)
-            for (building in buildings) {
-                city.cityConstructions.removeBuilding(building)
-            }
-        }
         city.cityConstructions.freeBuildingsProvidedFromThisCity.clear()
 
         for (building in city.cityConstructions.getBuiltBuildings()) {
             // Remove national wonders
             if (building.isNationalWonder && !building.hasUnique(UniqueType.NotDestroyedWhenCityCaptured))
-                city.cityConstructions.removeBuilding(building.name)
+                city.cityConstructions.removeBuilding(building)
 
             // Check if we exceed MaxNumberBuildable for any buildings
             for (unique in building.getMatchingUniques(UniqueType.MaxNumberBuildable)) {
@@ -78,7 +68,7 @@ class CityConquestFunctions(val city: City){
                         } >= unique.params[0].toInt()
                 ) {
                     // For now, just destroy in new city. Even if constructing in own cities
-                    this.city.cityConstructions.removeBuilding(building.name)
+                    city.cityConstructions.removeBuilding(building)
                 }
             }
         }
@@ -89,6 +79,9 @@ class CityConquestFunctions(val city: City){
      * should go in `this.moveToCiv()`, which is called by `this.conquerCity()`.
      */
     private fun conquerCity(conqueringCiv: Civilization, conqueredCiv: Civilization, receivingCiv: Civilization) {
+        city.espionage.removeAllPresentSpies(SpyFleeReason.CityCaptured)
+
+        // Gain gold for plundering city
         val goldPlundered = getGoldForCapturingCity(conqueringCiv)
         conqueringCiv.addGold(goldPlundered)
         conqueringCiv.addNotification("Received [$goldPlundered] Gold for capturing [${city.name}]",
@@ -100,7 +93,7 @@ class CityConquestFunctions(val city: City){
 
         city.moveToCiv(receivingCiv)
 
-        Battle.destroyIfDefeated(conqueredCiv, conqueringCiv)
+        Battle.destroyIfDefeated(conqueredCiv, conqueringCiv, city.location)
 
         city.health = city.getMaxHealth() / 2 // I think that cities recover to half health when conquered?
         if (city.population.population > 1)
@@ -115,16 +108,11 @@ class CityConquestFunctions(val city: City){
             // reconquering or liberating city in resistance so eliminate it
             city.removeFlag(CityFlags.Resistance)
         }
-
-        city.espionage.removeAllPresentSpies(SpyFleeReason.CityCaptured)
     }
 
 
     /** This happens when we either puppet OR annex, basically whenever we conquer a city and don't liberate it */
     fun puppetCity(conqueringCiv: Civilization) {
-        // Gain gold for plundering city
-        @Suppress("UNUSED_VARIABLE")  // todo: use this val
-        val goldPlundered = getGoldForCapturingCity(conqueringCiv)
         val oldCiv = city.civ
 
         // must be before moving the city to the conquering civ,
@@ -194,7 +182,8 @@ class CityConquestFunctions(val city: City){
 
         if (foundingCiv.cities.size == 1) {
             // Resurrection!
-            city.cityConstructions.addBuilding(city.capitalCityIndicator())
+            val capitalCityIndicator = city.capitalCityIndicator()
+            if (capitalCityIndicator != null) city.cityConstructions.addBuilding(capitalCityIndicator)
             for (civ in city.civ.gameInfo.civilizations) {
                 if (civ == foundingCiv || civ == conqueringCiv) continue // don't need to notify these civs
                 when {
@@ -234,6 +223,9 @@ class CityConquestFunctions(val city: City){
         if (foundingCiv.isMajorCiv()) {
             foundingCiv.getDiplomacyManager(conqueringCiv)
                     .addModifier(DiplomaticModifiers.CapturedOurCities, respectForLiberatingOurCity)
+            val openBordersTrade = TradeLogic(foundingCiv, conqueringCiv)
+            openBordersTrade.currentTrade.ourOffers.add(TradeOffer(Constants.openBorders, TradeType.Agreement))
+            openBordersTrade.acceptTrade(false)
         } else {
             //Liberating a city state gives a large amount of influence, and peace
             foundingCiv.getDiplomacyManager(conqueringCiv).setInfluence(90f)
@@ -241,7 +233,7 @@ class CityConquestFunctions(val city: City){
                 val tradeLogic = TradeLogic(foundingCiv, conqueringCiv)
                 tradeLogic.currentTrade.ourOffers.add(TradeOffer(Constants.peaceTreaty, TradeType.Treaty))
                 tradeLogic.currentTrade.theirOffers.add(TradeOffer(Constants.peaceTreaty, TradeType.Treaty))
-                tradeLogic.acceptTrade()
+                tradeLogic.acceptTrade(false)
             }
         }
 
@@ -258,7 +250,7 @@ class CityConquestFunctions(val city: City){
 
         // Remove/relocate palace for old Civ - need to do this BEFORE we move the cities between
         //  civs so the capitalCityIndicator recognizes the unique buildings of the conquered civ
-        if (city.isCapital())  oldCiv.moveCapitalToNextLargest(city)
+        if (city.isCapital()) oldCiv.moveCapitalToNextLargest(city)
 
         oldCiv.cities = oldCiv.cities.toMutableList().apply { remove(city) }
         newCiv.cities = newCiv.cities.toMutableList().apply { add(city) }
@@ -289,10 +281,10 @@ class CityConquestFunctions(val city: City){
 
         // Transfer unique buildings
         for (building in city.cityConstructions.getBuiltBuildings()) {
-            val civEquivalentBuilding = newCiv.getEquivalentBuilding(building.name)
+            val civEquivalentBuilding = newCiv.getEquivalentBuilding(building)
             if (building != civEquivalentBuilding) {
-                city.cityConstructions.removeBuilding(building.name)
-                city.cityConstructions.addBuilding(civEquivalentBuilding.name)
+                city.cityConstructions.removeBuilding(building)
+                city.cityConstructions.addBuilding(civEquivalentBuilding)
             }
         }
 
