@@ -6,13 +6,14 @@ import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.civilization.NotificationCategory
 import com.unciv.logic.civilization.NotificationIcon
 import com.unciv.logic.civilization.PopupAlert
-import com.unciv.logic.map.mapunit.UnitMovement
+import com.unciv.logic.map.mapunit.movement.UnitMovement
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.stats.Stat
 import com.unciv.models.stats.Stats
+import kotlin.math.max
 
-class DiplomacyFunctions(val civInfo: Civilization){
+class DiplomacyFunctions(val civInfo: Civilization) {
 
     /** A sorted Sequence of all other civs we know (excluding barbarians and spectators) */
     fun getKnownCivsSorted(includeCityStates: Boolean = true, includeDefeated: Boolean = false) =
@@ -57,7 +58,7 @@ class DiplomacyFunctions(val civInfo: Civilization){
             else
                 otherCiv.addNotification(meetString, NotificationCategory.Diplomacy, NotificationIcon.Gold)
 
-            if (otherCiv.isCityState() && otherCiv.cityStateFunctions.canProvideStat(Stat.Faith)){
+            if (otherCiv.isCityState() && otherCiv.cityStateFunctions.canProvideStat(Stat.Faith)) {
                 otherCiv.addNotification(religionMeetString, NotificationCategory.Diplomacy, NotificationIcon.Faith)
 
                 for ((key, value) in faithAmount)
@@ -86,6 +87,12 @@ class DiplomacyFunctions(val civInfo: Civilization){
         }
     }
 
+    fun canSignDeclarationOfFriendshipWith(otherCiv: Civilization): Boolean {
+        return otherCiv.isMajorCiv() && !otherCiv.isAtWarWith(civInfo)
+            && !civInfo.getDiplomacyManager(otherCiv).hasFlag(DiplomacyFlags.Denunciation)
+            && !civInfo.getDiplomacyManager(otherCiv).hasFlag(DiplomacyFlags.DeclarationOfFriendship)
+    }
+
     fun canSignResearchAgreement(): Boolean {
         if (!civInfo.isMajorCiv()) return false
         if (!civInfo.hasUnique(UniqueType.EnablesResearchAgreements)) return false
@@ -94,25 +101,28 @@ class DiplomacyFunctions(val civInfo: Civilization){
         return true
     }
 
-    fun canSignResearchAgreementsWith(otherCiv: Civilization): Boolean {
+    fun canSignResearchAgreementNoCostWith (otherCiv: Civilization): Boolean {
         val diplomacyManager = civInfo.getDiplomacyManager(otherCiv)
-        val cost = getResearchAgreementCost()
         return canSignResearchAgreement() && otherCiv.diplomacyFunctions.canSignResearchAgreement()
-                && diplomacyManager.hasFlag(DiplomacyFlags.DeclarationOfFriendship)
-                && !diplomacyManager.hasFlag(DiplomacyFlags.ResearchAgreement)
-                && !diplomacyManager.otherCivDiplomacy().hasFlag(DiplomacyFlags.ResearchAgreement)
-                && civInfo.gold >= cost && otherCiv.gold >= cost
+            && diplomacyManager.hasFlag(DiplomacyFlags.DeclarationOfFriendship)
+            && !diplomacyManager.hasFlag(DiplomacyFlags.ResearchAgreement)
+            && !diplomacyManager.otherCivDiplomacy().hasFlag(DiplomacyFlags.ResearchAgreement)
     }
 
-    fun getResearchAgreementCost(): Int {
+    fun canSignResearchAgreementsWith(otherCiv: Civilization): Boolean {
+        val cost = getResearchAgreementCost(otherCiv)
+        return canSignResearchAgreementNoCostWith(otherCiv)
+            && civInfo.gold >= cost && otherCiv.gold >= cost
+    }
+
+    fun getResearchAgreementCost(otherCiv: Civilization): Int {
         // https://forums.civfanatics.com/resources/research-agreements-bnw.25568/
-        return (
-                civInfo.getEra().researchAgreementCost * civInfo.gameInfo.speed.goldCostModifier
-                ).toInt()
+        return ( max(civInfo.getEra().researchAgreementCost, otherCiv.getEra().researchAgreementCost)
+                    * civInfo.gameInfo.speed.goldCostModifier
+            ).toInt()
     }
 
     fun canSignDefensivePact(): Boolean {
-        return false // TEMPORARY - remove at 4.7.17!
         if (!civInfo.isMajorCiv()) return false
         if (!civInfo.hasUnique(UniqueType.EnablesDefensivePacts)) return false
         return true
@@ -121,7 +131,8 @@ class DiplomacyFunctions(val civInfo: Civilization){
     fun canSignDefensivePactWith(otherCiv: Civilization): Boolean {
         val diplomacyManager = civInfo.getDiplomacyManager(otherCiv)
         return canSignDefensivePact() && otherCiv.diplomacyFunctions.canSignDefensivePact()
-            && diplomacyManager.hasFlag(DiplomacyFlags.DeclarationOfFriendship)
+            && (diplomacyManager.hasFlag(DiplomacyFlags.DeclarationOfFriendship)
+            || diplomacyManager.otherCivDiplomacy().hasFlag(DiplomacyFlags.DeclarationOfFriendship))
             && !diplomacyManager.hasFlag(DiplomacyFlags.DefensivePact)
             && !diplomacyManager.otherCivDiplomacy().hasFlag(DiplomacyFlags.DefensivePact)
             && diplomacyManager.diplomaticStatus != DiplomaticStatus.DefensivePact
@@ -145,7 +156,7 @@ class DiplomacyFunctions(val civInfo: Civilization){
         if (diplomacyManager != null && (diplomacyManager.hasOpenBorders || diplomacyManager.diplomaticStatus == DiplomaticStatus.War))
             return true
         // Players can always pass through city-state tiles
-        if (civInfo.isHuman() && otherCiv.isCityState()) return true
+        if (!civInfo.isAIOrAutoPlaying() && otherCiv.isCityState()) return true
         return false
     }
 
