@@ -37,12 +37,14 @@ class VictoryManager : IsPartOfGameInfoSerialization {
         return results
     }
 
+    private fun getVotingCivs() = civInfo.gameInfo.civilizations.asSequence()
+        .filterNot { it.isBarbarian() || it.isSpectator() || it.isDefeated() }
+
     /** Finds the Building and Owner of the United Nations (or whatever the Mod called it)
      *  - if it's built at all and only if the owner is alive
      *  @return `first`: Building name, `second`: Owner civ name; both null if not found
      */
-    fun getUNBuildingAndOwnerNames(): Pair<String?, String?> = civInfo.gameInfo.civilizations.asSequence()
-            .filterNot { it.isBarbarian() || it.isSpectator() || it.isDefeated() }
+    fun getUNBuildingAndOwnerNames(): Pair<String?, String?> = getVotingCivs()
             .flatMap { civ -> civ.cities.asSequence()
                 .flatMap { it.cityConstructions.getBuiltBuildings() }
                 .filter { it.hasUnique(UniqueType.OneTimeTriggerVoting, stateForConditionals = StateForConditionals.IgnoreConditionals) }
@@ -50,13 +52,19 @@ class VictoryManager : IsPartOfGameInfoSerialization {
             }.firstOrNull() ?: (null to null)
 
     private fun votesNeededForDiplomaticVictory(): Int {
-        val civCount = civInfo.gameInfo.civilizations.count { !it.isDefeated() }
+        // The original counts "teams ever alive", which excludes Observer and Barbarians.
+        // The "ever alive" part sounds unfair - could make a Vote unwinnable?
 
-        // CvGame.cpp::DoUpdateDiploVictory() in the source code of the original
-        return (
-            if (civCount > 28) 0.35 * civCount
-            else (67 - 1.1 * civCount) / 100 * civCount
-        ).toInt()
+        // So this is a slightly arbitrary decision: Apply original formula to count of available votes
+        // - including catering for the possibility we're voting without a UN thanks to razing -
+        val (_, civOwningUN) = getUNBuildingAndOwnerNames()
+        val voteCount = getVotingCivs().count() + (if (civOwningUN != null) 1 else 0)
+
+        // CvGame.cpp::DoUpdateDiploVictory() in the source code of the original - same integer math and rounding!
+        // To verify run `(1..30).map { voteCount -> voteCount to voteCount * (67 - (1.1 * voteCount).toInt()) / 100 + 1 }`
+        // ... and compare with: "4 votes needed to win in a game with 5 players, 7 with 13 and 11 with 28"
+        if (voteCount > 28) return voteCount * 35 / 100
+        return voteCount * (67 - (1.1 * voteCount).toInt()) / 100 + 1
     }
 
     fun hasEnoughVotesForDiplomaticVictory(): Boolean {
@@ -73,13 +81,14 @@ class VictoryManager : IsPartOfGameInfoSerialization {
         return (results.none { it != bestCiv && it.value == bestCiv.value })
     }
 
-    fun getDiplomaticVictoryVoteBreakdown(): String {
+    data class DiplomaticVictoryVoteBreakdown(val results: Counter<String>, val winnerText: String)
+    fun getDiplomaticVictoryVoteBreakdown(): DiplomaticVictoryVoteBreakdown {
         val results = calculateDiplomaticVotingResults(civInfo.gameInfo.diplomaticVictoryVotesCast)
         val (voteCount, winnerList) = results.asSequence()
             .groupBy({ it.value }, { it.key }).asSequence()
             .sortedByDescending { it.key }  // key is vote count here
             .firstOrNull()
-            ?: return "No valid votes were cast."
+            ?: return DiplomaticVictoryVoteBreakdown(results, "No valid votes were cast.")
 
         val lines = arrayListOf<String>()
         val minVotes = votesNeededForDiplomaticVictory()
@@ -91,9 +100,9 @@ class VictoryManager : IsPartOfGameInfoSerialization {
         lines += when {
             lines.isNotEmpty() -> "No world leader was elected."
             winnerCiv == civInfo -> "You have been elected world leader!"
-            else -> "${civInfo.nation.getLeaderDisplayName()} has been elected world leader!"
+            else -> "${winnerCiv.nation.getLeaderDisplayName()} has been elected world leader!"
         }
-        return lines.joinToString("\n") { "{$it}" }
+        return DiplomaticVictoryVoteBreakdown(results, lines.joinToString("\n") { "{$it}" })
     }
 
     fun getVictoryTypeAchieved(): String? {
