@@ -3,8 +3,10 @@ package com.unciv.logic.files
 import com.badlogic.gdx.Files
 import com.badlogic.gdx.Gdx
 import com.badlogic.gdx.files.FileHandle
+import com.badlogic.gdx.utils.GdxRuntimeException
 import com.badlogic.gdx.utils.JsonReader
 import com.badlogic.gdx.utils.SerializationException
+import com.unciv.Constants
 import com.unciv.UncivGame
 import com.unciv.json.fromJsonFile
 import com.unciv.json.json
@@ -14,9 +16,12 @@ import com.unciv.logic.GameInfoPreview
 import com.unciv.logic.GameInfoSerializationVersion
 import com.unciv.logic.HasGameInfoSerializationVersion
 import com.unciv.logic.UncivShowableException
+import com.unciv.logic.civilization.PlayerType
+import com.unciv.logic.civilization.managers.TurnManager
 import com.unciv.models.metadata.GameSettings
 import com.unciv.models.metadata.doMigrations
 import com.unciv.models.metadata.isMigrationNecessary
+import com.unciv.models.ruleset.RulesetCache
 import com.unciv.ui.screens.savescreens.Gzip
 import com.unciv.utils.Concurrency
 import com.unciv.utils.Log
@@ -41,11 +46,9 @@ class UncivFiles(
         debug("Creating UncivFiles, localStoragePath: %s, externalStoragePath: %s",
             files.localStoragePath, files.externalStoragePath)
     }
-    //region Data
 
-    var autoSaveJob: Job? = null
+    val autosaves = Autosaves(this)
 
-    //endregion
     //region Helpers
 
     fun getSave(gameName: String): FileHandle {
@@ -63,8 +66,8 @@ class UncivFiles(
         val externalFile = files.external(location)
 
         val toReturn = if (files.isExternalStorageAvailable && (
-                        preferExternalStorage && (externalFile.exists() || !localFile.exists())
-                        || !preferExternalStorage && (externalFile.exists() && !localFile.exists())
+                externalFile.exists() && !localFile.exists() || // external file is only valid choice
+                preferExternalStorage && (externalFile.exists() || !localFile.exists()) // unless local file is only valid choice, choose external
                 ) ) {
             externalFile
         } else {
@@ -79,14 +82,16 @@ class UncivFiles(
      * @throws GdxRuntimeException if the [path] represents a directory
      */
     fun fileWriter(path: String, append: Boolean = false): Writer {
-        val file = if (preferExternalStorage && files.isExternalStorageAvailable) {
-            files.external(path)
-        } else {
-            files.local(path)
-        }
-        return file.writer(append)
+        val file = pathToFileHandler(path)
+        return file.writer(append, Charsets.UTF_8.name())
     }
 
+    fun pathToFileHandler(path: String): FileHandle {
+        return if (preferExternalStorage && files.isExternalStorageAvailable) files.external(path)
+        else files.local(path)
+    }
+
+
     fun getMultiplayerSaves(): Sequence<FileHandle> {
         return getSaves(MULTIPLAYER_FILES_FOLDER)
     }
@@ -158,7 +163,7 @@ class UncivFiles(
         try {
             debug("Saving GameInfo %s to %s", game.gameId, file.path())
             val string = gameInfoToString(game)
-            file.writeString(string, false)
+            file.writeString(string, false, Charsets.UTF_8.name())
             saveCompletionCallback(null)
         } catch (ex: Exception) {
             saveCompletionCallback(ex)
@@ -198,12 +203,13 @@ class UncivFiles(
         game: GameInfo,
         gameName: String,
         onSaved: () -> Unit,
-        onError: (Exception) -> Unit) {
+        onError: (Exception) -> Unit
+    ) {
         val saveLocation = game.customSaveLocation ?: Gdx.files.local(gameName).path()
 
         try {
             val data = gameInfoToString(game)
-            debug("Saving GameInfo %s to custom location %s", game.gameId, saveLocation)
+            debug("Initiating UI to save GameInfo %s to custom location %s", game.gameId, saveLocation)
             saverLoader.saveGame(data, saveLocation,
                 { location ->
                     game.customSaveLocation = location
@@ -226,7 +232,7 @@ class UncivFiles(
             loadGameFromFile(getSave(gameName))
 
     fun loadGameFromFile(gameFile: FileHandle): GameInfo {
-        val gameData = gameFile.readString()
+        val gameData = gameFile.readString(Charsets.UTF_8.name())
         if (gameData.isNullOrBlank()) {
             throw emptyFile(gameFile)
         }
@@ -304,7 +310,34 @@ class UncivFiles(
     }
 
     fun setGeneralSettings(gameSettings: GameSettings) {
-        getGeneralSettingsFile().writeString(json().toJson(gameSettings), false)
+        getGeneralSettingsFile().writeString(json().toJson(gameSettings), false, Charsets.UTF_8.name())
+    }
+
+    val scenarioFolder = "scenarios"
+    fun getScenarioFiles() = sequence {
+
+        for (mod in RulesetCache.values) {
+            val modFolder = mod.folderLocation ?: continue
+            val scenarioFolder = modFolder.child(scenarioFolder)
+            if (scenarioFolder.exists())
+                for (file in scenarioFolder.list())
+                    yield(Pair(file, mod))
+        }
+    }
+
+    fun loadScenario(gameFile: FileHandle): GameInfo {
+        val game = loadGameFromFile(gameFile)
+        game.civilizations.removeAll { it.isSpectator() }
+        for (civ in game.civilizations)
+            civ.diplomacy.remove(Constants.spectator)
+        if (game.civilizations.none { it.isHuman() })
+            game.civilizations.first { it.isMajorCiv() }.playerType = PlayerType.Human
+
+        game.currentPlayerCiv = game.civilizations.first { it.playerType == PlayerType.Human }
+        game.currentPlayer = game.currentPlayerCiv.civName
+        TurnManager(game.currentPlayerCiv).startTurn()
+
+        return game
     }
 
     companion object {
@@ -371,7 +404,7 @@ class UncivFiles(
         }
 
         /** Returns gzipped serialization of [game], optionally gzipped ([forceZip] overrides [saveZipped]) */
-        fun gameInfoToString(game: GameInfo, forceZip: Boolean? = null, updateChecksum:Boolean=true): String {
+        fun gameInfoToString(game: GameInfo, forceZip: Boolean? = null, updateChecksum: Boolean = false): String {
             game.version = GameInfo.CURRENT_COMPATIBILITY_VERSION
 
             if (updateChecksum) game.checksum = game.calculateChecksum()
@@ -385,9 +418,11 @@ class UncivFiles(
         }
 
     }
+}
 
-    //endregion
-    //region Autosave
+class Autosaves(val files: UncivFiles) {
+
+    var autoSaveJob: Job? = null
 
     /**
      * Auto-saves a snapshot of the [gameInfo] in a new thread.
@@ -412,7 +447,7 @@ class UncivFiles(
 
     fun autoSave(gameInfo: GameInfo, nextTurn: Boolean = false) {
         try {
-            saveGame(gameInfo, AUTOSAVE_FILE_NAME)
+            files.saveGame(gameInfo, AUTOSAVE_FILE_NAME)
         } catch (oom: OutOfMemoryError) {
             Log.error("Ran out of memory during autosave", oom)
             return  // not much we can do here
@@ -422,40 +457,32 @@ class UncivFiles(
         if (nextTurn) {
             val newAutosaveFilename =
                 SAVE_FILES_FOLDER + File.separator + AUTOSAVE_FILE_NAME + "-${gameInfo.currentPlayer}-${gameInfo.turns}"
-            val file =
-                if (preferExternalStorage && files.isExternalStorageAvailable)
-                    files.external(newAutosaveFilename)
-                else
-                    files.local(newAutosaveFilename)
-            getSave(AUTOSAVE_FILE_NAME).copyTo(file)
+            val file = files.pathToFileHandler(newAutosaveFilename)
+            files.getSave(AUTOSAVE_FILE_NAME).copyTo(file)
 
             fun getAutosaves(): Sequence<FileHandle> {
-                return getSaves().filter { it.name().startsWith(AUTOSAVE_FILE_NAME) }
+                return files.getSaves().filter { it.name().startsWith(AUTOSAVE_FILE_NAME) }
             }
             while (getAutosaves().count() > 10) {
                 val saveToDelete = getAutosaves().minByOrNull { it.lastModified() }!!
-                deleteSave(saveToDelete.name())
+                files.deleteSave(saveToDelete.name())
             }
         }
     }
 
     fun loadLatestAutosave(): GameInfo {
         return try {
-            loadGameByName(AUTOSAVE_FILE_NAME)
+            files.loadGameByName(AUTOSAVE_FILE_NAME)
         } catch (_: Exception) {
             // silent fail if we can't read the autosave for any reason - try to load the last autosave by turn number first
-            val autosaves = getSaves().filter { it.name() != AUTOSAVE_FILE_NAME && it.name().startsWith(
+            val autosaves = files.getSaves().filter { it.name() != AUTOSAVE_FILE_NAME && it.name().startsWith(
                 AUTOSAVE_FILE_NAME
             ) }
-            loadGameFromFile(autosaves.maxByOrNull { it.lastModified() }!!)
+            files.loadGameFromFile(autosaves.maxByOrNull { it.lastModified() }!!)
         }
     }
 
-    fun autosaveExists(): Boolean {
-        return getSave(AUTOSAVE_FILE_NAME).exists()
-    }
-
-    // endregion
+    fun autosaveExists(): Boolean = files.getSave(AUTOSAVE_FILE_NAME).exists()
 }
 
 class IncompatibleGameInfoVersionException(
