@@ -1,7 +1,7 @@
 package com.unciv.logic.map.mapunit
 
-import com.unciv.UncivGame
 import com.unciv.logic.civilization.LocationAction
+import com.unciv.logic.civilization.MapUnitAction
 import com.unciv.logic.civilization.NotificationCategory
 import com.unciv.logic.civilization.NotificationIcon
 import com.unciv.models.ruleset.unique.UniqueTriggerActivation
@@ -11,19 +11,27 @@ class UnitTurnManager(val unit: MapUnit) {
 
     fun endTurn() {
         unit.movement.clearPathfindingCache()
+
+        for (unique in unit.getTriggeredUniques(UniqueType.TriggerUponTurnEnd))
+            UniqueTriggerActivation.triggerUnique(unique, unit)
+
         if (unit.currentMovement > 0
                 && unit.getTile().improvementInProgress != null
                 && unit.canBuildImprovement(unit.getTile().getTileImprovementInProgress()!!)
-        ) workOnImprovement()
-        if (unit.currentMovement == unit.getMaxMovement().toFloat() && unit.isFortified() && unit.turnsFortified < 2) {
+        ) {
+            val tile = unit.getTile()
+            if (tile.doWorkerTurn(unit))
+                tile.getCity()?.updateCitizens = true
+        }
+
+        if (!unit.hasUnitMovedThisTurn() && unit.isFortified() && unit.turnsFortified < 2) {
             unit.turnsFortified++
         }
         if (!unit.isFortified())
             unit.turnsFortified = 0
 
-        if (unit.currentMovement == unit.getMaxMovement().toFloat() // didn't move this turn
-                || unit.hasUnique(UniqueType.HealsEvenAfterAction)
-        ) healUnit()
+        if (!unit.hasUnitMovedThisTurn() || unit.hasUnique(UniqueType.HealsEvenAfterAction))
+            healUnit()
 
         if (unit.action != null && unit.health > 99)
             if (unit.isActionUntilHealed()) {
@@ -59,7 +67,7 @@ class UnitTurnManager(val unit: MapUnit) {
         for (unique in unit.getTriggeredUniques(UniqueType.TriggerUponEndingTurnInTile))
             if (unique.conditionals.any { it.type == UniqueType.TriggerUponEndingTurnInTile
                             && unit.getTile().matchesFilter(it.params[0], unit.civ) })
-                UniqueTriggerActivation.triggerUnitwideUnique(unique, unit)
+                UniqueTriggerActivation.triggerUnique(unique, unit)
     }
 
 
@@ -84,36 +92,40 @@ class UnitTurnManager(val unit: MapUnit) {
             }.maxByOrNull { it.second }
             ?: return
         if (damage == 0) return
-        unit.health -= damage
+        unit.takeDamage(damage)
+        val improvementName = citadelTile.improvement!!  // guarded by `getUnpillagedImprovement() != null` above
+        val improvementIcon = "ImprovementIcons/$improvementName"
         val locations = LocationAction(citadelTile.position, unit.currentTile.position)
         if (unit.health <= 0) {
             unit.civ.addNotification(
-                "An enemy [Citadel] has destroyed our [${unit.name}]",
+                "An enemy [$improvementName] has destroyed our [${unit.name}]",
                 locations,
                 NotificationCategory.War,
-                NotificationIcon.Citadel, NotificationIcon.Death, unit.name
+                improvementIcon, NotificationIcon.Death, unit.name
             )
             citadelTile.getOwner()?.addNotification(
-                "Your [Citadel] has destroyed an enemy [${unit.name}]",
+                "Your [$improvementName] has destroyed an enemy [${unit.name}]",
                 locations,
                 NotificationCategory.War,
-                NotificationIcon.Citadel, NotificationIcon.Death, unit.name
+                improvementIcon, NotificationIcon.Death, unit.name
             )
             unit.destroy()
         } else unit.civ.addNotification(
-            "An enemy [Citadel] has attacked our [${unit.name}]",
+            "An enemy [$improvementName] has attacked our [${unit.name}]",
             locations,
             NotificationCategory.War,
-            NotificationIcon.Citadel, NotificationIcon.War, unit.name
+            improvementIcon, NotificationIcon.War, unit.name
         )
     }
 
 
     private fun doTerrainDamage() {
         val tileDamage = unit.getDamageFromTerrain()
-        unit.health -= tileDamage
+        if (tileDamage == 0) return
 
-        if (unit.health <= 0) {
+        unit.takeDamage(tileDamage)
+
+        if (unit.isDestroyed) {
             unit.civ.addNotification(
                 "Our [${unit.name}] took [$tileDamage] tile damage and was destroyed",
                 unit.currentTile.position,
@@ -121,10 +133,9 @@ class UnitTurnManager(val unit: MapUnit) {
                 unit.name,
                 NotificationIcon.Death
             )
-            unit.destroy()
-        } else if (tileDamage > 0) unit.civ.addNotification(
+        } else unit.civ.addNotification(
             "Our [${unit.name}] took [$tileDamage] tile damage",
-            unit.currentTile.position,
+            MapUnitAction(unit),
             NotificationCategory.Units,
             unit.name
         )
@@ -137,17 +148,6 @@ class UnitTurnManager(val unit: MapUnit) {
         unit.attacksThisTurn = 0
         unit.due = true
 
-        // Hakkapeliitta movement boost
-        // For every double-stacked tile, check if our cohabitant can boost our speed
-        // (a test `count() > 1` is no optimization - two iterations of a sequence instead of one)
-        for (boostingUnit in unit.getTile().getUnits()) {
-            if (boostingUnit == unit) continue
-
-            if (boostingUnit.getMatchingUniques(UniqueType.TransferMovement)
-                        .none { unit.matchesFilter(it.params[0]) } ) continue
-            unit.currentMovement = unit.currentMovement.coerceAtLeast(boostingUnit.getMaxMovement().toFloat())
-        }
-
         // Wake sleeping units if there's an enemy in vision range:
         // Military units always but civilians only if not protected.
         if (unit.isSleeping() && (unit.isMilitary() || (unit.currentTile.militaryUnit == null && !unit.currentTile.isCityCenter())) &&
@@ -166,23 +166,4 @@ class UnitTurnManager(val unit: MapUnit) {
         unit.addMovementMemory()
         unit.attacksSinceTurnStart.clear()
     }
-
-    private fun workOnImprovement() {
-        val tile = unit.getTile()
-        if (tile.isMarkedForCreatesOneImprovement()) return
-        tile.turnsToImprovement -= 1
-        if (tile.turnsToImprovement != 0) return
-
-        if (unit.civ.isCurrentPlayer())
-            UncivGame.Current.settings.addCompletedTutorialTask("Construct an improvement")
-
-        val improvementInProgress = tile.improvementInProgress ?: return
-        tile.changeImprovement(improvementInProgress, unit.civ)
-
-        tile.improvementInProgress = null
-        tile.getCity()?.updateCitizens = true
-    }
-
-
-
 }
