@@ -5,11 +5,12 @@ import com.badlogic.gdx.files.FileHandle
 import com.unciv.json.fromJsonFile
 import com.unciv.json.json
 import com.unciv.logic.civilization.diplomacy.DiplomaticModifiers
-import com.unciv.logic.civilization.managers.SpyAction
+import com.unciv.models.SpyAction
 import com.unciv.models.metadata.BaseRuleset
-import com.unciv.models.metadata.LocaleCode
+import com.unciv.models.metadata.GameSettings.LocaleCode
 import com.unciv.models.ruleset.Belief
 import com.unciv.models.ruleset.Building
+import com.unciv.models.ruleset.Event
 import com.unciv.models.ruleset.GlobalUniques
 import com.unciv.models.ruleset.PolicyBranch
 import com.unciv.models.ruleset.Quest
@@ -35,7 +36,7 @@ import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.ruleset.unit.BaseUnit
 import com.unciv.models.ruleset.unit.Promotion
 import com.unciv.models.ruleset.unit.UnitType
-import com.unciv.ui.components.KeyboardBinding
+import com.unciv.ui.components.input.KeyboardBinding
 import com.unciv.utils.Log
 import com.unciv.utils.debug
 import java.io.File
@@ -127,7 +128,7 @@ object TranslationFileWriter {
 
             linesToTranslate += "\n\n#################### Lines from spy actions #######################\n"
             for (spyAction in SpyAction.values()) {
-                linesToTranslate += "$spyAction = "
+                linesToTranslate += "${spyAction.displayString} = "
             }
 
             linesToTranslate += "\n\n#################### Lines from diplomatic modifiers #######################\n"
@@ -135,6 +136,9 @@ object TranslationFileWriter {
                 linesToTranslate += "${diplomaticModifier.text} = "
 
             linesToTranslate += "\n\n#################### Lines from key bindings #######################\n"
+            for (category in KeyboardBinding.Category.values()) {
+                linesToTranslate += "${category.label} = "
+            }
             for (binding in KeyboardBinding.values()) {
                 linesToTranslate += "${binding.label} = "
             }
@@ -252,6 +256,7 @@ object TranslationFileWriter {
 
     private fun writeLanguagePercentages(percentages: HashMap<String, Int>, modFolder: FileHandle? = null) {
         val output = percentages.asSequence()
+            .sortedBy { it.key }
             .joinToString("\n", postfix = "\n") { "${it.key} = ${it.value}" }
         getFileHandle(modFolder, TranslationFileReader.percentagesFileLocation)
             .writeString(output, false)
@@ -311,8 +316,7 @@ object TranslationFileWriter {
                 val filename = jsonFile.nameWithoutExtension()
 
                 val javaClass = getJavaClassByName(filename)
-                if (javaClass == this.javaClass)
-                    continue // unknown JSON, let's skip it
+                    ?: continue // unknown JSON, let's skip it
 
                 val array = json().fromJsonFile(javaClass, jsonFile.path())
 
@@ -346,7 +350,7 @@ object TranslationFileWriter {
         }
 
         fun submitString(string: String, unique: Unique) {
-            if (unique.hasFlag(UniqueFlag.HiddenToUsers))
+            if (unique.isHiddenToUsers())
                 return // We don't need to translate this at all, not user-visible
 
             val stringToTranslate = string.removeConditionals()
@@ -359,19 +363,23 @@ object TranslationFileWriter {
                 return
             }
 
+            // Do simpler parameter numbering when typed, as the code below is susceptible to problems with nested brackets - UniqueTypes don't have them (yet)!
+            if (unique.type != null) {
+                for ((index, typeList) in unique.type.parameterTypeMap.withIndex()) {
+                    if (typeList.none { it in translatableUniqueParameterTypes }) continue
+                    // Unknown/Comment parameter contents better be offered to translators too
+                    resultStrings.add("${unique.params[index]} = ")
+                }
+                resultStrings.add("${unique.type.getTranslatable()} = ")
+                return
+            }
+
             val parameterNames = ArrayList<String>()
-            for ((index, parameter) in unique.params.withIndex()) {
-                val parameterName =
-                    if (unique.type != null) {
-                        val possibleParameterTypes = unique.type.parameterTypeMap[index]
-                        // for multiple types. will look like "[unitName/buildingName]"
-                        possibleParameterTypes.joinToString("/") { it.parameterName }
-                    } else {
-                        UniqueParameterType.guessTypeForTranslationWriter(parameter, ruleset).parameterName
-                    }
+            for (parameter in unique.params) {
+                val parameterName = UniqueParameterType.guessTypeForTranslationWriter(parameter, ruleset).parameterName
                 parameterNames.addNumberedParameter(parameterName)
-                if (parameterName == UniqueParameterType.Unknown.parameterName)
-                    resultStrings.add("$parameter = ")  // Unknown parameter contents better be offered to translators too
+                if (translatableUniqueParameterTypes.none { it.parameterName == parameterName }) continue
+                resultStrings.add("$parameter = ")
             }
             resultStrings.add("${stringToTranslate.fillPlaceholders(*parameterNames.toTypedArray())} = ")
         }
@@ -420,7 +428,7 @@ object TranslationFileWriter {
                     // Promotion names are not uniques but since we did the "[unitName] ability"
                     // they need the "parameters" treatment too
                     // Same for victory milestones
-                    (field.name == "uniques" || field.name == "promotions" || field.name == "milestones")
+                    (field.name in fieldsToProcessParameters)
                             && (fieldValue is java.util.AbstractCollection<*>) ->
                         for (item in fieldValue)
                             if (item is String) submitString(item, Unique(item)) else serializeElement(item!!)
@@ -456,11 +464,25 @@ object TranslationFileWriter {
                 "RuinReward.uniques", "TerrainType.name",
                 "CityStateType.friendBonusUniques", "CityStateType.allyBonusUniques",
                 "Era.citySound",
+                "keyShortcut",
+                "Event.name" // Presently not shown anywhere
             )
 
             /** Specifies Enums where the name property _is_ translatable, by Class name */
             private val translatableEnumsSet = setOf("BeliefType")
 
+            /** Only these Unique parameter types will be offered as translatables - for all others it is expected their content
+             *  corresponds with a ruleset object name which will be made translatable by their actual json definition */
+            private val translatableUniqueParameterTypes = setOf(
+                UniqueParameterType.Unknown,
+                UniqueParameterType.Comment
+            )
+
+            private val fieldsToProcessParameters = setOf(
+                "uniques", "promotions", "milestones",
+                "triggeredUniques", "conditions"
+            )
+
             private fun isFieldTypeRelevant(type: Class<*>) =
                     type == String::class.java ||
                     type == java.util.ArrayList::class.java ||
@@ -481,13 +503,14 @@ object TranslationFileWriter {
                         (clazz.componentType?.simpleName ?: clazz.simpleName) + "." + field.name !in untranslatableFieldSet
             }
 
-            private fun getJavaClassByName(name: String): Class<Any> {
+            private fun getJavaClassByName(name: String): Class<Any>? {
                 return when (name) {
                     "Beliefs" -> emptyArray<Belief>().javaClass
                     "Buildings" -> emptyArray<Building>().javaClass
+                    "CityStateTypes" -> emptyArray<CityStateType>().javaClass
                     "Difficulties" -> emptyArray<Difficulty>().javaClass
                     "Eras" -> emptyArray<Era>().javaClass
-                    "Speeds" -> emptyArray<Speed>().javaClass
+                    "Events" -> emptyArray<Event>().javaClass
                     "GlobalUniques" -> GlobalUniques().javaClass
                     "Nations" -> emptyArray<Nation>().javaClass
                     "Policies" -> emptyArray<PolicyBranch>().javaClass
@@ -495,6 +518,7 @@ object TranslationFileWriter {
                     "Religions" -> emptyArray<String>().javaClass
                     "Ruins" -> emptyArray<RuinReward>().javaClass
                     "Specialists" -> emptyArray<Specialist>().javaClass
+                    "Speeds" -> emptyArray<Speed>().javaClass
                     "Techs" -> emptyArray<TechColumn>().javaClass
                     "Terrains" -> emptyArray<Terrain>().javaClass
                     "TileImprovements" -> emptyArray<TileImprovement>().javaClass
@@ -504,8 +528,7 @@ object TranslationFileWriter {
                     "Units" -> emptyArray<BaseUnit>().javaClass
                     "UnitTypes" -> emptyArray<UnitType>().javaClass
                     "VictoryTypes" -> emptyArray<Victory>().javaClass
-                    "CityStateTypes" -> emptyArray<CityStateType>().javaClass
-                    else -> this.javaClass // dummy value
+                    else -> null // dummy value
                 }
             }
         }
@@ -541,7 +564,7 @@ object TranslationFileWriter {
                 else -> translated
             }
             val localeCode = LocaleCode.valueOf(language.replace("_",""))
-            val path = fastlanePath + localeCode.language
+            val path = fastlanePath + (localeCode.trueLanguage ?: localeCode.language)
             File(path).mkdirs()
             File(path + File.separator + fileName).writeText(fileContent)
         }
