@@ -24,7 +24,6 @@ import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.nation.Nation
 import com.unciv.models.skins.SkinCache
 import com.unciv.models.tilesets.TileSetCache
-import com.unciv.ui.components.Fonts
 import com.unciv.ui.components.extensions.center
 import com.unciv.ui.components.extensions.centerX
 import com.unciv.ui.components.extensions.centerY
@@ -35,6 +34,7 @@ import com.unciv.ui.components.extensions.surroundWithCircle
 import com.unciv.ui.components.extensions.surroundWithThinCircle
 import com.unciv.ui.components.extensions.toGroup
 import com.unciv.ui.components.extensions.toLabel
+import com.unciv.ui.components.fonts.FontRulesetIcons
 import com.unciv.ui.screens.basescreen.BaseScreen
 import com.unciv.utils.debug
 import kotlin.math.atan2
@@ -43,7 +43,8 @@ import kotlin.math.min
 import kotlin.math.sqrt
 
 object ImageGetter {
-    private const val whiteDotLocation = "OtherIcons/whiteDot"
+    const val whiteDotLocation = "OtherIcons/whiteDot"
+    const val circleLocation = "OtherIcons/Circle"
 
     // We use texture atlases to minimize texture swapping - see https://yairm210.medium.com/the-libgdx-performance-guide-1d068a84e181
     lateinit var atlas: TextureAtlas
@@ -55,24 +56,21 @@ object ImageGetter {
     private val textureRegionDrawables = HashMap<String, TextureRegionDrawable>()
     private val ninePatchDrawables = HashMap<String, NinePatchDrawable>()
 
+    fun getSpecificAtlas(name: String): TextureAtlas? = atlases[name]
+
     fun resetAtlases() {
         atlases.values.forEach { it.dispose() }
         atlases.clear()
-        atlas = TextureAtlas("game.atlas")
-        atlases["game"] = atlas
     }
 
+    fun reloadImages() = setNewRuleset(ruleset)
+
     /** Required every time the ruleset changes, in order to load mod-specific images */
     fun setNewRuleset(ruleset: Ruleset) {
         ImageGetter.ruleset = ruleset
         textureRegionDrawables.clear()
-        // These are the drawables from the base game
-        for (region in atlas.regions) {
-            val drawable = TextureRegionDrawable(region)
-            textureRegionDrawables[region.name] = drawable
-        }
 
-        // Load base (except game.atlas which is already loaded)
+        // Load base
         loadModAtlases("", Gdx.files.internal(""))
 
         // These are from the mods
@@ -85,7 +83,7 @@ object ImageGetter {
         SkinCache.assembleSkinConfigs(ruleset.mods)
 
         BaseScreen.setSkin()
-        Fonts.addRulesetImages(ruleset)
+        FontRulesetIcons.addRulesetImages(ruleset)
     }
 
     /** Loads all atlas/texture files from a folder, as controlled by an Atlases.json */
@@ -93,17 +91,22 @@ object ImageGetter {
         // See #4993 - you can't .list() on a jar file, so the ImagePacker leaves us the list of actual atlases.
         val controlFile = folder.child("Atlases.json")
         val fileNames = (if (controlFile.exists()) json().fromJson(Array<String>::class.java, controlFile)
-            else emptyArray()).toMutableList()
-        if (mod.isNotEmpty()) fileNames += "game"
+            else emptyArray()).toMutableSet()
+        if (mod.isNotEmpty()) fileNames += "game"  // Backwards compatibility - when packed by 4.9.15+ this is already in the control file
         for (fileName in fileNames) {
             val file = folder.child("$fileName.atlas")
             if (!file.exists()) continue
             val extraAtlas = if (mod.isEmpty()) fileName else if (fileName == "game") mod else "$mod/$fileName"
             var tempAtlas = atlases[extraAtlas]  // fetch if cached
             if (tempAtlas == null) {
-                debug("Loading %s = %s", extraAtlas, file.path())
-                tempAtlas = TextureAtlas(file)  // load if not
-                atlases[extraAtlas] = tempAtlas  // cache the freshly loaded
+                try {
+                    debug("Loading %s = %s", extraAtlas, file.path())
+                    tempAtlas = TextureAtlas(file)  // load if not
+                    atlases[extraAtlas] = tempAtlas  // cache the freshly loaded
+                } catch (ex: Exception) {
+                    debug("Could not load file $file")
+                    continue
+                }
             }
             for (region in tempAtlas.regions) {
                 if (region.name.startsWith("Skins")) {
@@ -175,14 +178,33 @@ object ImageGetter {
     fun getWhiteDotDrawable() = textureRegionDrawables[whiteDotLocation]!!
     fun getDot(dotColor: Color) = getWhiteDot().apply { color = dotColor }
 
-    fun getExternalImage(fileName: String): Image {
+    /** Finds an image file under /ExtraImages/, including Mods (which can override builtin).
+     *  Extension can be included or is guessed as png/jpg.
+     *  @return `null` if no match found.
+     */
+    fun findExternalImage(name: String): FileHandle? {
+        val folders = ruleset.mods.asSequence().map { Gdx.files.local("mods/$it/ExtraImages") } +
+            sequenceOf(Gdx.files.internal("ExtraImages"))
+        val extensions = sequenceOf("", ".png", ".jpg")
+        return folders.flatMap { folder ->
+            extensions.map { folder.child(name + it) }
+        }.firstOrNull { it.exists() }
+    }
+
+    /** Loads an image on the fly - uncached Texture, not too fast. */
+    fun getExternalImage(file: FileHandle): Image {
         // Since these are not packed in an atlas, they have no scaling filter metadata and
         // default to Nearest filter, anisotropic level 1. Use Linear instead, helps
         // loading screen and Tutorial.WorldScreen quite a bit. More anisotropy barely helps.
-        val texture = Texture("ExtraImages/$fileName")
+        val texture = Texture(file)
         texture.setFilter(TextureFilter.Linear, TextureFilter.Linear)
         return ImageWithCustomSize(TextureRegion(texture))
     }
+    /** Loads an image from (assets)/ExtraImages, from the jar if Unciv runs packaged.
+     *  Cannot load ExtraImages from a Mod - use [findExternalImage] and the [getExternalImage](FileHandle) overload instead.
+     */
+    fun getExternalImage(fileName: String) =
+        getExternalImage(Gdx.files.internal("ExtraImages/$fileName"))
 
     fun getImage(fileName: String?): Image {
         return ImageWithCustomSize(getDrawable(fileName))
@@ -229,7 +251,7 @@ object ImageGetter {
     }
 
     fun getRandomNationPortrait(size: Float): Portrait {
-        return PortraitNation("Random", size)
+        return PortraitNation(Constants.random, size)
     }
 
     fun getUnitIcon(unitName: String, color: Color = Color.BLACK): Image {
@@ -274,19 +296,23 @@ object ImageGetter {
 
     fun getReligionIcon(iconName: String): Image { return getImage("ReligionIcons/$iconName") }
     fun getReligionPortrait(iconName: String, size: Float): Portrait {
-        if (religionIconExists(iconName)) {
+        if (religionIconExists(iconName))
             return PortraitReligion(iconName, size)
-        } else {
-            val typeName = ruleset.beliefs[iconName]?.type?.name
-            if (typeName != null && religionIconExists(typeName))
-                return PortraitReligion(typeName, size)
-        }
+        val typeName = ruleset.beliefs[iconName]?.type?.name
+        if (typeName != null && religionIconExists(typeName))
+            return PortraitReligion(typeName, size)
         return PortraitReligion(iconName, size)
     }
 
     fun religionIconExists(iconName: String) = imageExists("ReligionIcons/$iconName")
 
-    fun getCircle() = getImage("OtherIcons/Circle")
+    fun getCircleDrawable() = getDrawable(circleLocation)
+    fun getCircle() = getImage(circleLocation)
+    fun getCircle(color: Color = Color.WHITE, size: Float? = null) = getCircle().apply {
+        setColor(color)
+        if (size != null) setSize(size, size)
+    }
+
     fun getTriangle() = getImage("OtherIcons/Triangle")
 
     fun getRedCross(size: Float, alpha: Float): Actor {
@@ -306,7 +332,7 @@ object ImageGetter {
             addActor(cross)
         }
 
-    fun getArrowImage(align:Int = Align.right): Image {
+    fun getArrowImage(align: Int = Align.right): Image {
         val image = getImage("OtherIcons/ArrowRight")
         image.setOrigin(Align.center)
         if (align == Align.left) image.rotation = 180f
@@ -327,7 +353,7 @@ object ImageGetter {
                 .setProgress(progressColor, percentComplete, padding = progressPadding)
     }
 
-    class ProgressBar(width: Float, height: Float, val vertical: Boolean = true):Group() {
+    class ProgressBar(width: Float, height: Float, val vertical: Boolean = true) : Group() {
 
         var primaryPercentage: Float = 0f
         var secondaryPercentage: Float = 0f
