@@ -1,19 +1,25 @@
 package com.unciv.ui.popups.options
 
+import com.badlogic.gdx.Gdx
 import com.badlogic.gdx.scenes.scene2d.ui.Table
+import com.badlogic.gdx.scenes.scene2d.ui.TextButton.TextButtonStyle
 import com.unciv.GUI
 import com.unciv.UncivGame
-import com.unciv.logic.files.UncivFiles
+import com.unciv.logic.UncivShowableException
 import com.unciv.logic.files.MapSaver
+import com.unciv.logic.files.UncivFiles
 import com.unciv.models.ruleset.RulesetCache
 import com.unciv.models.ruleset.tile.ResourceType
-import com.unciv.ui.screens.basescreen.BaseScreen
-import com.unciv.ui.components.UncivSlider
 import com.unciv.ui.components.UncivTextField
-import com.unciv.ui.components.input.onClick
+import com.unciv.ui.components.extensions.addSeparator
 import com.unciv.ui.components.extensions.toCheckBox
 import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.components.extensions.toTextButton
+import com.unciv.ui.components.input.onClick
+import com.unciv.ui.components.widgets.UncivSlider
+import com.unciv.ui.popups.ToastPopup
+import com.unciv.ui.screens.basescreen.BaseScreen
+import com.unciv.utils.Concurrency
 import com.unciv.utils.DebugUtils
 
 fun debugTab(
@@ -58,9 +64,6 @@ fun debugTab(
             curGameInfo.gameParameters.godMode = it
         }).colspan(2).row()
     }
-    add("Enable espionage option".toCheckBox(game.settings.enableEspionageOption) {
-        game.settings.enableEspionageOption = it
-    }).colspan(2).row()
 
     add("Save games compressed".toCheckBox(UncivFiles.saveZipped) {
         UncivFiles.saveZipped = it
@@ -69,21 +72,10 @@ fun debugTab(
         MapSaver.saveZipped = it
     }).colspan(2).row()
 
-    if (GUI.keyboardAvailable) {
-        add("Show keyboard bindings".toCheckBox(optionsPopup.enableKeyBindingsTab) {
-            optionsPopup.enableKeyBindingsTab = it
-            optionsPopup.showOrHideKeyBindings()
-        }).colspan(2).row()
-    }
-
     add("Gdx Scene2D debug".toCheckBox(BaseScreen.enableSceneDebug) {
         BaseScreen.enableSceneDebug = it
     }).colspan(2).row()
 
-    add("Allow untyped Uniques in mod checker".toCheckBox(RulesetCache.modCheckerAllowUntypedUniques) {
-        RulesetCache.modCheckerAllowUntypedUniques = it
-    }).colspan(2).row()
-
     add(Table().apply {
         add("Unique misspelling threshold".toLabel()).left().fillX()
         add(
@@ -119,10 +111,31 @@ fun debugTab(
             tile.resourceAmount = 999
             // Debug option, so if it crashes on this that's relatively fine
             // If this becomes a problem, check if such an improvement exists and otherwise plop down a great improvement or so
-            tile.changeImprovement(resource.getImprovements().first())
+            tile.setImprovement(resource.getImprovements().first())
         }
         curGameInfo.getCurrentPlayerCivilization().cache.updateSightAndResources()
         GUI.setUpdateWorldOnNextRender()
     }
     add(giveResourcesButton).colspan(2).row()
+
+    add("Load online multiplayer game as hotseat from clipboard".toTextButton().onClick {
+        // Code duplication : LoadGameScreen.getLoadFromClipboardButton
+        Concurrency.run {
+            try {
+                val clipboardContentsString = Gdx.app.clipboard.contents.trim()
+                val loadedGame = UncivFiles.gameInfoFromString(clipboardContentsString)
+                loadedGame.gameParameters.isOnlineMultiplayer = false
+                optionsPopup.game.loadGame(loadedGame, callFromLoadScreen =  true)
+                optionsPopup.close()
+            } catch (ex: Exception) {
+                ToastPopup(ex.message ?: ex::class.java.simpleName, optionsPopup.stageToShowOn)
+            }
+        }
+    }).colspan(2).row()
+
+    addSeparator()
+    add("* Crash Unciv! *".toTextButton(skin.get("negative", TextButtonStyle::class.java)).onClick {
+        throw UncivShowableException("Intentional crash")
+    }).colspan(2).row()
+    addSeparator()
 }
