@@ -7,23 +7,21 @@ import com.badlogic.gdx.scenes.scene2d.ui.Table
 import com.badlogic.gdx.utils.Align
 import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.RulesetCache
-import com.unciv.models.ruleset.validation.RulesetError
-import com.unciv.models.ruleset.validation.RulesetErrorSeverity
-import com.unciv.models.ruleset.validation.RulesetValidator
 import com.unciv.models.ruleset.unique.Unique
-import com.unciv.models.ruleset.unique.UniqueType
+import com.unciv.models.ruleset.validation.RulesetErrorSeverity
+import com.unciv.models.ruleset.validation.UniqueValidator
 import com.unciv.models.translations.tr
-import com.unciv.ui.components.ExpanderTab
-import com.unciv.ui.components.TabbedPager
 import com.unciv.ui.components.extensions.surroundWithCircle
 import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.components.extensions.toTextButton
 import com.unciv.ui.components.input.onChange
 import com.unciv.ui.components.input.onClick
+import com.unciv.ui.components.widgets.ExpanderTab
+import com.unciv.ui.components.widgets.TabbedPager
+import com.unciv.ui.components.widgets.TranslatedSelectBox
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.popups.ToastPopup
 import com.unciv.ui.screens.basescreen.BaseScreen
-import com.unciv.ui.screens.newgamescreen.TranslatedSelectBox
 import com.unciv.utils.Concurrency
 import com.unciv.utils.Log
 import com.unciv.utils.debug
@@ -75,11 +73,14 @@ class ModCheckTab(
         runAction()
     }
 
-    fun runModChecker(base: String = MOD_CHECK_WITHOUT_BASE) {
+    private fun runModChecker(base: String = MOD_CHECK_WITHOUT_BASE) {
 
         modCheckFirstRun = false
         if (modCheckBaseSelect == null) return
 
+        val openedExpanderTitles = modCheckResultTable.children.filterIsInstance<ExpanderTab>()
+            .filter { it.isOpen }.map { it.title }.toSet()
+
         modCheckResultTable.clear()
 
         val rulesetErrors = RulesetCache.loadRulesets()
@@ -94,7 +95,7 @@ class ModCheckTab(
         modCheckBaseSelect!!.isDisabled = true
 
         Concurrency.run("ModChecker") {
-            for (mod in RulesetCache.values.sortedBy { it.name }) {
+            for (mod in RulesetCache.values.sortedBy { it.name }.sortedByDescending { it.name in openedExpanderTitles }) {
                 if (base != MOD_CHECK_WITHOUT_BASE && mod.modOptions.isBaseRuleset) continue
 
                 val modLinks =
@@ -102,8 +103,8 @@ class ModCheckTab(
                     else RulesetCache.checkCombinedModLinks(linkedSetOf(mod.name), base, tryFixUnknownUniques = true)
                 modLinks.sortByDescending { it.errorSeverityToReport }
                 val noProblem = !modLinks.isNotOK()
-                if (modLinks.isNotEmpty()) modLinks += RulesetError("", RulesetErrorSeverity.OK)
-                if (noProblem) modLinks += RulesetError("No problems found.".tr(), RulesetErrorSeverity.OK)
+                if (modLinks.isNotEmpty()) modLinks.add("", RulesetErrorSeverity.OK, sourceObject = null)
+                if (noProblem) modLinks.add("No problems found.".tr(), RulesetErrorSeverity.OK, sourceObject = null)
 
                 launchOnGLThread {
                     // When the options popup is already closed before this postRunnable is run,
@@ -123,7 +124,7 @@ class ModCheckTab(
                         .apply { color = Color.BLACK }
                         .surroundWithCircle(30f, color = iconColor)
 
-                    val expanderTab = ExpanderTab(mod.name, icon = icon, startsOutOpened = false) {
+                    val expanderTab = ExpanderTab(mod.name, icon = icon, startsOutOpened = mod.name in openedExpanderTitles) {
                         it.defaults().align(Align.left)
                         if (!noProblem && mod.folderLocation != null) {
                             val replaceableUniques = getDeprecatedReplaceableUniques(mod)
@@ -200,22 +201,22 @@ class ModCheckTab(
                 uniqueReplacementText += " <${conditional.text}>"
             val replacementUnique = Unique(uniqueReplacementText)
 
-            val modInvariantErrors = RulesetValidator(mod).checkUnique(
+            val modInvariantErrors = UniqueValidator(mod).checkUnique(
                 replacementUnique,
                 false,
                 null,
-                UniqueType.UniqueComplianceErrorSeverity.RulesetInvariant
+                true
             )
             for (error in modInvariantErrors)
                 Log.error("ModInvariantError: %s - %s", error.text, error.errorSeverityToReport)
             if (modInvariantErrors.isNotEmpty()) continue // errors means no autoreplace
 
             if (mod.modOptions.isBaseRuleset) {
-                val modSpecificErrors = RulesetValidator(mod).checkUnique(
+                val modSpecificErrors = UniqueValidator(mod).checkUnique(
                     replacementUnique,
                     false,
                     null,
-                    UniqueType.UniqueComplianceErrorSeverity.RulesetInvariant
+                    true
                 )
                 for (error in modSpecificErrors)
                     Log.error("ModSpecificError: %s - %s", error.text, error.errorSeverityToReport)
@@ -255,7 +256,7 @@ class ModCheckTab(
             file.writeString(newFileText, false)
         }
         val toastText = "Uniques updated!"
-        ToastPopup(toastText, screen).open(true)
+        ToastPopup(toastText, screen)
         runModChecker()
     }
 }
