@@ -5,23 +5,24 @@ import com.badlogic.gdx.Gdx
 import com.badlogic.gdx.scenes.scene2d.ui.Table
 import com.badlogic.gdx.scenes.scene2d.ui.TextField
 import com.unciv.UncivGame
+import com.unciv.logic.files.IMediaFinder
 import com.unciv.logic.multiplayer.OnlineMultiplayer
 import com.unciv.logic.multiplayer.storage.FileStorageRateLimitReached
 import com.unciv.logic.multiplayer.storage.MultiplayerAuthException
 import com.unciv.models.UncivSound
-import com.unciv.models.metadata.GameSetting
 import com.unciv.models.metadata.GameSettings
+import com.unciv.models.metadata.GameSettings.GameSetting
 import com.unciv.models.ruleset.RulesetCache
 import com.unciv.ui.components.UncivTextField
 import com.unciv.ui.components.extensions.addSeparator
 import com.unciv.ui.components.extensions.brighten
 import com.unciv.ui.components.extensions.format
 import com.unciv.ui.components.extensions.isEnabled
-import com.unciv.ui.components.input.onChange
-import com.unciv.ui.components.input.onClick
 import com.unciv.ui.components.extensions.toGdxArray
 import com.unciv.ui.components.extensions.toLabel
 import com.unciv.ui.components.extensions.toTextButton
+import com.unciv.ui.components.input.onChange
+import com.unciv.ui.components.input.onClick
 import com.unciv.ui.popups.AuthPopup
 import com.unciv.ui.popups.Popup
 import com.unciv.ui.popups.options.SettingsSelect.SelectItem
@@ -76,19 +77,18 @@ fun multiplayerTab(
         turnCheckerSelect = null
     }
 
+    val sounds = IMediaFinder.LabeledSounds().getLabeledSounds()
     addSelectAsSeparateTable(tab, SettingsSelect("Sound notification for when it's your turn in your currently open game:",
-        createNotificationSoundOptions(),
+        sounds,
         GameSetting.MULTIPLAYER_CURRENT_GAME_TURN_NOTIFICATION_SOUND,
         settings
-    )
-    )
+    ))
 
     addSelectAsSeparateTable(tab, SettingsSelect("Sound notification for when it's your turn in any other game:",
-        createNotificationSoundOptions(),
+        sounds,
         GameSetting.MULTIPLAYER_OTHER_GAME_TURN_NOTIFICATION_SOUND,
         settings
-    )
-    )
+    ))
 
     addSeparator(tab)
 
@@ -99,35 +99,6 @@ fun multiplayerTab(
     return tab
 }
 
-private fun createNotificationSoundOptions(): List<SelectItem<UncivSound>> = listOf(
-    SelectItem("None", UncivSound.Silent),
-    SelectItem("Notification [1]", UncivSound.Notification1),
-    SelectItem("Notification [2]", UncivSound.Notification2),
-    SelectItem("Chimes", UncivSound.Chimes),
-    SelectItem("Choir", UncivSound.Choir),
-    SelectItem("Buy", UncivSound.Coin),
-    SelectItem("Create", UncivSound.Construction),
-    SelectItem("Fortify", UncivSound.Fortify),
-    SelectItem("Pick a tech", UncivSound.Paper),
-    SelectItem("Adopt policy", UncivSound.Policy),
-    SelectItem("Promote", UncivSound.Promote),
-    SelectItem("Set up", UncivSound.Setup),
-    SelectItem("Swap units", UncivSound.Swap),
-    SelectItem("Upgrade", UncivSound.Upgrade),
-    SelectItem("Bombard", UncivSound.Bombard)
-) + buildUnitAttackSoundOptions()
-
-private fun buildUnitAttackSoundOptions(): List<SelectItem<UncivSound>> {
-    return RulesetCache.getSortedBaseRulesets()
-        .asSequence()
-        .mapNotNull(RulesetCache::get)
-        .flatMap { it.units.values }
-        .filter { it.attackSound != null && it.attackSound != "nuke" } // much too long for a notification
-        .distinctBy { it.attackSound }
-        .map { SelectItem("[${it.name}] Attack Sound", UncivSound(it.attackSound!!)) }
-        .toList()
-}
-
 private fun addMultiplayerServerOptions(
     tab: Table,
     optionsPopup: OptionsPopup,
@@ -151,18 +122,14 @@ private fun addMultiplayerServerOptions(
         multiplayerServerTextField.text = Gdx.app.clipboard.contents
         }).colspan(2).row()
     multiplayerServerTextField.onChange {
+        fixTextFieldUrlOnType(multiplayerServerTextField)
+        // we can't trim on 'fixTextFieldUrlOnType' for reasons
+        settings.multiplayer.server = multiplayerServerTextField.text.trimEnd('/')
+
         val isCustomServer = OnlineMultiplayer.usesCustomServer()
         connectionToServerButton.isEnabled = isCustomServer
 
         for (refreshSelect in toUpdate) refreshSelect.update(isCustomServer)
-
-        if (isCustomServer) {
-            fixTextFieldUrlOnType(multiplayerServerTextField)
-            // we can't trim on 'fixTextFieldUrlOnType' for reasons
-            settings.multiplayer.server = multiplayerServerTextField.text.trimEnd('/')
-        } else {
-            settings.multiplayer.server = multiplayerServerTextField.text
-        }
     }
 
     serverIpTable.add(multiplayerServerTextField)
