@@ -7,17 +7,21 @@ import com.unciv.GUI
 import com.unciv.UncivGame
 import com.unciv.models.metadata.BaseRuleset
 import com.unciv.models.ruleset.RulesetCache
-import com.unciv.ui.components.TabbedPager
 import com.unciv.ui.components.extensions.areSecretKeysPressed
 import com.unciv.ui.components.extensions.center
+import com.unciv.ui.components.extensions.getCloseButton
 import com.unciv.ui.components.extensions.toCheckBox
+import com.unciv.ui.components.widgets.TabbedPager
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.popups.Popup
+import com.unciv.ui.popups.hasOpenPopups
 import com.unciv.ui.screens.basescreen.BaseScreen
+import com.unciv.ui.screens.basescreen.RecreateOnResize
 import com.unciv.ui.screens.mainmenuscreen.MainMenuScreen
 import com.unciv.ui.screens.worldscreen.WorldScreen
 import com.unciv.utils.Concurrency
 import com.unciv.utils.withGLContext
+import kotlinx.coroutines.delay
 import kotlin.reflect.KMutableProperty0
 
 /**
@@ -28,6 +32,7 @@ import kotlin.reflect.KMutableProperty0
 class OptionsPopup(
     screen: BaseScreen,
     private val selectPage: Int = defaultPage,
+    withDebug: Boolean = false,
     private val onClose: () -> Unit = {}
 ) : Popup(screen.stage, /** [TabbedPager] handles scrolling */ scrollable = Scrollability.None) {
 
@@ -37,20 +42,15 @@ class OptionsPopup(
     val selectBoxMinWidth: Float
     private val tabMinWidth: Float
 
-    private var keyBindingsTab: KeyBindingsTab? = null
-    /** Enable the still experimental Keyboard Bindings page in OptionsPopup */
-    var enableKeyBindingsTab: Boolean = false
-
     //endregion
 
     companion object {
         const val defaultPage = 2  // Gameplay
-
-        const val keysTabCaption = "Keys"
-        const val keysTabBeforeCaption = "Advanced"
     }
 
     init {
+        clickBehindToClose = true
+
         if (settings.addCompletedTutorialTask("Open the options table"))
             (screen as? WorldScreen)?.shouldUpdate = true
 
@@ -61,7 +61,7 @@ class OptionsPopup(
             selectBoxMinWidth = if (stage.width < 600f) 200f else 240f
             tabMaxWidth = if (isPortrait()) stage.width - 10f else 0.8f * stage.width
             tabMinWidth = 0.6f * stage.width
-            tabMaxHeight = (if (isPortrait()) 0.7f else 0.8f) * stage.height
+            tabMaxHeight = 0.8f * stage.height
         }
         tabs = TabbedPager(
             tabMinWidth, tabMaxWidth, 0f, tabMaxHeight,
@@ -84,9 +84,14 @@ class OptionsPopup(
             gameplayTab(this),
             ImageGetter.getImage("OtherIcons/Options"), 24f
         )
+        tabs.addPage(
+            "AutoPlay",
+            autoPlayTab(this),
+            ImageGetter.getImage("OtherIcons/NationSwap"), 24f
+        )
         tabs.addPage(
             "Language",
-            languageTab(this, ::reloadWorldAndOptions),
+            LanguageTab(this, ::reloadWorldAndOptions),
             ImageGetter.getImage("FlagIcons/${settings.language}"), 24f
         )
         tabs.addPage(
@@ -100,9 +105,17 @@ class OptionsPopup(
             ImageGetter.getImage("OtherIcons/Multiplayer"), 24f
         )
 
+        if (GUI.keyboardAvailable) {
+            tabs.addPage(
+                "Keys",
+                KeyBindingsTab(this, tabMinWidth - 40f),  // 40 = padding
+                ImageGetter.getImage("OtherIcons/Keyboard"), 24f
+            )
+        }
+
         tabs.addPage(
             "Advanced",
-            advancedTab(this, ::reloadWorldAndOptions),
+            AdvancedTab(this, ::reloadWorldAndOptions),
             ImageGetter.getImage("OtherIcons/Settings"), 24f
         )
 
@@ -110,21 +123,18 @@ class OptionsPopup(
             val content = ModCheckTab(screen)
             tabs.addPage("Locate mod errors", content, ImageGetter.getImage("OtherIcons/Mods"), 24f)
         }
-        if (Gdx.input.areSecretKeysPressed()) {
-            tabs.addPage("Debug", debugTab(this), ImageGetter.getImage("OtherIcons/SecretOptions"), 24f, secret = true)
+        if (withDebug || Gdx.input.areSecretKeysPressed()) {
+            tabs.addPage("Debug", debugTab(this), ImageGetter.getImage("OtherIcons/SecretOptions"), 24f)
         }
 
-        addCloseButton {
+        tabs.decorateHeader(getCloseButton {
             screen.game.musicController.onChange(null)
             center(screen.stage)
-            keyBindingsTab?.save()
+            tabs.selectPage(-1, false)
             settings.save()
-            onClose()
-        }.padBottom(10f)
-
-        if (GUI.keyboardAvailable) {
-            showOrHideKeyBindings()  // Do this late because it looks for the page to insert before
-        }
+            onClose() // activate the passed 'on close' callback
+            close() // close this popup
+        })
 
         pack() // Needed to show the background.
         center(screen.stage)
@@ -133,7 +143,6 @@ class OptionsPopup(
     override fun setVisible(visible: Boolean) {
         super.setVisible(visible)
         if (!visible) return
-        tabs.askForPassword(secretHashCode = 2747985)
         if (tabs.activePage < 0) tabs.selectPage(selectPage)
     }
 
@@ -159,7 +168,23 @@ class OptionsPopup(
         }
     }
 
-    fun addCheckbox(table: Table, text: String, initialState: Boolean, updateWorld: Boolean = false, newRow: Boolean = true, action: ((Boolean) -> Unit)) {
+    /** Call if an option change might trigger a Screen.resize
+     *
+     *  Does nothing if any Popup (which can only be this one) is still open after a short delay and context yield.
+     *  Reason: A resize might relaunch the parent screen ([MainMenuScreen] is [RecreateOnResize]) and thus close this Popup.
+     */
+    internal fun reopenAfterDisplayLayoutChange() {
+        Concurrency.run("Reload from options") {
+            delay(100)
+            withGLContext {
+                val screen = UncivGame.Current.screen ?: return@withGLContext
+                if (screen.hasOpenPopups()) return@withGLContext // e.g. Orientation auto to fixed while auto is already the new orientation
+                screen.openOptionsPopup(tabs.activePage)
+            }
+        }
+    }
+
+    internal fun addCheckbox(table: Table, text: String, initialState: Boolean, updateWorld: Boolean = false, newRow: Boolean = true, action: ((Boolean) -> Unit)) {
         val checkbox = text.toCheckBox(initialState) {
             action(it)
             val worldScreen = GUI.getWorldScreenIfActive()
@@ -169,7 +194,7 @@ class OptionsPopup(
         else table.add(checkbox).left()
     }
 
-    fun addCheckbox(
+    internal fun addCheckbox(
         table: Table,
         text: String,
         settingsProperty: KMutableProperty0<Boolean>,
@@ -181,23 +206,4 @@ class OptionsPopup(
             settingsProperty.set(it)
         }
     }
-
-    internal fun showOrHideKeyBindings() {
-        // At the moment, the Key bindings Tab exists only on-demand. To refactor it back to permanent,
-        // move the `keyBindingsTab =` line and addPage call to before the Advanced Tab creation,
-        // then delete this function, delete the enableKeyBindingsTab flag and clean up what is flagged by the compiler as missing or unused.
-        val existingIndex = tabs.getPageIndex(keysTabCaption)
-        if (enableKeyBindingsTab && existingIndex < 0) {
-            if (keyBindingsTab == null)
-                keyBindingsTab = KeyBindingsTab(this, tabMinWidth - 40f)  // 40 = padding
-            val beforeIndex = tabs.getPageIndex(keysTabBeforeCaption)
-            tabs.addPage(
-                keysTabCaption, keyBindingsTab,
-                ImageGetter.getImage("OtherIcons/Keyboard"), 24f,
-                insertBefore = beforeIndex
-            )
-        } else if (!enableKeyBindingsTab && existingIndex >= 0) {
-            tabs.removePage(existingIndex)
-        }
-    }
 }
