@@ -17,17 +17,28 @@ import com.badlogic.gdx.scenes.scene2d.utils.Drawable
 import com.badlogic.gdx.utils.viewport.ExtendViewport
 import com.unciv.UncivGame
 import com.unciv.models.TutorialTrigger
+import com.unciv.models.metadata.BaseRuleset
+import com.unciv.models.ruleset.Ruleset
+import com.unciv.models.ruleset.RulesetCache
 import com.unciv.models.skins.SkinStrings
-import com.unciv.ui.components.Fonts
-import com.unciv.ui.components.KeyShortcutDispatcher
-import com.unciv.ui.components.extensions.DispatcherVetoResult
-import com.unciv.ui.components.extensions.DispatcherVetoer
-import com.unciv.ui.components.extensions.installShortcutDispatcher
 import com.unciv.ui.components.extensions.isNarrowerThan4to3
+import com.unciv.ui.components.fonts.Fonts
+import com.unciv.ui.components.input.DispatcherVetoer
+import com.unciv.ui.components.input.KeyShortcutDispatcher
+import com.unciv.ui.components.input.KeyShortcutDispatcherVeto
+import com.unciv.ui.components.input.installShortcutDispatcher
+import com.unciv.ui.components.input.keyShortcuts
+import com.unciv.ui.crashhandling.CrashScreen
 import com.unciv.ui.images.ImageGetter
+import com.unciv.ui.popups.Popup
 import com.unciv.ui.popups.activePopup
 import com.unciv.ui.popups.options.OptionsPopup
-import com.unciv.ui.tutorials.TutorialController
+import com.unciv.ui.screens.civilopediascreen.CivilopediaScreen
+import com.unciv.ui.screens.mainmenuscreen.MainMenuScreen
+
+// Both `this is CrashScreen` and `this::createPopupBasedDispatcherVetoer` are flagged.
+// First - not a leak; second - passes out a pure function
+@Suppress("LeakingThis")
 
 abstract class BaseScreen : Screen {
 
@@ -38,7 +49,7 @@ abstract class BaseScreen : Screen {
 
     /**
      * Keyboard shortcuts global to the screen. While this is public and can be modified,
-     * you most likely should use [keyShortcuts][Actor.keyShortcuts] on appropriate [Actor] instead.
+     * you most likely should use [keyShortcuts] on the appropriate [Actor] instead.
      */
     val globalShortcuts = KeyShortcutDispatcher()
 
@@ -49,28 +60,27 @@ abstract class BaseScreen : Screen {
         /** The ExtendViewport sets the _minimum_(!) world size - the actual world size will be larger, fitted to screen/window aspect ratio. */
         stage = UncivStage(ExtendViewport(height, height))
 
-        if (enableSceneDebug) {
+        if (enableSceneDebug && this !is CrashScreen) {
             stage.setDebugUnderMouse(true)
             stage.setDebugTableUnderMouse(true)
             stage.setDebugParentUnderMouse(true)
+            stage.mouseOverDebug = true
         }
 
-        stage.installShortcutDispatcher(globalShortcuts, this::createPopupBasedDispatcherVetoer)
+        @Suppress("LeakingThis")
+        stage.installShortcutDispatcher(globalShortcuts, this::createDispatcherVetoer)
     }
 
-    private fun createPopupBasedDispatcherVetoer(): DispatcherVetoer? {
+    /** Hook allowing derived Screens to supply a key shortcut vetoer that can exclude parts of the
+     *  Stage Actor hierarchy from the search. Only called if no [Popup] is active.
+     *  @see installShortcutDispatcher
+     */
+    open fun getShortcutDispatcherVetoer(): DispatcherVetoer? = null
+
+    private fun createDispatcherVetoer(): DispatcherVetoer? {
         val activePopup = this.activePopup
-        if (activePopup == null)
-            return null
-        else {
-            // When any popup is active, disable all shortcuts on actor outside the popup
-            // and also the global shortcuts on the screen itself.
-            return { associatedActor: Actor?, _: KeyShortcutDispatcher? ->
-                when { associatedActor == null -> DispatcherVetoResult.Skip
-                       associatedActor.isDescendantOf(activePopup) -> DispatcherVetoResult.Accept
-                       else -> DispatcherVetoResult.SkipWithChildren }
-            }
-        }
+            ?: return getShortcutDispatcherVetoer()
+        return KeyShortcutDispatcherVeto.createPopupBasedDispatcherVetoer(activePopup)
     }
 
     override fun show() {}
@@ -133,7 +143,7 @@ abstract class BaseScreen : Screen {
                 add("Nativefont", Fonts.font, BitmapFont::class.java)
                 add("RoundedEdgeRectangle", skinStrings.getUiBackground("", skinStrings.roundedEdgeRectangleShape), Drawable::class.java)
                 add("Rectangle", ImageGetter.getDrawable(""), Drawable::class.java)
-                add("Circle", ImageGetter.getDrawable("OtherIcons/Circle").apply { setMinSize(20f, 20f) }, Drawable::class.java)
+                add("Circle", ImageGetter.getCircleDrawable().apply { setMinSize(20f, 20f) }, Drawable::class.java)
                 add("Scrollbar", ImageGetter.getDrawable("").apply { setMinSize(10f, 10f) }, Drawable::class.java)
                 add("RectangleWithOutline",
                     skinStrings.getUiBackground("", skinStrings.rectangleWithOutlineShape), Drawable::class.java)
@@ -169,9 +179,35 @@ abstract class BaseScreen : Screen {
     /** @return `true` if the screen is narrower than 4:3 landscape */
     fun isNarrowerThan4to3() = stage.isNarrowerThan4to3()
 
-    fun openOptionsPopup(startingPage: Int = OptionsPopup.defaultPage, onClose: () -> Unit = {}) {
-        OptionsPopup(this, startingPage, onClose).open(force = true)
+    open fun openOptionsPopup(startingPage: Int = OptionsPopup.defaultPage, withDebug: Boolean = false, onClose: () -> Unit = {}) {
+        OptionsPopup(this, startingPage, withDebug, onClose).open(force = true)
     }
+
+    /**
+     *  Determine a Ruleset for Civilopedia to use (remember: it is supposed to work without a running game loaded)
+     *
+     *  - `open` as some important screens are supposed to provide directly.
+     *  - The default implementation searches using the [screenStack][UncivGame.screenStack] for a source of a Ruleset and returns Civ_V_GnK when that fails.
+     *  - Care must be taken in [PickerScreen][com.unciv.ui.screens.pickerscreens.PickerScreen] derivates - they will default to the searching implementation, but often could do the task more efficiently.
+     */
+    open fun getCivilopediaRuleset(): Ruleset {
+        if (game.worldScreen != null) return game.worldScreen!!.gameInfo.ruleset
+        val mainMenuScreen = game.getScreensOfType(MainMenuScreen::class).firstOrNull()
+        if (mainMenuScreen != null) return mainMenuScreen.getCivilopediaRuleset()
+        return RulesetCache[BaseRuleset.Civ_V_GnK.fullName]!!
+    }
+
+    /** Opens Civilopedia
+     *
+     *  It's an open method of BaseScreen because especially MainMenuScreen has cleanup things to do first.
+     *  @see getCivilopediaRuleset
+     */
+    open fun openCivilopedia(link: String = "") = openCivilopedia(getCivilopediaRuleset(), link)
+
+    /** Helper for the [openCivilopedia] (link: String) overload to use
+     *  - Note: At the time of wrinting, this was the ***only*** CivilopediaScreen constructor call outside itself
+     */
+    fun openCivilopedia(ruleset: Ruleset, link: String = "") = game.pushScreen(CivilopediaScreen(ruleset, link = link))
 }
 
 interface RecreateOnResize {
