@@ -2,10 +2,10 @@ package com.unciv.ui.screens.pickerscreens
 
 import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.models.translations.tr
+import com.unciv.ui.components.extensions.surroundWithCircle
 import com.unciv.ui.images.ImageGetter
 import com.unciv.ui.popups.AskTextPopup
 import com.unciv.ui.screens.basescreen.BaseScreen
-import com.unciv.ui.components.extensions.surroundWithCircle
 
 class UnitRenamePopup(val screen: BaseScreen, val unit: MapUnit, val actionOnClose: ()->Unit) {
     init {
@@ -13,7 +13,7 @@ class UnitRenamePopup(val screen: BaseScreen, val unit: MapUnit, val actionOnClo
             screen,
             label = "Choose name for [${unit.baseUnit.name}]",
             icon = ImageGetter.getUnitIcon(unit.name).surroundWithCircle(80f),
-            defaultText = unit.instanceName ?: unit.baseUnit.name.tr(),
+            defaultText = unit.instanceName ?: unit.baseUnit.name.tr(hideIcons = true),
             validate = { it != unit.name },
             actionOnOk = { userInput ->
                 //If the user inputs an empty string, clear the unit instanceName so the base name is used
