@@ -9,10 +9,10 @@ import com.badlogic.gdx.scenes.scene2d.ui.TextButton
 import com.badlogic.gdx.utils.Align
 import com.unciv.logic.files.UncivFiles
 import com.unciv.ui.images.ImageGetter
-import com.unciv.ui.components.AutoScrollPane
+import com.unciv.ui.components.widgets.AutoScrollPane
 import com.unciv.ui.screens.basescreen.BaseScreen
-import com.unciv.ui.components.extensions.keyShortcuts
-import com.unciv.ui.components.extensions.onClick
+import com.unciv.ui.components.input.keyShortcuts
+import com.unciv.ui.components.input.onClick
 import com.unciv.utils.Concurrency
 import com.unciv.utils.launchOnGLThread
 
@@ -21,7 +21,6 @@ import com.unciv.utils.launchOnGLThread
 /** A widget holding TextButtons vertically in a Table contained in a ScrollPane, with methods to
  *  hold file names and FileHandle's in those buttons. Used to display existing saves in the Load and Save game dialogs.
  *
- *  @param shorcutScreen optionally pass in a [BaseScreen] where global keyboard shorcuts should be installed.
  *  @param existingSavesTable exists here for coder convenience. No need to touch.
  */
 class VerticalFileListScrollPane(
@@ -60,7 +59,7 @@ class VerticalFileListScrollPane(
         val loadImage = ImageGetter.getImage("OtherIcons/Load")
         loadImage.setSize(50f, 50f) // So the origin sets correctly
         loadImage.setOrigin(Align.center)
-        val loadAnimation = Actions.repeat(Int.MAX_VALUE, Actions.rotateBy(360f, 2f))
+        val loadAnimation = Actions.forever(Actions.rotateBy(360f, 2f))
         loadImage.addAction(loadAnimation)
         existingSavesTable.add(loadImage).size(50f).center()
 
