@@ -1,8 +1,10 @@
+@file:Suppress("UNUSED_VARIABLE")  // These are tests and the names serve readability
+
 package com.unciv.logic.civilization
 
 import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.testing.GdxTestRunner
-import com.unciv.uniques.TestGame
+import com.unciv.testing.TestGame
 import org.junit.Assert
 import org.junit.Before
 import org.junit.Test
@@ -36,8 +38,8 @@ class CapitalConnectionsFinderTests {
         ourCiv.tech.addTechnology(testGame.ruleset.tileImprovements[RoadStatus.Railroad.name]?.techRequired!!)
     }
 
-    private fun createMedium(from:Int, to: Int, type: RoadStatus) {
-        for (i in from..to){
+    private fun createMedium(from: Int, to: Int, type: RoadStatus) {
+        for (i in from..to) {
             val tile = testGame.tileMap[0, i]
             tile.roadStatus = type
         }
