@@ -7,6 +7,7 @@ import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.models.ruleset.BeliefType
 import com.unciv.models.stats.Stats
 import com.unciv.testing.GdxTestRunner
+import com.unciv.testing.TestGame
 import org.junit.Assert
 import org.junit.Before
 import org.junit.Test
@@ -29,11 +30,10 @@ class GlobalUniquesTests {
     @Test
     fun statsOnBuilding() {
         val civInfo = game.addCiv()
-        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val city = game.addCity(civInfo, tile, true)
-        val buildingName = game.createBuilding("[+1 Food]").name
+        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true)
+        val building = game.createBuilding("[+1 Food]")
 
-        city.cityConstructions.addBuilding(buildingName)
+        city.cityConstructions.addBuilding(building)
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.equals(Stats(food=1f)))
     }
@@ -57,11 +57,10 @@ class GlobalUniquesTests {
     @Test
     fun statsPerCity() {
         val civInfo = game.addCiv()
-        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val city = game.addCity(civInfo, tile, true)
-        val buildingName = game.createBuilding("[+1 Production] [in this city]").name
+        val city = game.addCity(civInfo,  game.getTile(Vector2.Zero), true)
+        val building = game.createBuilding("[+1 Production] [in this city]")
 
-        city.cityConstructions.addBuilding(buildingName)
+        city.cityConstructions.addBuilding(building)
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.equals(Stats(production=1f)))
     }
@@ -69,14 +68,13 @@ class GlobalUniquesTests {
     @Test
     fun statsPerSpecialist() {
         val civInfo = game.addCiv()
-        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val city = game.addCity(civInfo, tile, true, initialPopulation = 2)
+        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true, initialPopulation = 2)
         val building = game.createBuilding("[+3 Gold] from every specialist [in this city]")
         val specialistName = game.createSpecialist()
         building.specialistSlots.add(specialistName, 2)
         city.population.specialistAllocations[specialistName] = 2
 
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.finalStatList["Specialists"]!!.equals(Stats(gold=6f)))
     }
@@ -84,11 +82,10 @@ class GlobalUniquesTests {
     @Test
     fun statsPerPopulation() {
         val civInfo = game.addCiv()
-        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val city = game.addCity(civInfo, tile, true, initialPopulation = 4)
+        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true, initialPopulation = 4)
         val building = game.createBuilding("[+3 Gold] per [2] population [in this city]")
 
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.gold == 6f)
     }
@@ -96,11 +93,10 @@ class GlobalUniquesTests {
     @Test
     fun statsPerXPopulation() {
         val civInfo = game.addCiv()
-        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val city = game.addCity(civInfo, tile, true, initialPopulation = 2)
+        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true, initialPopulation = 2)
         val building = game.createBuilding("[+3 Gold] <in cities with at least [3] [Population]>")
 
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
 
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.gold == 0f)
@@ -112,14 +108,15 @@ class GlobalUniquesTests {
     @Test
     fun statsFromCitiesOnSpecificTiles() {
         val civInfo = game.addCiv()
-        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
+        val tile = game.setTileTerrain(Vector2.Zero, Constants.desert)
         val city = game.addCity(civInfo, tile, true)
         val building = game.createBuilding("[+3 Gold] in cities on [${Constants.desert}] tiles")
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
 
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.gold == 3f)
         tile.baseTerrain = Constants.grassland
+        tile.setTransients()
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.gold == 0f)
     }
@@ -128,12 +125,11 @@ class GlobalUniquesTests {
     fun statsFromTiles() {
         game.makeHexagonalMap(2)
         val civInfo = game.addCiv()
-        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val city = game.addCity(civInfo, tile, true)
+        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true)
         val building = game.createBuilding("[+4 Gold] from [${Constants.grassland}] tiles [in all cities]")
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
 
-        val tile2 = game.setTileFeatures(Vector2(0f,1f), Constants.grassland)
+        val tile2 = game.setTileTerrain(Vector2(0f,1f), Constants.grassland)
         Assert.assertTrue(tile2.stats.getTileStats(city, civInfo).gold == 4f)
     }
 
@@ -141,16 +137,15 @@ class GlobalUniquesTests {
     fun statsFromTilesWithout() {
         game.makeHexagonalMap(3)
         val civInfo = game.addCiv()
-        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val city = game.addCity(civInfo, tile, true)
+        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true)
         val building = game.createBuilding("[+4 Gold] from [${Constants.grassland}] tiles without [${Constants.forest}] [in this city]")
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
 
-        val tile2 = game.setTileFeatures(Vector2(0f,1f), Constants.grassland)
+        val tile2 = game.setTileTerrain(Vector2(0f,1f), Constants.grassland)
         game.addTileToCity(city, tile2)
         Assert.assertTrue(tile2.stats.getTileStats(city, civInfo).gold == 4f)
 
-        val tile3 = game.setTileFeatures(Vector2(0f, 2f), Constants.grassland, listOf(Constants.forest))
+        val tile3 = game.setTileTerrainAndFeatures(Vector2(0f, 2f), Constants.grassland, Constants.forest)
         game.addTileToCity(city, tile3)
         Assert.assertFalse(tile3.stats.getTileStats(city, civInfo).gold == 4f)
     }
@@ -159,31 +154,30 @@ class GlobalUniquesTests {
     fun statsFromObject() {
         game.makeHexagonalMap(1)
         val civInfo = game.addCiv()
-        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val city = game.addCity(civInfo, tile, true, initialPopulation = 2)
+        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true, initialPopulation = 2)
         val specialist = game.createSpecialist()
         val building = game.createBuilding("[+3 Faith] from every [${specialist}]")
 
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
         city.population.specialistAllocations[specialist] = 2
 
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.finalStatList["Specialists"]!!.faith == 6f)
 
-        city.cityConstructions.removeBuilding(building.name)
+        city.cityConstructions.removeBuilding(building)
         val building2 = game.createBuilding("[+3 Faith] from every [${Constants.grassland}]")
-        city.cityConstructions.addBuilding(building2.name)
+        city.cityConstructions.addBuilding(building2)
 
-        val tile2 = game.setTileFeatures(Vector2(0f,1f), Constants.grassland)
+        val tile2 = game.setTileTerrain(Vector2(0f,1f), Constants.grassland)
         Assert.assertTrue(tile2.stats.getTileStats(city, civInfo).faith == 3f)
 
-        city.cityConstructions.removeBuilding(building2.name)
+        city.cityConstructions.removeBuilding(building2)
 
         val emptyBuilding = game.createBuilding()
 
         val building3 = game.createBuilding("[+3 Faith] from every [${emptyBuilding.name}]")
-        city.cityConstructions.addBuilding(emptyBuilding.name)
-        city.cityConstructions.addBuilding(building3.name)
+        city.cityConstructions.addBuilding(emptyBuilding)
+        city.cityConstructions.addBuilding(building3)
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.faith == 3f)
     }
@@ -193,14 +187,14 @@ class GlobalUniquesTests {
         game.makeHexagonalMap(3)
         val civInfo = game.addCiv("[+30 Science] from each Trade Route")
         civInfo.tech.addTechnology("The Wheel") // Required to form trade routes
-        val tile1 = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val tile2 = game.setTileFeatures(Vector2(0f,2f), Constants.desert)
+        val tile1 = game.getTile(Vector2.Zero)
+        val tile2 = game.getTile(Vector2(0f, 2f))
         tile1.roadStatus = RoadStatus.Road
         tile2.roadStatus = RoadStatus.Road
         @Suppress("UNUSED_VARIABLE")
         val city1 = game.addCity(civInfo, tile1)
         val city2 = game.addCity(civInfo, tile2)
-        val inBetweenTile = game.setTileFeatures(Vector2(0f, 1f), Constants.desert)
+        val inBetweenTile = game.getTile(Vector2(0f, 1f))
         inBetweenTile.roadStatus = RoadStatus.Road
         civInfo.cache.updateCitiesConnectedToCapital()
         city2.cityStats.update()
@@ -214,7 +208,7 @@ class GlobalUniquesTests {
         val civInfo = game.addCiv("[+30 Science] per [2] social policies adopted")
         val policiesToAdopt = listOf("Tradition", "Aristocracy", "Legalism")
         civInfo.policies.freePolicies = 3
-        for (policyName in policiesToAdopt){
+        for (policyName in policiesToAdopt) {
             val policy = game.ruleset.policies[policyName]!!
             civInfo.policies.adopt(policy)
         }
@@ -228,7 +222,7 @@ class GlobalUniquesTests {
         val belief = game.createBelief(BeliefType.Founder, "[+30 Science] for each global city following this religion")
         religion.founderBeliefs.add(belief.name)
         val civ2 = game.addCiv()
-        val tile = game.getTile(Vector2(0f,0f))
+        val tile = game.getTile(Vector2.Zero)
         val cityOfCiv2 = game.addCity(civ2, tile, initialPopulation = 1) // Need someone to be converted
         cityOfCiv2.religion.addPressure(religion.name, 1000)
 
@@ -246,7 +240,7 @@ class GlobalUniquesTests {
         val belief = game.createBelief(BeliefType.Founder, "[+42 Happiness] for each global city following this religion")
         religion.founderBeliefs.add(belief.name)
         val civ2 = game.addCiv()
-        val tile = game.getTile(Vector2(0f,0f))
+        val tile = game.getTile(Vector2.Zero)
         val cityOfCiv2 = game.addCity(civ2, tile, initialPopulation = 1) // Need someone to be converted
         cityOfCiv2.religion.addPressure(religion.name, 1000)
 
@@ -264,7 +258,7 @@ class GlobalUniquesTests {
         val belief = game.createBelief(BeliefType.Founder, "[+30 Science] from every [3] global followers [in all cities]")
         religion.founderBeliefs.add(belief.name)
         val civ2 = game.addCiv()
-        val tile = game.getTile(Vector2(0f,0f))
+        val tile = game.getTile(Vector2.Zero)
         val cityOfCiv2 = game.addCity(civ2, tile, initialPopulation = 9) // Need people to be converted
         cityOfCiv2.religion.addPressure(religion.name, 1000000000) // To completely overwhelm the default atheism in a city
 
@@ -280,10 +274,10 @@ class GlobalUniquesTests {
     @Test
     fun statPercentBonus() {
         val civ = game.addCiv()
-        val tile = game.getTile(Vector2(0f, 0f))
+        val tile = game.getTile(Vector2.Zero)
         val city = game.addCity(civ, tile, true)
         val building = game.createBuilding("[+10 Science]", "[+200]% [Science]")
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
         city.cityStats.update()
 
         Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.science == 30f)
@@ -292,10 +286,10 @@ class GlobalUniquesTests {
     @Test
     fun statPercentBonusCities() {
         val civ = game.addCiv("[+200]% [Science] [in all cities]")
-        val tile = game.getTile(Vector2(0f, 0f))
+        val tile = game.getTile(Vector2.Zero)
         val city = game.addCity(civ, tile, true)
         val building = game.createBuilding("[+10 Science]")
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
         city.cityStats.update()
 
         Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.science == 30f)
@@ -310,17 +304,16 @@ class GlobalUniquesTests {
                 "[+200]% [Faith] from every [${emptyBuilding.name}]",
                 "[+200]% [Faith] from every [Farm]",
             )
-        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val city = game.addCity(civInfo, tile, true)
+        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true)
         val faithBuilding = game.createBuilding()
         faithBuilding.faith = 3f
-        city.cityConstructions.addBuilding(faithBuilding.name)
+        city.cityConstructions.addBuilding(faithBuilding)
 
-        val tile2 = game.setTileFeatures(Vector2(0f,1f), Constants.grassland)
-        tile2.changeImprovement("Farm")
+        val tile2 = game.setTileTerrain(Vector2(0f,1f), Constants.grassland)
+        tile2.setImprovement("Farm")
         Assert.assertTrue(tile2.stats.getTileStats(city, civInfo).faith == 9f)
 
-        city.cityConstructions.addBuilding(emptyBuilding.name)
+        city.cityConstructions.addBuilding(emptyBuilding)
         city.cityStats.update()
 
         Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.faith == 9f)
@@ -335,17 +328,16 @@ class GlobalUniquesTests {
                 "[+200]% Yield from every [${emptyBuilding.name}]",
                 "[+200]% Yield from every [Farm]",
             )
-        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val city = game.addCity(civInfo, tile, true)
+        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true)
         val faithBuilding = game.createBuilding()
         faithBuilding.faith = 3f
-        city.cityConstructions.addBuilding(faithBuilding.name)
+        city.cityConstructions.addBuilding(faithBuilding)
 
-        val tile2 = game.setTileFeatures(Vector2(0f,1f), Constants.grassland)
-        tile2.changeImprovement("Farm")
+        val tile2 = game.setTileTerrain(Vector2(0f,1f), Constants.grassland)
+        tile2.setImprovement("Farm")
         Assert.assertTrue(tile2.stats.getTileStats(city, civInfo).faith == 9f)
 
-        city.cityConstructions.addBuilding(emptyBuilding.name)
+        city.cityConstructions.addBuilding(emptyBuilding)
         city.cityStats.update()
 
         Assert.assertTrue(city.cityStats.finalStatList["Buildings"]!!.faith == 9f)
@@ -355,8 +347,7 @@ class GlobalUniquesTests {
     fun statPercentFromReligionFollowers() {
         game.makeHexagonalMap(1)
         val civInfo = game.addCiv()
-        val tile = game.getTile(Vector2(0f,0f))
-        val city = game.addCity(civInfo, tile, true, 1)
+        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true, 1)
         val religion = game.addReligion(civInfo)
         val belief = game.createBelief(BeliefType.Follower, "[+10]% [Faith] from every follower, up to [42]%")
         religion.followerBeliefs.add(belief.name)
@@ -379,8 +370,7 @@ class GlobalUniquesTests {
         val civInfo = game.addCiv()
         val cityState = game.addCiv(cityStateType = "Maritime")
 
-        val tile = game.getTile(Vector2(0f,0f))
-        val city = game.addCity(civInfo, tile, true)
+        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true)
         val cityStateTile = game.getTile(Vector2(0f, 1f))
         @Suppress("UNUSED_VARIABLE")
         val cityStateCity = game.addCity(cityState, cityStateTile, true)
@@ -391,7 +381,7 @@ class GlobalUniquesTests {
         Assert.assertTrue(city.cityStats.finalStatList[Constants.cityStates]!!.food == 3f)
 
         val building = game.createBuilding("[+100]% [Food] from City-States")
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.finalStatList[Constants.cityStates]!!.food == 6f)
     }
@@ -401,14 +391,14 @@ class GlobalUniquesTests {
         game.makeHexagonalMap(3)
         val civInfo = game.addCiv("[+30 Science] from each Trade Route", "[+100]% [Science] from Trade Routes")
         civInfo.tech.addTechnology("The Wheel") // Required to form trade routes
-        val tile1 = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val tile2 = game.setTileFeatures(Vector2(0f,2f), Constants.desert)
+        val tile1 = game.getTile(Vector2.Zero)
+        val tile2 =  game.getTile(Vector2(0f, 2f))
         tile1.roadStatus = RoadStatus.Road
         tile2.roadStatus = RoadStatus.Road
         @Suppress("UNUSED_VARIABLE")
         val city1 = game.addCity(civInfo, tile1)
         val city2 = game.addCity(civInfo, tile2)
-        val inBetweenTile = game.setTileFeatures(Vector2(0f, 1f), Constants.desert)
+        val inBetweenTile =  game.getTile(Vector2(0f, 1f))
         inBetweenTile.roadStatus = RoadStatus.Road
 
         civInfo.cache.updateCitiesConnectedToCapital()
@@ -427,11 +417,11 @@ class GlobalUniquesTests {
     fun nullifiesStat() {
         game.makeHexagonalMap(1)
         val civInfo = game.addCiv()
-        val tile = game.getTile(Vector2(0f,0f))
+        val tile = game.getTile(Vector2.Zero)
         val city = game.addCity(civInfo, tile, true, 1)
 
         val building = game.createBuilding("Nullifies [Faith] [in this city]", "[+10 Gold, +10 Faith] [in this city]")
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.finalStatList.map { it.value.gold }.sum() >= 10f)
         Assert.assertTrue(city.cityStats.finalStatList.map { it.value.faith }.sum() == 0f)
@@ -441,11 +431,11 @@ class GlobalUniquesTests {
     fun nullifiesGrowth() {
         game.makeHexagonalMap(1)
         val civInfo = game.addCiv()
-        val tile = game.getTile(Vector2(0f,0f))
+        val tile = game.getTile(Vector2.Zero)
         val city = game.addCity(civInfo, tile, true, 1)
 
         val building = game.createBuilding("Nullifies Growth [in this city]", "[+10 Food, +10 Gold] [in this city]")
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.finalStatList.map { it.value.gold }.sum() >= 10f)
         Assert.assertTrue(city.cityStats.finalStatList.map { it.value.food }.sum() == 0f)
@@ -462,12 +452,12 @@ class GlobalUniquesTests {
     @Test
     fun percentProductionBuildings() {
         val civInfo = game.addCiv()
-        val tile = game.getTile(Vector2(0f,0f))
+        val tile = game.getTile(Vector2.Zero)
         val city = game.addCity(civInfo, tile, true, 0)
 
         val buildingToConstruct = game.createBuilding()
         val building = game.createBuilding("[+300]% Production when constructing [${buildingToConstruct.name}] buildings [in all cities]", "[+1 Production]")
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
         city.cityConstructions.addToQueue(buildingToConstruct.name)
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.statPercentBonusTree.totalStats.production == 300f)
@@ -476,12 +466,12 @@ class GlobalUniquesTests {
     @Test
     fun percentProductionUnits() {
         val civInfo = game.addCiv()
-        val tile = game.getTile(Vector2(0f,0f))
+        val tile = game.getTile(Vector2.Zero)
         val city = game.addCity(civInfo, tile, true, 0)
 
         val unitToConstruct = game.createBaseUnit()
         val building = game.createBuilding("[+300]% Production when constructing [${unitToConstruct.name}] units [in all cities]", "[+1 Production]")
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
         city.cityConstructions.addToQueue(unitToConstruct.name)
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.statPercentBonusTree.totalStats.production == 300f)
@@ -490,12 +480,12 @@ class GlobalUniquesTests {
     @Test
     fun percentProductionWonders() {
         val civInfo = game.addCiv()
-        val tile = game.getTile(Vector2(0f,0f))
+        val tile = game.getTile(Vector2.Zero)
         val city = game.addCity(civInfo, tile, true, 0)
 
         val buildingToConstruct = game.createBuilding()
         val building = game.createBuilding("[+300]% Production when constructing [${buildingToConstruct.name}] wonders [in all cities]", "[+1 Production]")
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
         city.cityConstructions.addToQueue(buildingToConstruct.name)
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.statPercentBonusTree.totalStats.production == 0f)
@@ -518,7 +508,7 @@ class GlobalUniquesTests {
         city2.cityStats.update()
         Assert.assertTrue(city2.cityStats.statPercentBonusTree.totalStats.production == 0f)
 
-        city.cityConstructions.addBuilding(buildingToConstruct.name)
+        city.cityConstructions.addBuilding(buildingToConstruct)
         city2.cityStats.update()
         Assert.assertTrue(city2.cityStats.statPercentBonusTree.totalStats.production == 300f)
     }
@@ -532,10 +522,9 @@ class GlobalUniquesTests {
     @Test
     fun growthPercentBonusTest() {
         val civInfo = game.addCiv()
-        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val city = game.addCity(civInfo, tile, true)
+        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true)
         val building = game.createBuilding("[+100]% growth [in all cities]")
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
 
         city.cityStats.update()
         Assert.assertTrue(city.cityStats.finalStatList["[Buildings] ([Growth])"]!!.equals(Stats(food=2f)))
@@ -544,8 +533,7 @@ class GlobalUniquesTests {
     @Test
     fun carryOverFoodTest() {
         val civInfo = game.addCiv("[50]% Food is carried over after population increases [in all cities]")
-        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val city = game.addCity(civInfo, tile, true)
+        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true)
 
         val foodNecessary = city.population.getFoodToNextPopulation()
         city.population.nextTurn(foodNecessary)
@@ -557,8 +545,7 @@ class GlobalUniquesTests {
     @Test
     fun foodConsumptionBySpecialistsTest() {
         val civInfo = game.addCiv("[-50]% Food consumption by specialists [in all cities]")
-        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val city = game.addCity(civInfo, tile, true, initialPopulation = 1)
+        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true, initialPopulation = 1)
 
         val building = game.createBuilding()
         val specialistName = game.createSpecialist()
@@ -577,8 +564,7 @@ class GlobalUniquesTests {
     @Test
     fun unhappinessFromCitiesPercentageTest() {
         val civInfo = game.addCiv("[+100]% unhappiness from the number of cities")
-        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val city = game.addCity(civInfo, tile, true)
+        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true)
 
         city.cityStats.update()
         // Because of some weird design choices, -3.6 is correct, though I would expect it to be -6 instead.
@@ -591,8 +577,7 @@ class GlobalUniquesTests {
     @Test
     fun unhappinessFromPopulationTypePercentageChangeTest() {
         val civInfo = game.addCiv("[-50]% Unhappiness from [Population] [in all cities]")
-        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
-        val city = game.addCity(civInfo, tile, true, initialPopulation = 4)
+        val city = game.addCity(civInfo, game.getTile(Vector2.Zero), true, initialPopulation = 4)
 
         city.cityStats.update()
         // This test suffers from the same problems as `unhappinessFromCitiesPercentageTest()`.
@@ -604,7 +589,7 @@ class GlobalUniquesTests {
         val specialist = game.createSpecialist()
         building.specialistSlots[specialist] = 2
         city.population.specialistAllocations[specialist] = 2
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
 
         city.cityStats.update()
         println(city.cityStats.happinessList)
@@ -620,11 +605,11 @@ class GlobalUniquesTests {
     @Test
     fun statsSpendingGreatPeople() {
         val civInfo = game.addCiv()
-        val tile = game.setTileFeatures(Vector2(0f,0f), Constants.desert)
+        val tile = game.getTile(Vector2.Zero)
         val city = game.addCity(civInfo, tile, true)
         val unit = game.addUnit("Great Engineer", civInfo, tile)
         val building = game.createBuilding("[+250 Gold] whenever a Great Person is expended")
-        city.cityConstructions.addBuilding(building.name)
+        city.cityConstructions.addBuilding(building)
 
         civInfo.addGold(-civInfo.gold) // reset gold just to be sure
 
